@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -53,7 +55,6 @@
 #include "z_zone.h"
 
 #include "m_swap.h"
-#include "i_system.h"
 #include "i_sound.h"
 #include "m_argv.h"
 #include "m_misc.h"
@@ -72,14 +73,15 @@
 //  the size of the 16bit, 2 hardware channel (stereo)
 //  mixing buffer, and the samplerate of the raw data.
 
-#define PIPE_CHECK(fh) if (broken_pipe) { fclose(fh); fh = NULL; broken_pipe = 0; }
-
 // Variables used by Boom from Allegro
 // created here to avoid changes to core Boom files
 int snd_card = 1;
 int mus_card = 1;
 int detect_voices = 0; // God knows
 
+static boolean sound_inited = false;
+static boolean first_sound_init = true;
+
 // Needed for calling the actual sound output.
 static int SAMPLECOUNT=   512;
 #define MAX_CHANNELS    32
@@ -142,7 +144,7 @@ static void stopchan(int i)
 //  (eight, usually) of internal channels.
 // Returns a handle.
 //
-int addsfx(int sfxid, int channel)
+static int addsfx(int sfxid, int channel)
 {
     int   oldest = gametic;
     int   oldestnum = 0;
@@ -241,7 +243,7 @@ void I_UpdateSoundParams(int handle, int volume, int seperation, int pitch)
 // version.
 // See soundserver initdata().
 //
-void I_SetChannels()
+void I_SetChannels(void)
 {
   // Init internal lookups (raw data, mixing buffer, channels).
   // This function sets up internal lookups used during
@@ -276,17 +278,6 @@ void I_SetChannels()
     }
 }
 
-
-void I_SetSfxVolume(int volume)
-{
-  // Identical to DOS.
-  // Basically, this should propagate
-  //  the menu/config file setting
-  //  to the state variable used in
-  //  the mixing.
-  snd_SfxVolume = volume;
-}
-
 //
 // Retrieve the raw data lump index
 //  for a given SFX name.
@@ -371,7 +362,7 @@ boolean I_SoundIsPlaying(int handle)
 // This function currently supports only 16bit.
 //
 
-void I_UpdateSound(void *unused, Uint8 *stream, int len)
+static void I_UpdateSound(void *unused, Uint8 *stream, int len)
 {
   // Mix current sound data.
   // Data, from raw sound, for right and left.
@@ -473,19 +464,21 @@ void I_UpdateSound(void *unused, Uint8 *stream, int len)
 
 void I_ShutdownSound(void)
 {
-  lprintf(LO_INFO, "I_ShutdownSound: ");
+  if (sound_inited) {
+    lprintf(LO_INFO, "I_ShutdownSound: ");
 #ifdef HAVE_MIXER
-  Mix_CloseAudio();
+    Mix_CloseAudio();
 #else
-  SDL_CloseAudio();
+    SDL_CloseAudio();
 #endif
-  lprintf(LO_INFO, "\n");
+    lprintf(LO_INFO, "\n");
+    sound_inited = false;
+  }
 }
 
 //static SDL_AudioSpec audio;
 
-void
-I_InitSound()
+void I_InitSound(void)
 {
 #ifdef HAVE_MIXER
   int audio_rate;
@@ -493,6 +486,9 @@ I_InitSound()
   int audio_channels;
   int audio_buffers;
 
+  if (sound_inited)
+      I_ShutdownSound();
+
   // Secure and configure sound device first.
   lprintf(LO_INFO,"I_InitSound: ");
 
@@ -504,12 +500,14 @@ I_InitSound()
   audio_format = AUDIO_S16LSB;
 #endif
   audio_channels = 2;
+  SAMPLECOUNT = 512;
   audio_buffers = SAMPLECOUNT*snd_samplerate/11025;
 
   if (Mix_OpenAudio(audio_rate, audio_format, audio_channels, audio_buffers) < 0) {
     lprintf(LO_INFO,"couldn't open audio with desired format\n");
     return;
   }
+  sound_inited = true;
   SAMPLECOUNT = audio_buffers;
   Mix_SetPostMix(I_UpdateSound, NULL);
   lprintf(LO_INFO," configured audio device with %d samples/slice\n", SAMPLECOUNT);
@@ -537,7 +535,10 @@ I_InitSound()
   lprintf(LO_INFO," configured audio device with %d samples/slice\n", SAMPLECOUNT);
 #endif
 
-  atexit(I_ShutdownSound);
+  if (first_sound_init) {
+    atexit(I_ShutdownSound);
+    first_sound_init = false;
+  }
 
   if (!nomusicparm)
     I_InitMusic();
@@ -556,13 +557,15 @@ I_InitSound()
 // MUSIC API.
 //
 
+#ifndef HAVE_OWN_MUSIC
+
 #ifdef HAVE_MIXER
 #include "SDL_mixer.h"
 #include "mmus2mid.h"
 
 static Mix_Music *music[2] = { NULL, NULL };
 
-char* music_tmp; /* cph - name of music temporary file */
+char* music_tmp = NULL; /* cph - name of music temporary file */
 
 #endif
 
@@ -573,6 +576,7 @@ void I_ShutdownMusic(void)
     unlink(music_tmp);
     lprintf(LO_DEBUG, "I_ShutdownMusic: removing %s\n", music_tmp);
     free(music_tmp);
+	music_tmp = NULL;
   }
 #endif
 }
@@ -580,21 +584,23 @@ void I_ShutdownMusic(void)
 void I_InitMusic(void)
 {
 #ifdef HAVE_MIXER
+  if (!music_tmp) {
 #ifndef _WIN32
-  music_tmp = strdup("/tmp/prboom-music-XXXXXX");
-  {
-    int fd = mkstemp(music_tmp);
-    if (fd<0) {
-      lprintf(LO_ERROR, "I_InitMusic: failed to create music temp file %s", music_tmp);
-      free(music_tmp); return;
-    } else
-      close(fd);
-  }
+    music_tmp = strdup("/tmp/prboom-music-XXXXXX");
+    {
+      int fd = mkstemp(music_tmp);
+      if (fd<0) {
+        lprintf(LO_ERROR, "I_InitMusic: failed to create music temp file %s", music_tmp);
+        free(music_tmp); return;
+      } else 
+        close(fd);
+    }
 #else /* !_WIN32 */
-  music_tmp = strdup("doom.tmp");
+    music_tmp = strdup("doom.tmp");
 #endif
+    atexit(I_ShutdownMusic);
+  }
 #endif
-  atexit(I_ShutdownMusic);
 }
 
 void I_PlaySong(int handle, int looping)
@@ -726,3 +732,5 @@ void I_SetMusicVolume(int volume)
 #endif
 }
 
+#endif /* HAVE_OWN_MUSIC */
+
