@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -59,20 +61,16 @@
 #ifdef _MSC_VER
 #include <io.h>
 #endif
-#ifdef DREAMCAST
-#include <kos/fs.h>
-#else
 #include <fcntl.h>
 #include <sys/stat.h>
-#endif
 #include <errno.h>
 
-#include "i_system.h"
 #include "m_argv.h"
 #include "lprintf.h"
 #include "doomtype.h"
 #include "doomdef.h"
 #include "lprintf.h"
+#include "i_system.h"
 
 #ifdef __GNUG__
 #pragma implementation "i_system.h"
@@ -133,7 +131,11 @@ const char* I_SigString(char* buf, size_t sz, int signum)
     strcpy(buf,sys_siglist[signum]);
   else
 #endif
+#ifdef HAVE_SNPRINTF
+    snprintf(buf,sz,"signal %d",signum);
+#else
     sprintf(buf,"signal %d",signum);
+#endif
   return buf;
 }
 
@@ -165,12 +167,10 @@ void I_Read(int fd, void* vbuf, size_t sz)
 
 int I_Filelength(int handle)
 {
-#ifndef DREAMCAST
   struct stat   fileinfo;
   if (fstat(handle,&fileinfo) == -1)
     I_Error("I_Filelength: %s",strerror(errno));
   return fileinfo.st_size;
-#endif  
 }
 
 #ifndef PRBOOM_SERVER
@@ -178,7 +178,7 @@ int I_Filelength(int handle)
 // Return the path where the executable lies -- Lee Killough
 // proff_fs 2002-07-04 - moved to i_system
 #ifdef _WIN32
-char *I_DoomExeDir(void)
+const char *I_DoomExeDir(void)
 {
   static const char current_dir_dummy[] = {"."}; // proff - rem extra slash 8/21/03
   static char *base;
@@ -201,15 +201,24 @@ char *I_DoomExeDir(void)
     }
   return base;
 }
+
+#elif defined(AMIGA)
+
+const char *I_DoomExeDir(void)
+{
+  return "PROGDIR:";
+}
+
+#elif defined(MACOSX)
+
+/* Defined elsewhere */
+
 #else
 // cph - V.Aguilar (5/30/99) suggested return ~/.lxdoom/, creating
 //  if non-existant
 static const char prboom_dir[] = {"/.prboom"}; // Mead rem extra slash 8/21/03
 
-#ifdef MACOSX
-/* Defined elsewhere */
-#else
-char *I_DoomExeDir(void)
+const char *I_DoomExeDir(void)
 {
   static char *base;
   if (!base)        // cache multiple requests
@@ -227,7 +236,6 @@ char *I_DoomExeDir(void)
   return base;
 }
 #endif
-#endif
 
 /*
  * HasTrailingSlash
@@ -235,9 +243,13 @@ char *I_DoomExeDir(void)
  * cphipps - simple test for trailing slash on dir names
  */
 
-static boolean HasTrailingSlash(const char* dn)
+boolean HasTrailingSlash(const char* dn)
 {
-  return (dn[strlen(dn)-1] == '/');
+  return ( (dn[strlen(dn)-1] == '/')
+#if defined(AMIGA)
+        || (dn[strlen(dn)-1] == ':')
+#endif
+          );
 }
 
 /*
@@ -266,19 +278,22 @@ char* I_FindFile(const char* wfname, const char* ext)
   /* Precalculate a length we will need in the loop */
   size_t  pl = strlen(wfname) + strlen(ext) + 4;
 
-  for (i=0; i<8; i++) {
+  for (i=0; i<10; i++) {
     char  * p;
     const char  * d = NULL;
     const char  * s = NULL;
     /* Each entry in the switch sets d to the directory to look in,
      * and optionally s to a subdirectory of d */
     switch(i) {
-    case 1:
-      if (!(d = getenv("DOOMWADDIR"))) continue;
     case 0:
+      if (!(d = getenv("DOOMWADDIR"))) continue;
+    case 1:
       break;
     case 2:
-      d = DOOMWADDIR;
+      s = DOOMWADDIR;
+      break;
+    case 3:
+      s = "doom";
       break;
     case 4:
       d = "/usr/share/games/doom";
@@ -287,16 +302,20 @@ char* I_FindFile(const char* wfname, const char* ext)
       d = "/usr/local/share/games/doom";
       break;
     case 6:
+      d = "/usr/share/doom";
+      break;
+    case 7:
+      d = "/usr/local/share/doom";
+      break;
+    case 8:
       d = I_DoomExeDir();
       break;
-    case 3:
-      s = "doom";
-    case 7:
+    case 9:
       if (!(d = getenv("HOME"))) continue;
       break;
 #ifdef SIMPLECHECKS
     default:
-      I_Error("FindWADFile: Internal failure");
+      I_Error("I_FindFile: Internal failure");
 #endif
     }
 
