@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2006 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -42,7 +44,6 @@
 
 #include "SDL.h"
 
-#include "i_system.h"
 #include "m_argv.h"
 #include "doomstat.h"
 #include "doomdef.h"
@@ -57,16 +58,12 @@
 #include "s_sound.h"
 #include "sounds.h"
 #include "w_wad.h"
+#include "st_stuff.h"
 #include "lprintf.h"
 
-#ifdef GL_DOOM
-#include "gl_struct.h"
-
 int gl_colorbuffer_bits=16;
 int gl_depthbuffer_bits=16;
 
-#endif
-
 extern void M_QuitDOOM(int choice);
 #ifdef DISABLE_DOUBLEBUFFER
 int use_doublebuffer = 0;
@@ -76,8 +73,6 @@ int use_doublebuffer = 1; // Included not to break m_misc, but not relevant to S
 int use_fullscreen;
 static SDL_Surface *screen;
 
-unsigned char* out_buffer = NULL;
-
 ////////////////////////////////////////////////////////////////////////////
 // Input code
 int             leds_always_off = 0; // Expected by m_misc, not relevant
@@ -85,6 +80,7 @@ int             leds_always_off = 0; // Expected by m_misc, not relevant
 // Mouse handling
 extern int     usemouse;        // config file var
 static boolean grabMouse;       // internal var
+static int mouse_currently_grabbed;
 
 /////////////////////////////////////////////////////////////////////////////////
 // Keyboard handling
@@ -159,17 +155,6 @@ static int I_TranslateKey(SDL_keysym* key)
 
 }
 
-// Null keyboard translation to satisfy m_misc.c
-int I_DoomCode2ScanCode(int c)
-{
-  return c;
-}
-
-int I_ScanCode2DoomCode(int c)
-{
-  return c;
-}
-
 /////////////////////////////////////////////////////////////////////////////////
 // Main input code
 
@@ -203,7 +188,7 @@ static void I_GetEvent(SDL_Event *Event)
 
   case SDL_MOUSEBUTTONDOWN:
   case SDL_MOUSEBUTTONUP:
-  if (usemouse)
+  if (mouse_currently_grabbed)
   {
     event.type = ev_mouse;
     event.data1 = I_SDLtoDoomMouseState(SDL_GetMouseState(NULL, NULL));
@@ -213,7 +198,7 @@ static void I_GetEvent(SDL_Event *Event)
   break;
 
   case SDL_MOUSEMOTION:
-  if (usemouse) {
+  if (mouse_currently_grabbed) {
     event.type = ev_mouse;
     event.data1 = I_SDLtoDoomMouseState(Event->motion.state);
     event.data2 = Event->motion.xrel << 5;
@@ -236,7 +221,6 @@ static void I_GetEvent(SDL_Event *Event)
 //
 // I_StartTic
 //
-static int mouse_currently_grabbed;
 
 void I_StartTic (void)
 {
@@ -272,7 +256,8 @@ static void I_InitInputs(void)
   // check if the user wants to grab the mouse
   grabMouse = M_CheckParm("-nomouse") ? false : usemouse ? true : false;
   // e6y: fix for turn-snapping bug on fullscreen in software mode
-  SDL_WarpMouse((unsigned short)(desired_screenwidth/2), (unsigned short)(desired_screenheight/2));
+  if (!M_CheckParm("-nomouse"))
+    SDL_WarpMouse((unsigned short)(SCREENWIDTH/2), (unsigned short)(SCREENHEIGHT/2));
 
   I_InitJoystick();
 }
@@ -310,16 +295,17 @@ static void I_UploadNewPalette(int pal)
   static int cachedgamma;
   static size_t num_pals;
 
-#ifdef GL_DOOM
-  return;
-#endif
+  if (V_GetMode() == VID_MODEGL)
+    return;
+
   if ((colours == NULL) || (cachedgamma != usegamma)) {
-    int            lump = W_GetNumForName("PLAYPAL");
-    const byte *palette = W_CacheLumpNum(lump);
-    register const byte *const gtable = gammatable[cachedgamma = usegamma];
+    int pplump = W_GetNumForName("PLAYPAL");
+    int gtlump = (W_CheckNumForName)("GAMMATBL",ns_prboom);
+    register const byte * palette = W_CacheLumpNum(pplump);
+    register const byte * const gtable = (const byte *)W_CacheLumpNum(gtlump) + 256*(cachedgamma = usegamma);
     register int i;
 
-    num_pals = W_LumpLength(lump) / (3*256);
+    num_pals = W_LumpLength(pplump) / (3*256);
     num_pals *= 256;
 
     if (!colours) {
@@ -335,7 +321,8 @@ static void I_UploadNewPalette(int pal)
       palette += 3;
     }
 
-    W_UnlockLumpNum(lump);
+    W_UnlockLumpNum(pplump);
+    W_UnlockLumpNum(gtlump);
     num_pals/=256;
   }
 
@@ -346,7 +333,8 @@ static void I_UploadNewPalette(int pal)
 #endif
 
   // store the colors to the current display
-  SDL_SetColors(SDL_GetVideoSurface(), colours+256*pal, 0, 256);
+  // SDL_SetColors(SDL_GetVideoSurface(), colours+256*pal, 0, 256);
+  SDL_SetPalette(SDL_GetVideoSurface(),SDL_PHYSPAL,colours+256*pal, 0, 256);
 }
 
 //////////////////////////////////////////////////////////////////////////////
@@ -379,26 +367,30 @@ void I_FinishUpdate (void)
   }
 #endif
 
-#ifndef GL_DOOM
+#ifdef GL_DOOM
+  if (V_GetMode() == VID_MODEGL) {
+    // proff 04/05/2000: swap OpenGL buffers
+    gld_Finish();
+    return;
+  }
+#endif
   if (SDL_MUSTLOCK(screen)) {
       int h;
-      int w;
-      char *src;
-      char *dest;
+      byte *src;
+      byte *dest;
 
       if (SDL_LockSurface(screen) < 0) {
-	lprintf(LO_INFO,"I_FinishUpdate: %s\n", SDL_GetError());
+        lprintf(LO_INFO,"I_FinishUpdate: %s\n", SDL_GetError());
         return;
       }
-      dest=(char *)screen->pixels;
-      src=screens[0];
-      w=screen->w;
+      dest=screen->pixels;
+      src=screens[0].data;
       h=screen->h;
       for (; h>0; h--)
       {
-        memcpy(dest,src,w);
+        memcpy(dest,src,SCREENWIDTH);
         dest+=screen->pitch;
-        src+=SCREENWIDTH;
+        src+=screens[0].pitch;
       }
       SDL_UnlockSurface(screen);
   }
@@ -407,20 +399,29 @@ void I_FinishUpdate (void)
   if (newpal != NO_PALETTE_CHANGE) {
     I_UploadNewPalette(newpal);
     newpal = NO_PALETTE_CHANGE;
-  } else
-    SDL_Flip(screen);
-#else
-  // proff 04/05/2000: swap OpenGL buffers
-  gld_Finish();
-#endif
+  }
+  SDL_Flip(screen);
 }
 
 //
 // I_ReadScreen
 //
-void I_ReadScreen (byte* scr)
+#ifndef min
+#define min(a,b) ((a)<(b)?(a):(b))
+#endif
+void I_ReadScreen (screeninfo_t *dest)
 {
-  memcpy(scr, screens[0], SCREENWIDTH*SCREENHEIGHT);
+  int h;
+  byte *srcofs = screens[0].data;
+  byte *dstofs = dest->data;
+  int width, height;
+  width = min(screens[0].width, dest->width);
+  height = min(screens[0].height, dest->height);
+  for (h=height; h>0; h--) {
+    memcpy(dstofs, srcofs, width);
+    srcofs += screens[0].pitch;
+    dstofs += dest->pitch;
+  }
 }
 
 //
@@ -433,7 +434,7 @@ void I_SetPalette (int pal)
 
 // I_PreInitGraphics
 
-void I_ShutdownSDL(void)
+static void I_ShutdownSDL(void)
 {
   SDL_Quit();
   return;
@@ -442,21 +443,125 @@ void I_ShutdownSDL(void)
 void I_PreInitGraphics(void)
 {
   // Initialize SDL
-  if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
+  unsigned int flags = 0;
+  if (!(M_CheckParm("-nodraw") && M_CheckParm("-nosound")))
+    flags = SDL_INIT_VIDEO;
+#ifdef _DEBUG
+  flags |= SDL_INIT_NOPARACHUTE;
+#endif
+  if ( SDL_Init(flags) < 0 ) {
     I_Error("Could not initialize SDL [%s]", SDL_GetError());
   }
 
   atexit(I_ShutdownSDL);
 }
 
-// CPhipps -
-// I_SetRes
-// Sets the screen resolution, possibly using the supplied guide
-
-void I_SetRes(unsigned int width, unsigned int height)
+// e6y
+// GLBoom use this function for trying to set the closest supported resolution if the requested mode can't be set correctly.
+// For example glboom.exe -geom 1025x768 -nowindow will set 1024x768.
+// It should be used only for fullscreen modes.
+static void I_ClosestResolution (int *width, int *height, int flags)
 {
+  SDL_Rect **modes;
+  int twidth, theight;
+  int cwidth = 0, cheight = 0;
+//  int iteration;
+  int i;
+  unsigned int closest = UINT_MAX;
+  unsigned int dist;
+
+  modes = SDL_ListModes(NULL, flags);
+
+  //for (iteration = 0; iteration < 2; iteration++)
+  {
+    for(i=0; modes[i]; ++i)
+    {
+      twidth = modes[i]->w;
+      theight = modes[i]->h;
+
+      if (twidth > MAX_SCREENWIDTH || theight> MAX_SCREENHEIGHT)
+        continue;
+      
+      if (twidth == *width && theight == *height)
+        return;
+
+      //if (iteration == 0 && (twidth < *width || theight < *height))
+      //  continue;
+
+      dist = (twidth - *width) * (twidth - *width) + 
+             (theight - *height) * (theight - *height);
+
+      if (dist < closest)
+      {
+        closest = dist;
+        cwidth = twidth;
+        cheight = theight;
+      }
+    }
+    if (closest != 4294967295u)
+    {
+      *width = cwidth;
+      *height = cheight;
+      return;
+    }
+  }
+}  
+
+// CPhipps -
+// I_CalculateRes
+// Calculates the screen resolution, possibly using the supplied guide
+void I_CalculateRes(unsigned int width, unsigned int height)
+{
+  // e6y: how about 1680x1050?
+  /*
   SCREENWIDTH = (width+3) & ~3;
   SCREENHEIGHT = (height+3) & ~3;
+  */
+
+// e6y
+// GLBoom will try to set the closest supported resolution 
+// if the requested mode can't be set correctly.
+// For example glboom.exe -geom 1025x768 -nowindow will set 1024x768.
+// It affects only fullscreen modes.
+  if (V_GetMode() == VID_MODEGL) {
+    if ( use_fullscreen && !M_CheckParm("-window") )
+    {
+      I_ClosestResolution(&width, &height, SDL_OPENGL|SDL_FULLSCREEN);
+    }
+    SCREENWIDTH = width;
+    SCREENHEIGHT = height;
+    SCREENPITCH = SCREENWIDTH;
+  } else {
+    SCREENWIDTH = (width+15) & ~15;
+    SCREENHEIGHT = height;
+    if (!(SCREENWIDTH % 1024)) {
+      SCREENPITCH = SCREENWIDTH+32;
+    } else {
+      SCREENPITCH = SCREENWIDTH;
+    }
+  }
+}
+
+// CPhipps -
+// I_SetRes
+// Sets the screen resolution
+void I_SetRes(void)
+{
+  int i;
+
+  I_CalculateRes(SCREENWIDTH, SCREENHEIGHT);
+
+  // set first three to standard values
+  for (i=0; i<3; i++) {
+    screens[i].width = SCREENWIDTH;
+    screens[i].height = SCREENHEIGHT;
+    screens[i].pitch = SCREENPITCH;
+  }
+
+  // statusbar
+  screens[4].width = SCREENWIDTH;
+  screens[4].height = (ST_SCALED_HEIGHT+1);
+  screens[4].pitch = SCREENPITCH;
 
   lprintf(LO_INFO,"I_SetRes: Using resolution %dx%d\n", SCREENWIDTH, SCREENHEIGHT);
 }
@@ -473,8 +578,6 @@ void I_InitGraphics(void)
     atexit(I_ShutdownGraphics);
     lprintf(LO_INFO, "I_InitGraphics: %dx%d\n", SCREENWIDTH, SCREENHEIGHT);
 
-    out_buffer=screens[0];
-
     /* Set the video mode */
     I_UpdateVideoMode();
 
@@ -491,49 +594,73 @@ void I_InitGraphics(void)
 
 void I_UpdateVideoMode(void)
 {
-  unsigned int w, h;
   int init_flags;
+  int i;
+  video_mode_t mode;
 
   lprintf(LO_INFO, "I_UpdateVideoMode: %dx%d (%s)\n", SCREENWIDTH, SCREENHEIGHT, use_fullscreen ? "fullscreen" : "nofullscreen");
 
-  w = SCREENWIDTH;
-  h = SCREENHEIGHT;
+  mode = default_videomode;
+  if ((i=M_CheckParm("-vidmode")) && i<myargc-1) {
+    /*if (!stricmp(myargv[i+1],"16")) {
+      mode = VID_MODE16;
+    } else if (!stricmp(myargv[i+1],"32")) {
+      mode = VID_MODE32;
+    } else*/ if (!stricmp(myargv[i+1],"gl")) {
+      mode = VID_MODEGL;
+    } else {
+      mode = VID_MODE8;
+    }
+  }
+  V_InitMode(mode);
+
+  V_FreeScreens();
+
+  I_SetRes();
 
   // Initialize SDL with this graphics mode
-#ifdef GL_DOOM
-  init_flags = SDL_OPENGL;
-#else
-  if (use_doublebuffer)
-    init_flags = SDL_SWSURFACE | SDL_DOUBLEBUF;
-  else
-    init_flags = SDL_SWSURFACE;
+  if (V_GetMode() == VID_MODEGL) {
+    init_flags = SDL_OPENGL;
+  } else {
+    if (use_doublebuffer)
+      init_flags = SDL_DOUBLEBUF;
+    else
+      init_flags = SDL_SWSURFACE;
 #ifndef _DEBUG
-  init_flags |= SDL_HWPALETTE;
-#endif
+    init_flags |= SDL_HWPALETTE;
 #endif
+  }
+
   if ( use_fullscreen )
     init_flags |= SDL_FULLSCREEN;
 
-#ifdef GL_DOOM
-  SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0 );
-  SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
-  SDL_GL_SetAttribute( SDL_GL_BUFFER_SIZE, gl_colorbuffer_bits );
-  SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, gl_depthbuffer_bits );
-  screen = SDL_SetVideoMode(w, h, gl_colorbuffer_bits, init_flags);
-#else
-  screen = SDL_SetVideoMode(w, h, 8, init_flags);
-#endif
+  // e6y
+  // New command-line options for setting a window (-window) 
+  // or fullscreen (-nowindow) mode temporarily which is not saved in cfg.
+  // It works like "-geom" switch
+  if (M_CheckParm("-window")) init_flags &= ~SDL_FULLSCREEN;
+  if (M_CheckParm("-nowindow")) init_flags |= SDL_FULLSCREEN;
+
+  if (V_GetMode() == VID_MODEGL) {
+    SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_ALPHA_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_STENCIL_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_ACCUM_RED_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_ACCUM_GREEN_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_ACCUM_BLUE_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, 0 );
+    SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
+    SDL_GL_SetAttribute( SDL_GL_BUFFER_SIZE, gl_colorbuffer_bits );
+    SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, gl_depthbuffer_bits );
+    screen = SDL_SetVideoMode(SCREENWIDTH, SCREENHEIGHT, gl_colorbuffer_bits, init_flags);
+  } else {
+    screen = SDL_SetVideoMode(SCREENWIDTH, SCREENHEIGHT, 8, init_flags);
+  }
 
   if(screen == NULL) {
-    I_Error("Couldn't set %dx%d video mode [%s]", w, h, SDL_GetError());
+    I_Error("Couldn't set %dx%d video mode [%s]", SCREENWIDTH, SCREENHEIGHT, SDL_GetError());
   }
 
   lprintf(LO_INFO, "I_UpdateVideoMode: 0x%x, %s, %s\n", init_flags, screen->pixels ? "SDL buffer" : "own buffer", SDL_MUSTLOCK(screen) ? "lock-and-copy": "direct access");
@@ -543,51 +670,49 @@ void I_UpdateVideoMode(void)
   // Get the info needed to render to the display
   if (!SDL_MUSTLOCK(screen))
   {
-    if (out_buffer)
-      free(out_buffer);
-    out_buffer=NULL;
-    screens[0] = (unsigned char *) (screen->pixels);
+    screens[0].not_on_heap = true;
+    screens[0].data = (unsigned char *) (screen->pixels);
+    screens[0].pitch = screen->pitch;
   }
   else
   {
-    if (!out_buffer)
-      free(out_buffer);
-    out_buffer = calloc(SCREENWIDTH*SCREENHEIGHT, 1);
-    screens[0] = out_buffer;
+    screens[0].not_on_heap = false;
   }
 
+  V_AllocScreens();
+
   // Hide pointer while over this window
   SDL_ShowCursor(0);
 
-  R_InitBuffer(w,h);
+  R_InitBuffer(SCREENWIDTH, SCREENHEIGHT);
 
-#ifdef GL_DOOM
-  {
-  int temp;
+  if (V_GetMode() == VID_MODEGL) {
+    int temp;
     lprintf(LO_INFO,"SDL OpenGL PixelFormat:\n");
-  SDL_GL_GetAttribute( SDL_GL_RED_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_RED_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_GREEN_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_GREEN_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_BLUE_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_BLUE_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_STENCIL_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_STENCIL_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_ACCUM_RED_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_ACCUM_RED_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_ACCUM_GREEN_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_ACCUM_GREEN_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_ACCUM_BLUE_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_ACCUM_BLUE_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_ACCUM_ALPHA_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &temp );
-  lprintf(LO_INFO,"    SDL_GL_DOUBLEBUFFER: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_BUFFER_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_BUFFER_SIZE: %i\n",temp);
-  SDL_GL_GetAttribute( SDL_GL_DEPTH_SIZE, &temp );
-  lprintf(LO_INFO,"    SDL_GL_DEPTH_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_RED_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_RED_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_GREEN_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_GREEN_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_BLUE_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_BLUE_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_STENCIL_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_STENCIL_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_ACCUM_RED_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_ACCUM_RED_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_ACCUM_GREEN_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_ACCUM_GREEN_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_ACCUM_BLUE_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_ACCUM_BLUE_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_ACCUM_ALPHA_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_ACCUM_ALPHA_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_DOUBLEBUFFER, &temp );
+    lprintf(LO_INFO,"    SDL_GL_DOUBLEBUFFER: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_BUFFER_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_BUFFER_SIZE: %i\n",temp);
+    SDL_GL_GetAttribute( SDL_GL_DEPTH_SIZE, &temp );
+    lprintf(LO_INFO,"    SDL_GL_DEPTH_SIZE: %i\n",temp);
+#ifdef GL_DOOM
     gld_Init(SCREENWIDTH, SCREENHEIGHT);
-  }
 #endif
+  }
 }
