@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -69,6 +71,7 @@ int mapcolor_flat;    // line with no floor/ceiling changes
 int mapcolor_sprt;    // general sprite color
 int mapcolor_item;    // item sprite color
 int mapcolor_frnd;    // friendly sprite color
+int mapcolor_enemy;   // enemy sprite color
 int mapcolor_hair;    // crosshair color
 int mapcolor_sngl;    // single player arrow color
 int mapcolor_plyr[4] = { 112, 88, 64, 176 }; // colors for player arrows in multiplayer
@@ -101,26 +104,11 @@ int map_secret_after;
 #define CXMTOF(x)  (f_x + MTOF((x)-m_x))
 #define CYMTOF(y)  (f_y + (f_h - MTOF((y)-m_y)))
 
-typedef struct
-{
-    int x, y;
-} fpoint_t;
-
-typedef struct
-{
-    fpoint_t a, b;
-} fline_t;
-
 typedef struct
 {
     mpoint_t a, b;
 } mline_t;
 
-typedef struct
-{
-    fixed_t slp, islp;
-} islope_t;
-
 //
 // The vector graphics for the automap.
 //  A line drawing of the player pointing right,
@@ -259,34 +247,6 @@ int markpointnum_max = 0;       // killough 2/22/98
 
 static boolean stopped = true;
 
-//
-// AM_getIslope()
-//
-// Calculates the slope and slope according to the x-axis of a line
-// segment in map coordinates (with the upright y-axis n' all) so
-// that it can be used with the brain-dead drawing stuff.
-//
-// Passed the line slope is desired for and an islope_t structure for return
-// Returns nothing
-//
-void AM_getIslope
-( mline_t*  ml,
-  islope_t* is )
-{
-  int dx, dy;
-
-  dy = ml->a.y - ml->b.y;
-  dx = ml->b.x - ml->a.x;
-  if (!dy)
-    is->islp = (dx<0?-INT_MAX:INT_MAX);
-  else
-    is->islp = FixedDiv(dx, dy);
-  if (!dx)
-    is->slp = (dy<0?-INT_MAX:INT_MAX);
-  else
-    is->slp = FixedDiv(dy, dx);
-}
-
 //
 // AM_activateNewScale()
 //
@@ -294,7 +254,7 @@ void AM_getIslope
 //
 // Passed nothing, returns nothing
 //
-void AM_activateNewScale(void)
+static void AM_activateNewScale(void)
 {
   m_x += m_w/2;
   m_y += m_h/2;
@@ -314,7 +274,7 @@ void AM_activateNewScale(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_saveScaleAndLoc(void)
+static void AM_saveScaleAndLoc(void)
 {
   old_m_x = m_x;
   old_m_y = m_y;
@@ -330,7 +290,7 @@ void AM_saveScaleAndLoc(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_restoreScaleAndLoc(void)
+static void AM_restoreScaleAndLoc(void)
 {
   m_w = old_m_w;
   m_h = old_m_h;
@@ -360,7 +320,7 @@ void AM_restoreScaleAndLoc(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_addMark(void)
+static void AM_addMark(void)
 {
   // killough 2/22/98:
   // remove limit on automap marks
@@ -383,7 +343,7 @@ void AM_addMark(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_findMinMaxBoundaries(void)
+static void AM_findMinMaxBoundaries(void)
 {
   int i;
   fixed_t a;
@@ -425,7 +385,7 @@ void AM_findMinMaxBoundaries(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_changeWindowLoc(void)
+static void AM_changeWindowLoc(void)
 {
   if (m_paninc.x || m_paninc.y)
   {
@@ -460,10 +420,10 @@ void AM_changeWindowLoc(void)
 // Status bar is notified that the automap has been entered
 // Passed nothing, returns nothing
 //
-void AM_initVariables(void)
+static void AM_initVariables(void)
 {
   int pnum;
-  static event_t st_notify = { ev_keyup, AM_MSGENTERED };
+  static event_t st_notify = { ev_keyup, AM_MSGENTERED, 0, 0 };
 
   automapmode |= am_active;
 
@@ -500,7 +460,7 @@ void AM_initVariables(void)
 //
 // AM_loadPics()
 //
-void AM_loadPics(void)
+static void AM_loadPics(void)
 {
   // cph - mark numbers no longer needed cached
 }
@@ -508,7 +468,7 @@ void AM_loadPics(void)
 //
 // AM_unloadPics()
 //
-void AM_unloadPics(void)
+static void AM_unloadPics(void)
 {
   // cph - mark numbers no longer needed cached
 }
@@ -536,7 +496,7 @@ void AM_clearMarks(void)
 // Affects automap's global variables
 //
 // CPhipps - get status bar height from status bar code
-void AM_LevelInit(void)
+static void AM_LevelInit(void)
 {
   leveljuststarted = 0;
 
@@ -560,7 +520,7 @@ void AM_LevelInit(void)
 //
 void AM_Stop (void)
 {
-  static event_t st_notify = { 0, ev_keyup, AM_MSGEXITED };
+  static event_t st_notify = { 0, ev_keyup, AM_MSGEXITED, 0 };
 
   AM_unloadPics();
   automapmode &= ~am_active;
@@ -578,7 +538,7 @@ void AM_Stop (void)
 //
 // Passed nothing, returns nothing
 //
-void AM_Start()
+void AM_Start(void)
 {
   static int lastlevel = -1, lastepisode = -1;
 
@@ -602,7 +562,7 @@ void AM_Start()
 //
 // Passed nothing, returns nothing
 //
-void AM_minOutWindowScale()
+static void AM_minOutWindowScale(void)
 {
   scale_mtof = min_scale_mtof;
   scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
@@ -616,7 +576,7 @@ void AM_minOutWindowScale()
 //
 // Passed nothing, returns nothing
 //
-void AM_maxOutWindowScale(void)
+static void AM_maxOutWindowScale(void)
 {
   scale_mtof = max_scale_mtof;
   scale_ftom = FixedDiv(FRACUNIT, scale_mtof);
@@ -636,7 +596,6 @@ boolean AM_Responder
   int rc;
   static int cheatstate=0;
   static int bigstate=0;
-  static char buffer[20];
   int ch;                                                       // phares
 
   rc = false;
@@ -714,13 +673,9 @@ boolean AM_Responder
     }
     else if (ch == key_map_mark)
     {
-      // Ty 03/27/98 - *not* externalized
-#ifdef HAVE_SNPRINTF
-      snprintf(buffer, sizeof(buffer), "%s %d", s_AMSTR_MARKEDSPOT, markpointnum);
-#else
-      sprintf(buffer, "%s %d", s_AMSTR_MARKEDSPOT, markpointnum);
-#endif
-      plr->message = buffer;
+      /* Ty 03/27/98 - *not* externalized     
+       * cph 2001/11/20 - use doom_printf so we don't have our own buffer */
+      doom_printf("%s %d", s_AMSTR_MARKEDSPOT, markpointnum);
       AM_addMark();
     }
     else if (ch == key_map_clear)
@@ -808,7 +763,7 @@ static void AM_rotate(fixed_t* x,  fixed_t* y, angle_t a, fixed_t xorig, fixed_t
 //
 // Passed nothing, returns nothing
 //
-void AM_changeWindowScale(void)
+static void AM_changeWindowScale(void)
 {
   // Change the scaling multipliers
   scale_mtof = FixedMul(scale_mtof, mtof_zoommul);
@@ -829,7 +784,7 @@ void AM_changeWindowScale(void)
 //
 // Passed nothing, returns nothing
 //
-void AM_doFollowPlayer(void)
+static void AM_doFollowPlayer(void)
 {
   if (f_oldloc.x != plr->mo->x || f_oldloc.y != plr->mo->y)
   {
@@ -879,7 +834,7 @@ void AM_Ticker (void)
 // clipping on them in the lines frame coordinates.
 // Returns true if any part of line was not clipped
 //
-boolean AM_clipMline
+static boolean AM_clipMline
 ( mline_t*  ml,
   fline_t*  fl )
 {
@@ -1005,96 +960,6 @@ boolean AM_clipMline
 }
 #undef DOOUTCODE
 
-//
-// AM_drawFline()
-//
-// Draw a line in the frame buffer.
-// Classic Bresenham w/ whatever optimizations needed for speed
-//
-// Passed the frame coordinates of line, and the color to be drawn
-// Returns nothing
-//
-#ifndef GL_DOOM
-void AM_drawFline
-( fline_t*  fl,
-  int   color )
-{
-  register int x;
-  register int y;
-  register int dx;
-  register int dy;
-  register int sx;
-  register int sy;
-  register int ax;
-  register int ay;
-  register int d;
-
-#ifdef RANGECHECK         // killough 2/22/98
-  static int fuck = 0;
-
-  // For debugging only
-  if
-  (
-       fl->a.x < 0 || fl->a.x >= f_w
-    || fl->a.y < 0 || fl->a.y >= f_h
-    || fl->b.x < 0 || fl->b.x >= f_w
-    || fl->b.y < 0 || fl->b.y >= f_h
-  )
-  {
-    //jff 8/3/98 use logical output routine
-    lprintf(LO_DEBUG, "fuck %d \r", fuck++);
-    return;
-  }
-#endif
-
-#define PUTDOT(xx,yy,cc) V_PlotPixel(FB,xx,yy,(byte)cc)
-
-  dx = fl->b.x - fl->a.x;
-  ax = 2 * (dx<0 ? -dx : dx);
-  sx = dx<0 ? -1 : 1;
-
-  dy = fl->b.y - fl->a.y;
-  ay = 2 * (dy<0 ? -dy : dy);
-  sy = dy<0 ? -1 : 1;
-
-  x = fl->a.x;
-  y = fl->a.y;
-
-  if (ax > ay)
-  {
-    d = ay - ax/2;
-    while (1)
-    {
-      PUTDOT(x,y,color);
-      if (x == fl->b.x) return;
-      if (d>=0)
-      {
-        y += sy;
-        d -= ax;
-      }
-      x += sx;
-      d += ay;
-    }
-  }
-  else
-  {
-    d = ax - ay/2;
-    while (1)
-    {
-      PUTDOT(x, y, color);
-      if (y == fl->b.y) return;
-      if (d >= 0)
-      {
-        x += sx;
-        d -= ay;
-      }
-      y += sy;
-      d += ax;
-    }
-  }
-}
-#endif
-
 //
 // AM_drawMline()
 //
@@ -1106,7 +971,7 @@ void AM_drawFline
 // in the defaults file.
 // Returns nothing.
 //
-void AM_drawMline
+static void AM_drawMline
 ( mline_t*  ml,
   int   color )
 {
@@ -1118,11 +983,7 @@ void AM_drawMline
     color=0;
 
   if (AM_clipMline(ml, &fl))
-#ifdef GL_DOOM
-    gld_DrawLine(fl.a.x, fl.a.y, fl.b.x, fl.b.y, (byte)color);
-#else
-    AM_drawFline(&fl, color); // draws it on frame buffer using fb coords
-#endif
+    V_DrawLine(&fl, color); // draws it on frame buffer using fb coords
 }
 
 //
@@ -1133,7 +994,7 @@ void AM_drawMline
 // Passed the color to draw the grid lines
 // Returns nothing
 //
-void AM_drawGrid(int color)
+static void AM_drawGrid(int color)
 {
   fixed_t x, y;
   fixed_t start, end;
@@ -1188,7 +1049,7 @@ void AM_drawGrid(int color)
 //
 // jff 4/3/98 add routine to get color of generalized keyed door
 //
-int AM_DoorColor(int type)
+static int AM_DoorColor(int type)
 {
   if (GenLockedBase <= type && type< GenDoorBase)
   {
@@ -1212,7 +1073,6 @@ int AM_DoorColor(int type)
     default:
       return -1; //not a keyed door
   }
-  return -1;     //not a keyed door
 }
 
 //
@@ -1231,7 +1091,7 @@ int AM_DoorColor(int type)
 // jff 4/3/98 changed mapcolor_xxxx=0 as control to disable feature
 // jff 4/3/98 changed mapcolor_xxxx=-1 to disable drawing line completely
 //
-void AM_drawWalls(void)
+static void AM_drawWalls(void)
 {
   int i;
   static mline_t l;
@@ -1423,7 +1283,7 @@ void AM_drawWalls(void)
 // the color to draw it with, and the map coordinates to draw it at.
 // Returns nothing
 //
-void AM_drawLineCharacter
+static void AM_drawLineCharacter
 ( mline_t*  lineguy,
   int   lineguylines,
   fixed_t scale,
@@ -1481,7 +1341,7 @@ void AM_drawLineCharacter
 //
 // Passed nothing, returns nothing
 //
-void AM_drawPlayers(void)
+static void AM_drawPlayers(void)
 {
   int   i;
 
@@ -1538,9 +1398,7 @@ void AM_drawPlayers(void)
 // Passed colors and colorrange, no longer used
 // Returns nothing
 //
-void AM_drawThings
-( int colors,
-  int  colorrange)
+static void AM_drawThings(void)
 {
   int   i;
   mobj_t* t;
@@ -1611,6 +1469,8 @@ void AM_drawThings
         16<<FRACBITS,
         t->angle,
 	t->flags & MF_FRIEND && !t->player ? mapcolor_frnd : 
+	/* cph 2006/07/30 - Show count-as-kills in red. */
+          ((t->flags & (MF_COUNTKILL | MF_CORPSE)) == MF_COUNTKILL) ? mapcolor_enemy :
         /* bbm 2/28/03 Show countable items in yellow. */
           t->flags & MF_COUNTITEM ? mapcolor_item : mapcolor_sprt,
         x, y
@@ -1630,7 +1490,7 @@ void AM_drawThings
 // killough 2/22/98:
 // Rewrote AM_drawMarks(). Removed limit on marks.
 //
-void AM_drawMarks(void)
+static void AM_drawMarks(void)
 {
   int i;
   for (i=0;i<markpointnum;i++) // killough 2/22/98: remove automap mark limit
@@ -1678,13 +1538,19 @@ void AM_drawMarks(void)
 
 inline static void AM_drawCrosshair(int color)
 {
-  // single point for now
-#ifdef GL_DOOM
-  gld_DrawLine((f_w/2)-1, (f_h/2), (f_w/2)+1, (f_h/2), (byte)color);
-  gld_DrawLine((f_w/2), (f_h/2)-1, (f_w/2), (f_h/2)+1, (byte)color);
-#else
-  V_PlotPixel(FB, f_w/2, f_h/2, (byte)color);
-#endif
+  fline_t line;
+
+  line.a.x = (f_w/2)-1;
+  line.a.y = (f_h/2);
+  line.b.x = (f_w/2)+1;
+  line.b.y = (f_h/2);
+  V_DrawLine(&line, color);
+
+  line.a.x = (f_w/2);
+  line.a.y = (f_h/2)-1;
+  line.b.x = (f_w/2);
+  line.b.y = (f_h/2)+1;
+  V_DrawLine(&line, color);
 }
 
 //
@@ -1706,7 +1572,7 @@ void AM_Drawer (void)
   AM_drawWalls();
   AM_drawPlayers();
   if (ddt_cheating==2)
-    AM_drawThings(mapcolor_sprt, 0); //jff 1/5/98 default double IDDT sprite
+    AM_drawThings(); //jff 1/5/98 default double IDDT sprite
   AM_drawCrosshair(mapcolor_hair);   //jff 1/7/98 default crosshair color
 
   AM_drawMarks();
