@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -42,6 +44,7 @@
 #include "d_main.h"
 #include "g_game.h"
 #include "m_menu.h"
+#include "p_checksum.h"
 
 #include "protocol.h"
 #include "i_network.h"
@@ -74,6 +77,8 @@ int ticdup = 1;
 static int xtratics = 0;
 int              wanted_player_number;
 
+static void D_QuitNetGame (void);
+
 #ifndef HAVE_NET
 doomcom_t*      doomcom;
 #endif
@@ -89,6 +94,10 @@ void D_InitNetGame (void)
 
   if (!(netgame = server =  !!i)) {
     playeringame[consoleplayer = 0] = true;
+    // e6y
+    // for play, recording or playback using "single-player coop" mode.
+    // Equivalent to using prboom_server with -N 1
+    netgame = M_CheckParm("-solo-net");
   } else {
     // Get game info from server
     packet_header_t *packet = Z_Malloc(1000, PU_STATIC, NULL);
@@ -350,7 +359,7 @@ void NetUpdate(void)
 }
 #else
 
-void D_BuildNewTiccmds()
+static void D_BuildNewTiccmds()
 {
     static int lastmadetic;
     int newtics = I_GetTime() - lastmadetic;
@@ -472,6 +481,7 @@ void TryRunTics (void)
       D_DoAdvanceDemo ();
     M_Ticker ();
     G_Ticker ();
+    P_Checksum(gametic);
     gametic++;
 #ifdef HAVE_NET
     NetUpdate(); // Keep sending our tics to avoid stalling remote nodes
@@ -480,7 +490,7 @@ void TryRunTics (void)
 }
 
 #ifdef HAVE_NET
-void D_QuitNetGame (void)
+static void D_QuitNetGame (void)
 {
   byte buf[1 + sizeof(packet_header_t)];
   packet_header_t *packet = (void*)buf;
