@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2000 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -50,6 +52,7 @@
 #include "doomdef.h"
 #include "doomtype.h"
 #include "doomstat.h"
+#include "d_net.h"
 #include "dstrings.h"
 #include "sounds.h"
 #include "z_zone.h"
@@ -57,12 +60,11 @@
 #include "s_sound.h"
 #include "v_video.h"
 #include "f_finale.h"
-#ifndef GL_DOOM
 #include "f_wipe.h"
-#endif /* GL_DOOM */
 #include "m_argv.h"
 #include "m_misc.h"
 #include "m_menu.h"
+#include "p_checksum.h"
 #include "i_main.h"
 #include "i_system.h"
 #include "i_sound.h"
@@ -79,14 +81,9 @@
 #include "d_deh.h"  // Ty 04/08/98 - Externalizations
 #include "lprintf.h"  // jff 08/03/98 - declaration of lprintf
 #include "am_map.h"
-#ifdef GL_DOOM
-#include "gl_struct.h"
-#endif
-
-// DEHacked support - Ty 03/09/97 // CPhipps - const char*'s
-void ProcessDehFile(const char *filename, const char *outfilename, int lumpnum);
 
 void GetFirstMap(int *ep, int *map); // Ty 08/29/98 - add "-warp x" functionality
+static void D_PageDrawer(void);
 
 // CPhipps - removed wadfiles[] stuff
 
@@ -120,11 +117,8 @@ int ffmap;
 
 boolean advancedemo;
 
-extern boolean timingdemo, singledemo, demoplayback, fastdemo; // killough
-
 char    wadfile[PATH_MAX+1];       // primary wad file
 char    mapdir[PATH_MAX+1];        // directory of development maps
-char    basedefault[PATH_MAX+1];   // default file
 char    baseiwad[PATH_MAX+1];      // jff 3/23/98: iwad directory
 char    basesavegame[PATH_MAX+1];  // killough 2/16/98: savegame directory
 
@@ -138,11 +132,10 @@ const char *const standard_iwads[]=
   "doom.wad",
   "doom1.wad",
   "doomu.wad", /* CPhipps - alow doomu.wad */
+  "freedoom.wad", /* wart@kobold.org:  added freedoom for Fedora Extras */
 };
 static const int nstandard_iwads = sizeof standard_iwads/sizeof*standard_iwads;
 
-void D_DoAdvanceDemo (void);
-
 /*
  * D_PostEvent - Event handling
  *
@@ -174,7 +167,6 @@ void D_PostEvent(event_t *ev)
 // The screens to wipe between are already stored, this just does the timing
 // and screen updating
 
-#ifndef GL_DOOM
 static void D_Wipe(void)
 {
   boolean done;
@@ -191,14 +183,13 @@ static void D_Wipe(void)
         }
       while (!tics);
       wipestart = nowtime;
-      done = wipe_ScreenWipe(0,0,SCREENWIDTH,SCREENHEIGHT,tics);
+      done = wipe_ScreenWipe(tics);
       I_UpdateNoBlit();
       M_Drawer();                   // menu is drawn even on top of wipes
       I_FinishUpdate();             // page flip or blit buffer
     }
   while (!done);
 }
-#endif /* GL_DOOM */
 
 //
 // D_Display
@@ -210,25 +201,21 @@ gamestate_t    wipegamestate = GS_DEMOSCREEN;
 extern boolean setsizeneeded;
 extern int     showMessages;
 
-void D_Display (void)
+static void D_Display (void)
 {
   static boolean inhelpscreensstate   = false;
   static boolean isborderstate        = false;
   static boolean borderwillneedredraw = false;
   static gamestate_t oldgamestate = -1;
-#ifndef GL_DOOM
   boolean wipe;
-#endif
   boolean viewactive = false, isborder = false;
 
   if (nodrawers)                    // for comparative timing / profiling
     return;
 
-#ifndef GL_DOOM
   // save the current screen if about to wipe
-  if ((wipe = gamestate != wipegamestate))
-    wipe_StartScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
-#endif /* GL_DOOM */
+  if ((wipe = gamestate != wipegamestate) && (V_GetMode() != VID_MODEGL))
+    wipe_StartScreen();
 
   if (gamestate != GS_LEVEL) { // Not a level
     switch (oldgamestate) {
@@ -278,12 +265,8 @@ void D_Display (void)
       // and there is a menu being displayed
       borderwillneedredraw = menuactive && isborder && viewactive && (viewwidth != SCREENWIDTH);
     }
-#ifdef GL_DOOM
-    R_DrawViewBorder();
-#else
-    if (redrawborderstuff)
+    if (redrawborderstuff || (V_GetMode() == VID_MODEGL))
       R_DrawViewBorder();
-#endif
 
     // Now do the drawing
     if (viewactive)
@@ -291,9 +274,8 @@ void D_Display (void)
     if (automapmode & am_active)
       AM_Drawer();
     ST_Drawer((viewheight != SCREENHEIGHT) || ((automapmode & am_active) && !(automapmode & am_overlay)), redrawborderstuff);
-#ifndef GL_DOOM
-    R_DrawViewBorder();
-#endif
+    if (V_GetMode() != VID_MODEGL)
+      R_DrawViewBorder();
     HU_Drawer();
   }
 
@@ -303,19 +285,9 @@ void D_Display (void)
 
   // draw pause pic
   if (paused) {
-      static int x;
-
-      if (!x) { // Cache results of x pos calc
-  int lump = W_GetNumForName("M_PAUSE");
-  const patch_t* p = W_CacheLumpNum(lump);
-  x = (320 - SHORT(p->width))/2;
-  W_UnlockLumpNum(lump);
-      }
-
-      // CPhipps - updated for new patch drawing
-      V_DrawNamePatch(x, (!(automapmode & am_active) || (automapmode & am_overlay))
-          ? 4+(viewwindowy*200/SCREENHEIGHT) : 4, // cph - Must un-stretch viewwindowy
-          0, "M_PAUSE", CR_DEFAULT, VPT_STRETCH);
+    // Simplified the "logic" here and no need for x-coord caching - POPE
+    V_DrawNamePatch((320 - V_NamePatchWidth("M_PAUSE"))/2, 4,
+                    0, "M_PAUSE", CR_DEFAULT, VPT_STRETCH);
   }
 
   // menus go directly to the screen
@@ -326,18 +298,14 @@ void D_Display (void)
   D_BuildNewTiccmds();
 #endif
 
-#ifndef GL_DOOM
   // normal update
-  if (!wipe)
+  if (!wipe || (V_GetMode() == VID_MODEGL))
     I_FinishUpdate ();              // page flip or blit buffer
   else {
     // wipe update
-    wipe_EndScreen(0, 0, SCREENWIDTH, SCREENHEIGHT);
+    wipe_EndScreen();
     D_Wipe();
   }
-#else
-  I_FinishUpdate ();              // page flip or blit buffer
-#endif /* GL_DOOM */
 }
 
 // CPhipps - Auto screenshot Variables
@@ -356,8 +324,6 @@ static const char *auto_shot_fname;
 //  calls I_GetTime, I_StartFrame, and I_StartTic
 //
 
-extern boolean demorecording;
-
 static void D_DoomLoop(void)
 {
   for (;;)
@@ -376,6 +342,7 @@ static void D_DoomLoop(void)
             D_DoAdvanceDemo ();
           M_Ticker ();
           G_Ticker ();
+          P_Checksum(gametic);
           gametic++;
           maketic++;
         }
@@ -418,7 +385,7 @@ void D_PageTicker(void)
 //
 // D_PageDrawer
 //
-void D_PageDrawer(void)
+static void D_PageDrawer(void)
 {
   // proff/nicolas 09/14/98 -- now stretchs bitmaps to fullscreen!
   // CPhipps - updated for new patch drawing
@@ -613,16 +580,15 @@ static void CheckIWAD(const char *iwadname,GameMode_t *gmode,boolean *hassec)
   if ( !access (iwadname,R_OK) )
   {
     int ud=0,rg=0,sw=0,cm=0,sc=0;
-    int handle;
+    FILE* fp;
 
     // Identify IWAD correctly
-    if ( (handle = open (iwadname,O_RDONLY | O_BINARY)) != -1)
+    if ((fp = fopen(iwadname, "rb")))
     {
       wadinfo_t header;
 
       // read IWAD header
-      read (handle, &header, sizeof(header));
-      if (!strncmp(header.identification,"IWAD",4))
+      if (fread(&header, sizeof(header), 1, fp) == 1 && !strncmp(header.identification, "IWAD", 4))
       {
         size_t length;
         filelump_t *fileinfo;
@@ -632,9 +598,10 @@ static void CheckIWAD(const char *iwadname,GameMode_t *gmode,boolean *hassec)
         header.infotableofs = LONG(header.infotableofs);
         length = header.numlumps;
         fileinfo = malloc(length*sizeof(filelump_t));
-        lseek (handle, header.infotableofs, SEEK_SET);
-        read (handle, fileinfo, length*sizeof(filelump_t));
-        close(handle);
+        if (fseek (fp, header.infotableofs, SEEK_SET) ||
+            fread (fileinfo, sizeof(filelump_t), length, fp) != length ||
+            fclose(fp))
+          I_Error("CheckIWAD: failed to read directory %s",iwadname);
 
         // scan directory for levelname lumps
         while (length--)
@@ -702,7 +669,7 @@ static void CheckIWAD(const char *iwadname,GameMode_t *gmode,boolean *hassec)
 //
 // jff 4/19/98 Make killoughs slash fixer a subroutine
 //
-void NormalizeSlashes(char *str)
+static void NormalizeSlashes(char *str)
 {
   int l;
 
@@ -718,47 +685,12 @@ void NormalizeSlashes(char *str)
       str[l]='/';
 }
 
-// jff 4/19/98 Add routine to check a pathname for existence as
-// a file or directory. If neither append .wad and check if it
-// exists as a file then. Else return non-existent.
-
-boolean WadFileStatus(char *filename,boolean *isdir)
-{
-  struct stat sbuf;
-  int i;
-
-  *isdir = false;                 //default is directory to false
-  if (!filename || !*filename)    //if path NULL or empty, doesn't exist
-    return false;
-
-  if (!stat(filename,&sbuf))      //check for existence
-  {
-    *isdir=S_ISDIR(sbuf.st_mode); //if it does, set whether a dir or not
-    return true;                  //return does exist
-  }
-
-  i = strlen(filename);           //get length of path
-  if (i>=4)
-    if(!strnicmp(filename+i-4,".wad",4))
-      return false;               //if already ends in .wad, not found
-
-  strcat(filename,".wad");        //try it with .wad added
-  if (!stat(filename,&sbuf))      //if it exists then
-  {
-    if (S_ISDIR(sbuf.st_mode))    //but is a dir, then say we didn't find it
-      return false;
-    return true;                  //otherwise return file found, w/ .wad added
-  }
-  filename[i]=0;                  //remove .wad
-  return false;                   //and report doesn't exist
-}
-
 /*
  * FindIWADFIle
  *
  * Search for one of the standard IWADs
  * CPhipps  - static, proper prototype
- *    - 12/1999 - rewritten to use FindWADFile
+ *    - 12/1999 - rewritten to use I_FindFile
  */
 static char *FindIWADFile(void)
 {
@@ -796,19 +728,12 @@ static char *FindIWADFile(void)
 //
 // jff 4/19/98 rewritten to use a more advanced search algorithm
 
-void IdentifyVersion (void)
+static void IdentifyVersion (void)
 {
   int         i;    //jff 3/24/98 index of args on commandline
   struct stat sbuf; //jff 3/24/98 used to test save path for existence
   char *iwad;
 
-  // get config file from same directory as executable
-#ifdef GL_DOOM
-  sprintf(basedefault,"%s/glboom.cfg", I_DoomExeDir());  // killough
-#else
-  sprintf(basedefault,"%s/prboom.cfg", I_DoomExeDir());  // killough
-#endif
-
   // set save path to -save parm or current dir
 
   //jff 3/27/98 default to current dir
@@ -899,7 +824,7 @@ void IdentifyVersion (void)
 
 #define MAXARGVS 100
 
-void FindResponseFile (void)
+static void FindResponseFile (void)
 {
   int i;
 
@@ -909,7 +834,7 @@ void FindResponseFile (void)
         int  size;
         int  index;
 	int indexinfile;
-        char *file = NULL;
+        byte *file = NULL;
         const char **moreargs = malloc(myargc * sizeof(const char*));
         const char **newargv;
         // proff 04/05/2000: Added for searching responsefile
@@ -966,7 +891,7 @@ void FindResponseFile (void)
 	}
 
         {
-	  char *infile = file;
+	  byte *infile = file;
 	  indexinfile = 0;
 	  indexinfile++;  // SKIP PAST ARGV[0] (KEEP IT)
 	  do {
@@ -1031,7 +956,7 @@ void FindResponseFile (void)
 //
 // CPhipps - OUCH! Writing into *myargv is too dodgy, damn
 
-void DoLooseFiles(void)
+static void DoLooseFiles(void)
 {
   char *wads[MAXARGVS];  // store the respective loose filenames
   char *lmps[MAXARGVS];
@@ -1146,41 +1071,52 @@ const char *wad_files[MAXLOADFILES], *deh_files[MAXLOADFILES];
 // CPhipps - misc screen stuff
 unsigned int desired_screenwidth, desired_screenheight;
 
+static void L_SetupConsoleMasks(void) {
+  int p;
+  int i;
+  const char *cena="ICWEFDA",*pos;  //jff 9/3/98 use this for parsing console masks // CPhipps - const char*'s
+
+  //jff 9/3/98 get mask for console output filter
+  if ((p = M_CheckParm ("-cout"))) {
+    lprintf(LO_DEBUG, "mask for stdout console output: ");
+    if (++p != myargc && *myargv[p] != '-')
+      for (i=0,cons_output_mask=0;(size_t)i<strlen(myargv[p]);i++)
+        if ((pos = strchr(cena,toupper(myargv[p][i])))) {
+          cons_output_mask |= (1<<(pos-cena));
+          lprintf(LO_DEBUG, "%c", toupper(myargv[p][i]));
+        }
+    lprintf(LO_DEBUG, "\n");
+  }
+
+  //jff 9/3/98 get mask for redirected console error filter
+  if ((p = M_CheckParm ("-cerr"))) {
+    lprintf(LO_DEBUG, "mask for stderr console output: ");
+    if (++p != myargc && *myargv[p] != '-')
+      for (i=0,cons_error_mask=0;(size_t)i<strlen(myargv[p]);i++)
+        if ((pos = strchr(cena,toupper(myargv[p][i])))) {
+          cons_error_mask |= (1<<(pos-cena));
+          lprintf(LO_DEBUG, "%c", toupper(myargv[p][i]));
+        }
+    lprintf(LO_DEBUG, "\n");
+  }
+}
+
 //
 // D_DoomMainSetup
 //
 // CPhipps - the old contents of D_DoomMain, but moved out of the main
 //  line of execution so its stack space can be freed
 
-void D_DoomMainSetup(void)
+static void D_DoomMainSetup(void)
 {
-  int p,i,slot;
-  const char *cena="ICWEFDA",*pos;  //jff 9/3/98 use this for parsing console masks // CPhipps - const char*'s
+  int p,slot;
 
-  // figgi 09/18/00-- added switch to force classic bsp nodes
-  if (M_CheckParm ("-forceoldbsp"))
-  {
-    extern boolean forceOldBsp;
-    forceOldBsp = true;
-  }
-
-  //jff 9/3/98 get mask for console output filter
-  if ((p = M_CheckParm ("-cout")))
-    if (++p != myargc && *myargv[p] != '-')
-      for (i=0,cons_output_mask=0;(size_t)i<strlen(myargv[p]);i++)
-        if ((pos = strchr(cena,toupper(myargv[p][i]))))
-          cons_output_mask |= (1<<(pos-cena));
-
-  //jff 9/3/98 get mask for redirected console error filter
-  if ((p = M_CheckParm ("-cerr")))
-    if (++p != myargc && *myargv[p] != '-')
-      for (i=0,cons_error_mask=0;(size_t)i<strlen(myargv[p]);i++)
-        if ((pos = strchr(cena,toupper(myargv[p][i]))))
-          cons_error_mask |= (1<<(pos-cena));
+  L_SetupConsoleMasks();
 
   setbuf(stdout,NULL);
 
   // proff 04/05/2000: Added support for include response files
+  /* proff 2001/7/1 - Moved up, so -config can be in response files */
   {
     boolean rsp_found;
     int i;
@@ -1193,6 +1129,17 @@ void D_DoomMainSetup(void)
       FindResponseFile();
     } while (rsp_found==true);
   }
+
+  lprintf(LO_INFO,"M_LoadDefaults: Load system defaults.\n");
+  M_LoadDefaults();              // load before initing other systems
+
+  // figgi 09/18/00-- added switch to force classic bsp nodes
+  if (M_CheckParm ("-forceoldbsp"))
+  {
+    extern boolean forceOldBsp;
+    forceOldBsp = true;
+  }
+
   DoLooseFiles();  // Ty 08/29/98 - handle "loose" files on command line
   IdentifyVersion();
 
@@ -1361,9 +1308,6 @@ void D_DoomMainSetup(void)
   }
 
   // init subsystems
-  //jff 9/3/98 use logical output routine
-  lprintf(LO_INFO,"M_LoadDefaults: Load system defaults.\n");
-  M_LoadDefaults();              // load before initing other systems
 
   G_ReloadDefaults();    // killough 3/4/98: set defaults just loaded.
   // jff 3/24/98 this sets startskill if it was -1
@@ -1378,27 +1322,18 @@ void D_DoomMainSetup(void)
       desired_screenheight = atoi(myargv[p+1]);
 
   { // -geometry handling, change screen size for this session only
-    int w = desired_screenwidth, h = desired_screenheight;
-    char buf[20];
+    // e6y: new code by me
+    int w, h;
 
     if (!(p = M_CheckParm("-geom")))
       p = M_CheckParm("-geometry");
 
-    // Carefully, carefully here, as we step around potential buffer overruns
-    // and bad parameters
-    if (p && (p+1<myargc))
-      if (strlen(myargv[p+1]) < 19) {
-  char* ph;
-  strcpy(buf, myargv[p+1]);
-  if ((ph = strchr(buf, 'x'))) {
-    *ph++ = 0;
-    if (strchr(ph, '-')) *strchr(ph, '-') = 0;
-    if (strchr(ph, '+')) *strchr(ph, '+') = 0;
-    w = atoi(buf); h = atoi(ph);
-  }
-      }
-
-    I_SetRes(w, h);
+    if (!(p && (p+1<myargc) && sscanf(myargv[p+1], "%dx%d", &w, &h) == 2))
+    {
+      w = desired_screenwidth;
+      h = desired_screenheight;
+    }
+    I_CalculateRes(w, h);
   }
 
   if ((p = M_CheckParm("-fullscreen")))
@@ -1455,6 +1390,8 @@ void D_DoomMainSetup(void)
   // Using -deh in BOOM, others use -dehacked.
   // Ty 03/18/98 also allow .bex extension.  .bex overrides if both exist.
 
+  D_BuildBEXTables(); // haleyjd
+
   p = M_CheckParm ("-deh");
   if (p)
     {
@@ -1529,8 +1466,11 @@ void D_DoomMainSetup(void)
 
   lprintf(LO_INFO,"\n");     // killough 3/6/98: add a newline, by popular demand :)
 
-  if ((p = W_CheckNumForName("DEHACKED")) != -1) // cph - add dehacked-in-a-wad support
-    ProcessDehFile(NULL, D_dehout(), p);
+  // e6y 
+  // option to disable automatic loading of dehacked-in-wad lump
+  if (!M_CheckParm ("-nodeh"))
+    if ((p = W_CheckNumForName("DEHACKED")) != -1) // cph - add dehacked-in-a-wad support
+      ProcessDehFile(NULL, D_dehout(), p);
 
   V_InitColorTranslation(); //jff 4/24/98 load color translation lumps
 
@@ -1589,20 +1529,6 @@ void D_DoomMainSetup(void)
     if ((auto_shot_count = auto_shot_time = atoi(myargv[p+1])))
       auto_shot_fname = myargv[p+2];
 
-  // check for a driver that wants intermission stats
-  if ((p = M_CheckParm ("-statcopy")) && p<myargc-1)
-    {
-      // for statistics driver
-      extern  void* statcopy;
-
-      // killough 5/2/98: this takes a memory
-      // address as an integer on the command line!
-
-      statcopy = (void*) atoi(myargv[p+1]);
-      //jff 9/3/98 use logical output routine
-      lprintf (LO_CONFIRM,"External statistics registered.\n");
-    }
-
   // start the apropriate game based on parms
 
   // killough 12/98:
@@ -1620,6 +1546,11 @@ void D_DoomMainSetup(void)
   }
     }
 
+  if ((p = M_CheckParm ("-checksum")) && ++p < myargc)
+    {
+      P_RecordChecksum (myargv[p]);
+    }
+
   if ((p = M_CheckParm ("-fastdemo")) && ++p < myargc)
     {                                 // killough
       fastdemo = true;                // run at fastest speed possible
