@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -33,7 +35,6 @@
 #ifndef __D_PLAYER__
 #define __D_PLAYER__
 
-typedef struct player_s player_t;
 
 // The player data structure depends on a number
 // of other structs: items (internal inventory),
@@ -66,7 +67,7 @@ typedef enum
   // Dead on the ground, view follows killer.
   PST_DEAD,
   // Ready to restart/respawn???
-  PST_REBORN            
+  PST_REBORN
 
 } playerstate_t;
 
@@ -89,7 +90,7 @@ typedef enum
 //
 // Extended player object info: player_t
 //
-struct player_s
+typedef struct player_s
 {
   mobj_t*             mo;
   playerstate_t       playerstate;
@@ -104,7 +105,7 @@ struct player_s
   // Bob/squat speed.
   fixed_t             deltaviewheight;
   // bounded/scaled total momentum.
-  fixed_t             bob;    
+  fixed_t             bob;
 
   /* killough 10/98: used for realistic bobbing (i.e. not simply overall speed)
    * mo->momx and mo->momy represent true momenta experienced by player.
@@ -115,21 +116,20 @@ struct player_s
 
   // This is only used between levels,
   // mo->health is used during levels.
-  int                 health; 
+  int                 health;
   int                 armorpoints;
   // Armor type is 0-2.
-  int                 armortype;      
+  int                 armortype;
 
   // Power ups. invinc and invis are tic counters.
   int                 powers[NUMPOWERS];
   boolean             cards[NUMCARDS];
   boolean             backpack;
-  
+
   // Frags, kills of other players.
   int                 frags[MAXPLAYERS];
-  int                 totalfrags;
   weapontype_t        readyweapon;
-  
+
   // Is wp_nochange if not changing.
   weapontype_t        pendingweapon;
 
@@ -143,10 +143,10 @@ struct player_s
 
   // Bit flags, for cheats and debug.
   // See cheat_t, above.
-  int                 cheats;         
+  int                 cheats;
 
   // Refired shots are less accurate.
-  int                 refire;         
+  int                 refire;
 
    // For intermission stats.
   int                 killcount;
@@ -154,16 +154,15 @@ struct player_s
   int                 secretcount;
 
   // Hint messages. // CPhipps - const
-  // sf: now done with doom_printf and player_printf
-  // const char*         message;        
-  
+  const char*         message;
+
   // For screen flashing (red or bright).
   int                 damagecount;
   int                 bonuscount;
 
   // Who did damage (NULL for floors/ceilings).
   mobj_t*             attacker;
-  
+
   // So gun flashes light up areas.
   int                 extralight;
 
@@ -173,16 +172,15 @@ struct player_s
 
   // Player skin colorshift,
   //  0-3 for which color to draw player.
-  int                 colormap;       
+  int                 colormap;
 
   // Overlay view sprites (gun, etc).
   pspdef_t            psprites[NUMPSPRITES];
 
   // True if secret level has been done.
-  boolean             didsecret;      
+  boolean             didsecret;
 
-  char                name[20];
-};
+} player_t;
 
 
 //
@@ -192,15 +190,15 @@ struct player_s
 typedef struct
 {
   boolean     in;     // whether the player is in game
-    
+
   // Player stats, kills, collected items etc.
   int         skills;
   int         sitems;
   int         ssecret;
-  int         stime; 
+  int         stime;
   int         frags[4];
   int         score;  // current score on entry, modified on return
-  
+
 } wbplayerstruct_t;
 
 typedef struct
@@ -209,11 +207,11 @@ typedef struct
 
   // if true, splash the secret level
   boolean     didsecret;
-    
+
   // previous and next levels, origin 0
   int         last;
-  int         next;   
-    
+  int         next;
+
   int         maxkills;
   int         maxitems;
   int         maxsecret;
@@ -221,12 +219,12 @@ typedef struct
 
   // the par time
   int         partime;
-    
+
   // index of this player in game
-  int         pnum;   
+  int         pnum;
 
   wbplayerstruct_t    plyr[MAXPLAYERS];
-  
+
   // CPhipps - total game time for completed levels so far
   int         totaltimes;
 
