@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2006 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -322,7 +324,12 @@ static int badplayer(int n) { return (n < 0 || n >= MAXPLAYERS); }
 
 int main(int argc, char** argv)
 {
-  int localport = 5030, numplayers = 2, xtratics = 0, ticdup = 1;
+#ifndef USE_SDL_NET
+  int localport = 5030;
+#else
+  Uint16 localport = 5030;
+#endif
+  int numplayers = 2, xtratics = 0, ticdup = 1;
   int exectics = 0; // gametics completed
   struct setup_packet_s setupinfo = { 2, 0, 1, 1, 1, 0, best_compatibility, 0, 0};
   char**wadname = NULL;
