@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2006 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -50,9 +52,13 @@ typedef unsigned char byte;
 #ifndef _MSC_VER
 typedef signed long long int_64_t;
 typedef unsigned long long uint_64_t;
+// define compiled-specific long-long contstant notation here
+#define LONGLONG(num)   (uint_64_t)num ## ll
 #else
 typedef __int64 int_64_t;
 typedef unsigned __int64 uint_64_t;
+// define compiled-specific long-long contstant notation here
+#define LONGLONG(num) (uint_64_t)num
 #undef PATH_MAX
 #define PATH_MAX 1024
 #define strcasecmp _stricmp
@@ -62,8 +68,10 @@ typedef unsigned __int64 uint_64_t;
 
 #ifdef __GNUC__
 #define CONSTFUNC __attribute__((const))
+#define PUREFUNC __attribute__((pure))
 #else
 #define CONSTFUNC
+#define PUREFUNC
 #endif
 
 /* CPhipps - use limits.h instead of depreciated values.h */
@@ -95,4 +103,12 @@ typedef enum {
   best_compatibility = prboom_6_compatibility,
 } complevel_t;
 
+/* cph - from v_video.h, needed by gl_struct.h */
+enum patch_translation_e {
+  VPT_NONE    = 0, // Normal
+  VPT_FLIP    = 1, // Flip image horizontally
+  VPT_TRANS   = 2, // Translate image via a translation table
+  VPT_STRETCH = 4, // Stretch to compensate for high-res
+};
+
 #endif
