@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2006 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -51,6 +53,7 @@
 #endif
 
 #include "doomstat.h"
+#include "d_net.h"
 #include "f_finale.h"
 #include "m_argv.h"
 #include "m_misc.h"
@@ -60,6 +63,7 @@
 #include "p_saveg.h"
 #include "p_tick.h"
 #include "p_map.h"
+#include "p_checksum.h"
 #include "d_main.h"
 #include "wi_stuff.h"
 #include "hu_stuff.h"
@@ -80,6 +84,7 @@
 #include "lprintf.h"
 #include "i_main.h"
 #include "i_system.h"
+#include "r_demo.h"
 
 #define SAVEGAMESIZE  0x20000
 #define SAVESTRINGSIZE  24
@@ -115,7 +120,6 @@ player_t        players[MAXPLAYERS];
 int             consoleplayer; // player taking events and displaying
 int             displayplayer; // view being displayed
 int             gametic;
-int             levelstarttic; // gametic at level start
 int             basetic;       /* killough 9/29/98: for demo sync */
 int             totalkills, totallive, totalitems, totalsecret;    // for intermission
 boolean         demorecording;
@@ -244,16 +248,13 @@ static buttoncode_t special_event; // Event triggered by local player, to send
 static byte  savegameslot;         // Slot to load if gameaction == ga_loadgame
 char         savedescription[SAVEDESCLEN];  // Description to save in savegame if gameaction == ga_savegame
 
-//jff 3/24/98 declare startskill external, define defaultskill here
-extern skill_t startskill;      //note 0-based
+//jff 3/24/98 define defaultskill here
 int defaultskill;               //note 1-based
 
 // killough 2/8/98: make corpse queue variable in size
 int    bodyqueslot, bodyquesize;        // killough 2/8/98
 mobj_t **bodyque = 0;                   // phares 8/10/98
 
-void   *statcopy;       // for statistics driver
-
 static void G_DoSaveGame (boolean menu);
 static const byte* G_ReadDemoHeader(const byte* demo_p);
 
@@ -295,7 +296,8 @@ void G_BuildTiccmd(ticcmd_t* cmd)
 
   strafe = gamekeydown[key_strafe] || mousebuttons[mousebstrafe]
     || joybuttons[joybstrafe];
-  speed = autorun || gamekeydown[key_speed] || joybuttons[joybspeed]; // phares
+  //e6y: the "RUN" key inverts the autorun state
+  speed = (gamekeydown[key_speed] || joybuttons[joybspeed] ? !autorun : autorun); // phares
 
   forward = side = 0;
 
@@ -538,8 +540,6 @@ void G_RestartLevel(void)
 // G_DoLoadLevel
 //
 
-extern gamestate_t wipegamestate;
-
 static void G_DoLoadLevel (void)
 {
   int i;
@@ -582,7 +582,7 @@ static void G_DoLoadLevel (void)
         break;
       }//jff 3/27/98 end sky setting fix
 
-  levelstarttic = gametic;        // for time calculation
+  /* cph 2006/07/31 - took out unused levelstarttic variable */
 
   if (!demo_compatibility && !mbf_features)   // killough 9/29/98
     basetic = gametic;
@@ -671,6 +671,7 @@ boolean G_Responder (event_t* ev)
     ST_Start();    // killough 3/7/98: switch status bar views too
     HU_Start();
     S_UpdateSounds(players[displayplayer].mo);
+    R_SmoothPlaying_Reset(NULL); // e6y
   }
       return true;
     }
@@ -762,8 +763,6 @@ boolean G_Responder (event_t* ev)
 // Make ticcmd_ts for the players.
 //
 
-extern int mapcolor_me;
-
 void G_Ticker (void)
 {
   int i;
@@ -914,6 +913,15 @@ void G_Ticker (void)
   // cph - if the gamestate changed, we may need to clean up the old gamestate
   if (gamestate != prevgamestate) {
     switch (prevgamestate) {
+    case GS_LEVEL:
+      // This causes crashes at level end - Neil Stevens
+      // The crash is because the sounds aren't stopped before freeing them
+      // the following is a possible fix
+      // This fix does avoid the crash wowever, with this fix in, the exit
+      // switch sound is cut off
+      // S_Stop();
+      // Z_FreeTags(PU_LEVEL, PU_PURGELEVEL-1);
+      break;
     case GS_INTERMISSION:
       WI_End();
     default:
@@ -956,12 +964,12 @@ void G_Ticker (void)
 // Can when a player completes a level.
 //
 
-void G_PlayerFinishLevel(int player)
+static void G_PlayerFinishLevel(int player)
 {
   player_t *p = &players[player];
   memset(p->powers, 0, sizeof p->powers);
   memset(p->cards, 0, sizeof p->cards);
-  p->mo->flags &= ~MF_SHADOW;   // cancel invisibility
+  p->mo = NULL;           // cph - this is allocated PU_LEVEL so it's gone
   p->extralight = 0;      // cancel gun flashes
   p->fixedcolormap = 0;   // cancel ir gogles
   p->damagecount = 0;     // no palette changes
@@ -974,7 +982,6 @@ void G_PlayerFinishLevel(int player)
 // G_SetPlayerColour
 
 #include "r_draw.h"
-extern byte playernumtotrans[MAXPLAYERS];
 
 void G_ChangedPlayerColour(int pn, int cl)
 {
@@ -1048,9 +1055,7 @@ void G_PlayerReborn (int player)
 // because something is occupying it
 //
 
-void P_SpawnPlayer(mapthing_t *mthing);
-
-boolean G_CheckSpot(int playernum, mapthing_t *mthing)
+static boolean G_CheckSpot(int playernum, mapthing_t *mthing)
 {
   fixed_t     x,y;
   subsector_t *ss;
@@ -1088,7 +1093,6 @@ boolean G_CheckSpot(int playernum, mapthing_t *mthing)
 
   if (bodyquesize > 0)
     {
-      static mobj_t **bodyque;
       static int queuesize;
       if (queuesize < bodyquesize)
 	{
@@ -1169,16 +1173,13 @@ void G_DeathMatchSpawnPlayer (int playernum)
       if (G_CheckSpot (playernum, &deathmatchstarts[i]) )
         {
           deathmatchstarts[i].type = playernum+1;
-          P_SpawnPlayer (&deathmatchstarts[i]);
+          P_SpawnPlayer (playernum, &deathmatchstarts[i]);
           return;
         }
     }
 
   // no good spot, so the player will probably get stuck
-  if (playerstarts[playernum].type > 0)
-    P_SpawnPlayer (&playerstarts[playernum]);
-  else
-    I_Error("Tried & failed to use co-op start");
+  P_SpawnPlayer (playernum, &playerstarts[playernum]);
 }
 
 //
@@ -1205,7 +1206,7 @@ void G_DoReborn (int playernum)
 
       if (G_CheckSpot (playernum, &playerstarts[playernum]) )
         {
-          P_SpawnPlayer (&playerstarts[playernum]);
+          P_SpawnPlayer (playernum, &playerstarts[playernum]);
           return;
         }
 
@@ -1214,14 +1215,12 @@ void G_DoReborn (int playernum)
         {
           if (G_CheckSpot (playernum, &playerstarts[i]) )
             {
-              playerstarts[i].type = playernum+1; // fake as other player
-              P_SpawnPlayer (&playerstarts[i]);
-              playerstarts[i].type = i+1;   // restore
+              P_SpawnPlayer (playernum, &playerstarts[i]);
               return;
             }
           // he's going to be inside something.  Too bad.
         }
-      P_SpawnPlayer (&playerstarts[playernum]);
+      P_SpawnPlayer (playernum, &playerstarts[playernum]);
     }
 }
 
@@ -1374,9 +1373,6 @@ void G_DoCompleted (void)
   gamestate = GS_INTERMISSION;
   automapmode &= ~am_active;
 
-  if (statcopy)
-    memcpy (statcopy, &wminfo, sizeof(wminfo));
-
   WI_Start (&wminfo);
 }
 
@@ -1429,7 +1425,6 @@ void G_DoWorldDone (void)
 #define MIN_MAXPLAYERS 32
 
 extern boolean setsizeneeded;
-void R_ExecuteSetViewSize(void);
 
 //CPhipps - savename variable redundant
 
@@ -1507,6 +1502,7 @@ void G_LoadGame(int slot, boolean command)
     demoplayback = false;
   }
   command_loadgame = command;
+  R_SmoothPlaying_Reset(NULL); // e6y
 }
 
 // killough 5/15/98:
@@ -1539,6 +1535,7 @@ static byte comp_options_by_version[] =
   21,22, /* PrBoom v2.1-v2.2 have 21 */
   23,23,23 /* PrBoom v2.3 has 23 and counting... */
 };
+// FIXME? - prboom 2.3 - r646 - has 24
 
 static byte map_old_comp_levels[] =
 { 0, 1, 2, 4, 5, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16 };
@@ -1569,6 +1566,8 @@ void G_DoLoadGame(void)
   gameaction = ga_nothing;
 
   length = M_ReadFile(name, &savebuffer);
+  if (length<=0)
+    I_Error("Couldn't read file %s: %s", name, "(Unknown Error)");
   save_p = savebuffer + SAVESTRINGSIZE;
 
   // CPhipps - read the description field, compare with supported ones
@@ -1663,6 +1662,7 @@ void G_DoLoadGame(void)
   P_UnArchiveSpecials ();
   P_UnArchiveRNG ();    // killough 1/18/98: load RNG information
   P_UnArchiveMap ();    // killough 1/22/98: load automap information
+  R_SmoothPlaying_Reset(NULL); // e6y
 
   if (*save_p != 0xe6)
     I_Error ("G_DoLoadGame: Bad savegame");
@@ -1713,9 +1713,23 @@ void G_SaveGame(int slot, char *description)
 }
 
 // Check for overrun and realloc if necessary -- Lee Killough 1/22/98
-void CheckSaveGame(size_t size)
+void (CheckSaveGame)(size_t size, const char* file, int line)
 {
   size_t pos = save_p - savebuffer;
+
+#ifdef RANGECHECK
+  /* cph 2006/08/07 - after-the-fact sanity checking of CheckSaveGame calls */
+  static size_t prev_check;
+  static const char* prevf;
+  static int prevl;
+
+  if (pos > prev_check)
+    I_Error("CheckSaveGame at %s:%d called for insufficient buffer (%u < %u)", prevf, prevl, prev_check, pos);
+  prev_check = size + pos;
+  prevf = file;
+  prevl = line;
+#endif
+
   size += 1024;  // breathing room
   if (pos+size > savegamesize)
     save_p = (savebuffer = realloc(savebuffer,
@@ -1767,14 +1781,6 @@ static void G_DoSaveGame (boolean menu)
       i = num_version_headers+1;
     }
 
-  if ((size_t)i == num_version_headers) {
-    doom_printf("No savegame signature known for\nthis compatibility level\n"
-    "%d/%d, %u registered", compatibility_level,
-    MAX_COMPATIBILITY_LEVEL, num_version_headers);
-    free(savebuffer); // cph - free data
-    return;
-  }
-
   save_p += VERSIONSIZE;
 
   { /* killough 3/16/98, 12/98: store lump name checksum */
@@ -1786,17 +1792,19 @@ static void G_DoSaveGame (boolean menu)
   // killough 3/16/98: store pwad filenames in savegame
   {
     // CPhipps - changed for new wadfiles handling
-    int i = 0;
-    for (*save_p = 0; (size_t)i<numwadfiles; i++)
+    size_t i;
+    for (i = 0; i<numwadfiles; i++)
       {
-  const char *const w = wadfiles[i].name;
+        const char *const w = wadfiles[i].name;
         CheckSaveGame(strlen(w)+2);
-        strcat(strcat(save_p, w), "\n");
+        strcpy(save_p, w);
+        save_p += strlen(save_p);
+        *save_p++ = '\n';
       }
-    save_p += strlen(save_p)+1;
+    *save_p++ = 0;
   }
 
-  CheckSaveGame(GAME_OPTION_SIZE+MIN_MAXPLAYERS+10);
+  CheckSaveGame(GAME_OPTION_SIZE+MIN_MAXPLAYERS+14);
 
   *save_p++ = compatibility_level;
 
@@ -1882,19 +1890,6 @@ void G_DeferedInitNew(skill_t skill, int episode, int map)
   gameaction = ga_newgame;
 }
 
-extern int variable_friction;
-extern int default_variable_friction;  // ice & mud
-
-extern int weapon_recoil, default_weapon_recoil;    // weapon recoil
-
-extern int allow_pushers;
-extern int default_allow_pushers;     // MT_PUSH Things
-
-extern int player_bobbing;
-extern int default_player_bobbing;    // whether player bobs or not
-
-extern int monsters_remember, default_monsters_remember;
-
 /* cph -
  * G_Compatibility
  *
@@ -1947,7 +1942,7 @@ void G_Compatibility(void)
     prboom_2_compatibility, /* comp_respawn - objects which aren't on the map
                              * at game start respawn at (0,0) */
     boom_compatibility_compatibility,  /* comp_sound - see s_sound.c */
-    ultdoom_compatibility, /* comp_666 - enables tag 666 in non-E1Mx levels */
+    ultdoom_compatibility, /* comp_666 - enables tag 666 in non-ExM8 levels */
     prboom_4_compatibility, /* comp_soul - enables lost souls bouncing (see P_ZMovement */
     doom_1666_compatibility, /* comp_maskedanim - 2s mid textures don't animate */
   };
@@ -1999,7 +1994,10 @@ void G_ReloadDefaults(void)
 
   player_bobbing = default_player_bobbing;  // whether player bobs or not
 
-  variable_friction = allow_pushers = true;
+  /* cph 2007/06/31 - for some reason, the default_* of the next 2 vars was never implemented */
+  variable_friction = default_variable_friction;
+  allow_pushers     = default_allow_pushers;
+
 
   monsters_remember = default_monsters_remember;   // remember former enemies
 
@@ -2045,7 +2043,10 @@ void G_ReloadDefaults(void)
   compatibility_level = default_compatibility_level;
   {
     int i = M_CheckParm("-complevel");
-    if (i && (1+i) < myargc) compatibility_level = atoi(myargv[i+1]);
+    if (i && (1+i) < myargc) {
+      int l = atoi(myargv[i+1]);;
+      if (l >= -1) compatibility_level = l;
+    }
   }
   if (compatibility_level == -1)
     compatibility_level = best_compatibility;
@@ -2100,9 +2101,6 @@ void G_SetFastParms(int fast_pending)
   }
 }
 
-// The sky texture to be used instead of the F_SKY1 dummy.
-extern int skytexture;
-
 //
 // G_InitNew
 // Can be called by the startup code or the menu task,
@@ -2178,6 +2176,8 @@ void G_InitNew(skill_t skill, int episode, int map)
 
 void G_ReadDemoTiccmd (ticcmd_t* cmd)
 {
+  unsigned char at; // e6y: tasdoom stuff
+
   if (*demo_p == DEMOMARKER)
     G_CheckDemoStatus();      // end of demo data stream
   else
@@ -2185,12 +2185,21 @@ void G_ReadDemoTiccmd (ticcmd_t* cmd)
       cmd->forwardmove = ((signed char)*demo_p++);
       cmd->sidemove = ((signed char)*demo_p++);
       if (!longtics) {
-        cmd->angleturn = ((unsigned char)*demo_p++)<<8;
+        cmd->angleturn = ((unsigned char)(at = *demo_p++))<<8;
       } else {
 	unsigned int lowbyte = (unsigned char)*demo_p++;
         cmd->angleturn = (((signed int)(*demo_p++))<<8) + lowbyte;
       }
       cmd->buttons = (unsigned char)*demo_p++;
+      // e6y: ability to play tasdoom demos directly
+      if (compatibility_level == tasdoom_compatibility)
+      {
+        signed char k = cmd->forwardmove;
+        cmd->forwardmove = cmd->sidemove;
+        cmd->sidemove = (signed char)at;
+        cmd->angleturn = ((unsigned char)cmd->buttons)<<8;
+        cmd->buttons = (byte)k;
+      }
     }
 }
 
@@ -2584,8 +2593,14 @@ static int demolumpnum = -1;
 static int G_GetOriginalDoomCompatLevel(int ver)
 {
   {
+    int lev;
     int i = M_CheckParm("-complevel");
-    if (i && (i+1 < myargc)) return atoi(myargv[i+1]);
+    if (i && (i+1 < myargc))
+    {
+      lev = atoi(myargv[i+1]);
+      if (lev>=0)
+        return lev;
+    }
   }
   if (ver < 107) return doom_1666_compatibility;
   if (gamemode == retail) return ultdoom_compatibility;
@@ -2780,6 +2795,7 @@ void G_DoPlayDemo(void)
   usergame = false;
 
   demoplayback = true;
+  R_SmoothPlaying_Reset(NULL); // e6y
 }
 
 //
@@ -2801,6 +2817,8 @@ void G_TimeDemo(const char *name) // CPhipps - const char*
  */
 boolean G_CheckDemoStatus (void)
 {
+  P_ChecksumFinal();
+
   if (demorecording)
     {
       demorecording = false;
