@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -54,6 +56,7 @@
 #include "r_sky.h"
 #include "r_plane.h"
 #include "r_data.h"
+#include "r_things.h"
 #include "p_maputl.h"
 #include "m_bbox.h"
 #include "lprintf.h"
@@ -67,7 +70,6 @@ extern int tran_filter_pct;
 boolean use_fog=false;
 
 int gl_nearclip=5;
-int gl_farclip=6400;
 char *gl_tex_filter_string;
 int gl_tex_filter;
 int gl_mipmap_filter;
@@ -245,6 +247,18 @@ static const float lighttable[5][256] =
 
 #define gld_CalcLightLevel(lightlevel) (lighttable[usegamma][max(min((lightlevel),255),0)])
 
+/*
+// experimental new lighting code
+static float gld_CalcLightLevel(int lightlevel) {
+  if (lightlevel < 192) {
+    lightlevel = lightlevel - ((192 - lightlevel) * 85 / 100);
+  }
+  if (lightlevel < 20)
+    lightlevel = 20;
+  return lightlevel / 255.0;
+}
+*/
+
 static void gld_StaticLightAlpha(float light, float alpha)
 {
   player_t *player;
@@ -258,7 +272,7 @@ static void gld_StaticLightAlpha(float light, float alpha)
 
 #define gld_StaticLight(light) gld_StaticLightAlpha(light, 1.0f)
 
-void gld_InitExtensions(const char *_extensions)
+static void gld_InitExtensions(const char *_extensions)
 {
   char *extensions;
   char *extension;
@@ -292,7 +306,7 @@ void gld_InitExtensions(const char *_extensions)
 	if (gld_ColorTableEXT == NULL)
 	  gl_paletted_texture = false;
 	else
-          lprintf(LO_INFO,"using GL_EXT_paletted_texture\n",glGetString(GL_VENDOR));
+          lprintf(LO_INFO,"using GL_EXT_paletted_texture\n");
       }
     }
     else if (strcasecmp(extension, "GL_EXT_shared_texture_palette") == 0)
@@ -302,7 +316,7 @@ void gld_InitExtensions(const char *_extensions)
 	if (gld_ColorTableEXT == NULL)
 	  gl_shared_texture_palette = false;
 	else
-          lprintf(LO_INFO,"using GL_EXT_shared_texture_palette\n",glGetString(GL_VENDOR));
+          lprintf(LO_INFO,"using GL_EXT_shared_texture_palette\n");
       }
 
     extension = p;
@@ -468,7 +482,7 @@ void gld_Init(int width, int height)
   }
 }
 
-void gld_InitCommandLine()
+void gld_InitCommandLine(void)
 {
 }
 
@@ -518,122 +532,6 @@ void gld_DrawNumPatch(int x, int y, int lump, int cm, enum patch_translation_e f
   glEnd();
 }
 
-void gld_DrawPatchFromMem(int x, int y, const patch_t *patch, int cm, enum patch_translation_e flags)
-{
-  extern int gld_GetTexDimension(int value);
-  extern void gld_AddPatchToTexture(GLTexture *gltexture, unsigned char *buffer, const patch_t *patch, int originx, int originy, int cm, int paletted);
-  extern void gld_SetTexturePalette(GLenum target);
-
-  GLTexture *gltexture;
-  unsigned char *buffer;
-  float fU1,fU2,fV1,fV2;
-  float width,height;
-  float xpos, ypos;
-
-  gltexture=(GLTexture *)Z_Malloc(sizeof(GLTexture),PU_STATIC,0);
-  if (!gltexture)
-    return;
-  gltexture->realtexwidth=SHORT(patch->width);
-  gltexture->realtexheight=SHORT(patch->height);
-  gltexture->leftoffset=SHORT(patch->leftoffset);
-  gltexture->topoffset=SHORT(patch->topoffset);
-  gltexture->tex_width=gld_GetTexDimension(gltexture->realtexwidth);
-  gltexture->tex_height=gld_GetTexDimension(gltexture->realtexheight);
-  gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-  gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
-  gltexture->buffer_width=gltexture->tex_width;
-  gltexture->buffer_height=gltexture->tex_height;
-#ifdef USE_GLU_IMAGESCALE
-  gltexture->width=min(gltexture->realtexwidth, gltexture->tex_width);
-  gltexture->height=min(gltexture->realtexheight, gltexture->tex_height);
-  gltexture->buffer_width=max(gltexture->realtexwidth, gltexture->tex_width);
-  gltexture->buffer_height=max(gltexture->realtexheight, gltexture->tex_height);
-#endif
-  gltexture->buffer_size=gltexture->buffer_width*gltexture->buffer_height*4;
-  if (gltexture->realtexwidth>gltexture->buffer_width)
-    return;
-  if (gltexture->realtexheight>gltexture->buffer_height)
-    return;
-  buffer=(unsigned char*)Z_Malloc(gltexture->buffer_size,PU_STATIC,0);
-  if (gl_paletted_texture)
-    memset(buffer,transparent_pal_index,gltexture->buffer_size);
-  else
-    memset(buffer,0,gltexture->buffer_size);
-  gld_AddPatchToTexture(gltexture, buffer, patch, 0, 0, cm, gl_paletted_texture);
-  glGenTextures(1,&gltexture->glTexID[cm]);
-  glBindTexture(GL_TEXTURE_2D, gltexture->glTexID[cm]);
-  last_gltexture = NULL;
-  last_cm = -1;
-#ifdef USE_GLU_IMAGESCALE
-  if ((gltexture->buffer_width>gltexture->tex_width) ||
-      (gltexture->buffer_height>gltexture->tex_height)
-     )
-  {
-    unsigned char *scaledbuffer;
-
-    scaledbuffer=(unsigned char*)Z_Malloc(gltexture->tex_width*gltexture->tex_height*4,PU_STATIC,0);
-    if (scaledbuffer)
-    {
-      gluScaleImage(GL_RGBA,
-                    gltexture->buffer_width, gltexture->buffer_height,
-                    GL_UNSIGNED_BYTE,buffer,
-                    gltexture->tex_width, gltexture->tex_height,
-                    GL_UNSIGNED_BYTE,scaledbuffer);
-      Z_Free(buffer);
-      buffer=scaledbuffer;
-      glTexImage2D( GL_TEXTURE_2D, 0, gl_tex_format,
-                    gltexture->tex_width, gltexture->tex_height,
-                    0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
-    }
-  }
-  else
-#endif /* USE_GLU_IMAGESCALE */
-  {
-    if (gl_paletted_texture) {
-      gld_SetTexturePalette(GL_TEXTURE_2D);
-      glTexImage2D( GL_TEXTURE_2D, 0, GL_COLOR_INDEX8_EXT,
-                    gltexture->buffer_width, gltexture->buffer_height,
-                    0, GL_COLOR_INDEX, GL_UNSIGNED_BYTE, buffer);
-    } else {
-      glTexImage2D( GL_TEXTURE_2D, 0, gl_tex_format,
-                    gltexture->buffer_width, gltexture->buffer_height,
-                    0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
-    }
-  }
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, gl_tex_filter);
-  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, gl_tex_filter);
-  Z_Free(buffer);
-
-  fV1=0.0f;
-  fV2=(float)gltexture->height/(float)gltexture->tex_height;
-  if (flags & VPT_FLIP)
-  {
-    fU1=(float)gltexture->width/(float)gltexture->tex_width;
-    fU2=0.0f;
-  }
-  else
-  {
-    fU1=0.0f;
-    fU2=(float)gltexture->width/(float)gltexture->tex_width;
-  }
-  xpos=SCALE_X(x-gltexture->leftoffset);
-  ypos=SCALE_Y(y-gltexture->topoffset);
-  width=SCALE_X(gltexture->realtexwidth);
-  height=SCALE_Y(gltexture->realtexheight);
-  glBindTexture(GL_TEXTURE_2D, gltexture->glTexID[cm]);
-  glBegin(GL_TRIANGLE_STRIP);
-    glTexCoord2f(fU1, fV1); glVertex2f((xpos),(ypos));
-    glTexCoord2f(fU1, fV2); glVertex2f((xpos),(ypos+height));
-    glTexCoord2f(fU2, fV1); glVertex2f((xpos+width),(ypos));
-    glTexCoord2f(fU2, fV2); glVertex2f((xpos+width),(ypos+height));
-  glEnd();
-
-  glDeleteTextures(1,&gltexture->glTexID[cm]);
-  Z_Free(gltexture);
-}
-
 #undef SCALE_X
 #undef SCALE_Y
 
@@ -661,7 +559,7 @@ void gld_DrawBackground(const char* name)
   glEnd();
 }
 
-void gld_DrawLine(int x0, int y0, int x1, int y1, byte BaseColor)
+void gld_DrawLine(int x0, int y0, int x1, int y1, int BaseColor)
 {
   const unsigned char *playpal=W_CacheLumpName("PLAYPAL");
 
@@ -830,7 +728,7 @@ void gld_ReadScreen (byte* scr)
   glReadPixels(0,0,SCREENWIDTH,SCREENHEIGHT,GL_RGB,GL_UNSIGNED_BYTE,scr);
 }
 
-GLvoid gld_Set2DMode()
+GLvoid gld_Set2DMode(void)
 {
   glMatrixMode(GL_MODELVIEW);
   glLoadIdentity();
@@ -851,7 +749,7 @@ void gld_InitDrawScene(void)
 {
 }
 
-void gld_Finish()
+void gld_Finish(void)
 {
   gld_Set2DMode();
   glFinish();
@@ -1416,8 +1314,8 @@ static void gld_PrecalculateSector(int num)
   for (i=0; i<sectors[num].linecount; i++)
   {
     lineadded[i]=false;
-    if (sectors[num].lines[i]->sidenum[0]>=0)
-      if (sectors[num].lines[i]->sidenum[1]>=0)
+    if (sectors[num].lines[i]->sidenum[0]!=NO_INDEX)
+      if (sectors[num].lines[i]->sidenum[1]!=NO_INDEX)
         if (sides[sectors[num].lines[i]->sidenum[0]].sector
           ==sides[sectors[num].lines[i]->sidenum[1]].sector)
         {
@@ -1450,7 +1348,7 @@ static void gld_PrecalculateSector(int num)
         {
           currentline=i;
           currentloop++;
-          if ((sectors[num].lines[currentline]->sidenum[0]!=-1) ? (sides[sectors[num].lines[currentline]->sidenum[0]].sector==&sectors[num]) : false)
+          if ((sectors[num].lines[currentline]->sidenum[0]!=NO_INDEX) ? (sides[sectors[num].lines[currentline]->sidenum[0]].sector==&sectors[num]) : false)
             startvertex=sectors[num].lines[currentline]->v1;
           else
             startvertex=sectors[num].lines[currentline]->v2;
@@ -1472,7 +1370,7 @@ static void gld_PrecalculateSector(int num)
     // add current line
     lineadded[currentline]=true;
     // check if currentsector is on the front side of the line ...
-    if ((sectors[num].lines[currentline]->sidenum[0]!=-1) ? (sides[sectors[num].lines[currentline]->sidenum[0]].sector==&sectors[num]) : false)
+    if ((sectors[num].lines[currentline]->sidenum[0]!=NO_INDEX) ? (sides[sectors[num].lines[currentline]->sidenum[0]].sector==&sectors[num]) : false)
     {
       // v2 is ending vertex
       currentvertex=sectors[num].lines[currentline]->v2;
@@ -1516,7 +1414,7 @@ static void gld_PrecalculateSector(int num)
     bestline=-1; // set to start values
     bestlinecount=0;
     // set backsector if there is one
-    if (sectors[num].lines[currentline]->sidenum[1]!=-1)
+    if (sectors[num].lines[currentline]->sidenum[1]!=NO_INDEX)
       backsector=sides[sectors[num].lines[currentline]->sidenum[1]].sector;
     else
       backsector=NULL;
@@ -1527,7 +1425,7 @@ static void gld_PrecalculateSector(int num)
         if ((sectors[num].lines[i]->v1==currentvertex) || (sectors[num].lines[i]->v2==currentvertex))
         {
           // calculate the angle of this best line candidate
-          if ((sectors[num].lines[i]->sidenum[0]!=-1) ? (sides[sectors[num].lines[i]->sidenum[0]].sector==&sectors[num]) : false)
+          if ((sectors[num].lines[i]->sidenum[0]!=NO_INDEX) ? (sides[sectors[num].lines[i]->sidenum[0]].sector==&sectors[num]) : false)
             angle = R_PointToAngle2(sectors[num].lines[i]->v1->x,sectors[num].lines[i]->v1->y,sectors[num].lines[i]->v2->x,sectors[num].lines[i]->v2->y);
           else
             angle = R_PointToAngle2(sectors[num].lines[i]->v2->x,sectors[num].lines[i]->v2->y,sectors[num].lines[i]->v1->x,sectors[num].lines[i]->v1->y);
@@ -1535,7 +1433,7 @@ static void gld_PrecalculateSector(int num)
           if (angle>=180)
             angle=angle-360;
           // check if line is flipped ...
-          if ((sectors[num].lines[i]->sidenum[0]!=-1) ? (sides[sectors[num].lines[i]->sidenum[0]].sector==&sectors[num]) : false)
+          if ((sectors[num].lines[i]->sidenum[0]!=NO_INDEX) ? (sides[sectors[num].lines[i]->sidenum[0]].sector==&sectors[num]) : false)
           {
             // when the line is not flipped and startvertex is not the currentvertex then skip this line
             if (sectors[num].lines[i]->v1!=currentvertex)
@@ -1557,7 +1455,7 @@ static void gld_PrecalculateSector(int num)
           else
             // check if the angle between the current line and this best line candidate is smaller then
             // the angle of the last candidate
-            if (abs(lineangle-angle)<abs(bestangle))
+            if (D_abs(lineangle-angle)<D_abs(bestangle))
             {
               bestline=i;
               bestangle=lineangle-angle;
@@ -1641,16 +1539,16 @@ static void gld_PrepareSectorSpecialEffects(int num)
   sectors[num].no_bottomtextures=true;
   for (i=0; i<sectors[num].linecount; i++)
   {
-    if ( (sectors[num].lines[i]->sidenum[0]>=0) &&
-         (sectors[num].lines[i]->sidenum[1]>=0) )
+    if ( (sectors[num].lines[i]->sidenum[0]!=NO_INDEX) &&
+         (sectors[num].lines[i]->sidenum[1]!=NO_INDEX) )
     {
-      if (sides[sectors[num].lines[i]->sidenum[0]].toptexture!=R_TextureNumForName("-"))
+      if (sides[sectors[num].lines[i]->sidenum[0]].toptexture!=NO_TEXTURE)
         sectors[num].no_toptextures=false;
-      if (sides[sectors[num].lines[i]->sidenum[0]].bottomtexture!=R_TextureNumForName("-"))
+      if (sides[sectors[num].lines[i]->sidenum[0]].bottomtexture!=NO_TEXTURE)
         sectors[num].no_bottomtextures=false;
-      if (sides[sectors[num].lines[i]->sidenum[1]].toptexture!=R_TextureNumForName("-"))
+      if (sides[sectors[num].lines[i]->sidenum[1]].toptexture!=NO_TEXTURE)
         sectors[num].no_toptextures=false;
-      if (sides[sectors[num].lines[i]->sidenum[1]].bottomtexture!=R_TextureNumForName("-"))
+      if (sides[sectors[num].lines[i]->sidenum[1]].bottomtexture!=NO_TEXTURE)
         sectors[num].no_bottomtextures=false;
     }
     else
@@ -1755,13 +1653,13 @@ void gld_PreprocessSectors(void)
       v2num=((int)sectors[i].lines[j]->v2-(int)vertexes)/sizeof(vertex_t);
       if ((v1num>=numvertexes) || (v2num>=numvertexes))
         continue;
-      if (sectors[i].lines[j]->sidenum[0]>=0)
+      if (sectors[i].lines[j]->sidenum[0]!=NO_INDEX)
         if (sides[sectors[i].lines[j]->sidenum[0]].sector==&sectors[i])
         {
           vertexcheck[v1num]|=1;
           vertexcheck[v2num]|=2;
         }
-      if (sectors[i].lines[j]->sidenum[1]>=0)
+      if (sectors[i].lines[j]->sidenum[1]!=NO_INDEX)
         if (sides[sectors[i].lines[j]->sidenum[1]].sector==&sectors[i])
         {
           vertexcheck[v1num]|=2;
@@ -1818,7 +1716,7 @@ static float pitch    = 0.0f;
 
 #define __glPi 3.14159265358979323846
 
-void infinitePerspective(GLdouble fovy, GLdouble aspect, GLdouble znear)
+static void infinitePerspective(GLdouble fovy, GLdouble aspect, GLdouble znear)
 {
 	GLdouble left, right, bottom, top;
 	GLdouble m[16];
@@ -1896,7 +1794,6 @@ void gld_StartDrawScene(void)
   glMatrixMode(GL_PROJECTION);
   glLoadIdentity();
 
-  //gluPerspective(64.0f, 320.0f/200.0f, (float)gl_nearclip/100.0f, (float)gl_farclip/100.0f);
   infinitePerspective(64.0f, 320.0f/200.0f, (float)gl_nearclip/100.0f);
 
   glMatrixMode(GL_MODELVIEW);
@@ -1920,7 +1817,6 @@ void gld_StartDrawScene(void)
 void gld_EndDrawScene(void)
 {
   player_t *player = &players[displayplayer];
-  extern void R_DrawPlayerSprites (void);
 
   glDisable(GL_POLYGON_SMOOTH);
 
@@ -1931,7 +1827,7 @@ void gld_EndDrawScene(void)
   if (viewangleoffset <= 1024<<ANGLETOFINESHIFT ||
     viewangleoffset >=-1024<<ANGLETOFINESHIFT)
   { // don't draw on side views
-    R_DrawPlayerSprites ();
+    R_DrawPlayerSprites();
   }
   if (player->fixedcolormap == 32) {
 		glBlendFunc(GL_ONE_MINUS_DST_COLOR, GL_ZERO);
@@ -2119,7 +2015,15 @@ static void gld_DrawWall(GLWall *wall)
     (w).vb=OV((w),(seg))+((float)(lineheight)/(float)(w).gltexture->buffer_height)\
   )
 
+// e6y: about isskytexture hack
+// The sky in the third episode of Requiem was not drawn.
+// It did not work correctly because the SKY3 has a zero index in the TEXTURE1 table.
+// Textures with a zero (FALSE) index are not displayed in vanilla,
+// AASHITTY in doom2.wad for example.
+// But the sky textures are processed by different code in DOOM,
+// which does not have this bug.
 #define SKYTEXTURE(sky1,sky2)\
+  isskytexture = true;\
   if ((sky1) & PL_SKYFLAT)\
   {\
     const line_t *l = &lines[sky1 & ~PL_SKYFLAT];\
@@ -2145,7 +2049,8 @@ static void gld_DrawWall(GLWall *wall)
     wall.skyyaw=-2.0f*((yaw+90.0f)/90.0f);\
     wall.skyymid = 200.0f/319.5f*((100.0f)/100.0f);\
     wall.flag = GLDWF_SKY;\
-  }
+  }\
+  isskytexture = false;
 
 #define ADDWALL(wall)\
 {\
@@ -2167,6 +2072,7 @@ void gld_AddWall(seg_t *seg)
   sector_t ftempsec; // needed for R_FakeFlat
   sector_t btempsec; // needed for R_FakeFlat
   float lineheight;
+  int rellight = 0;
 
   if (!segrendered)
     return;
@@ -2180,7 +2086,8 @@ void gld_AddWall(seg_t *seg)
     return;
   wall.glseg=&gl_segs[seg->iSegID];
 
-  wall.light=gld_CalcLightLevel(frontsector->lightlevel+(extralight<<5));
+  rellight = seg->linedef->dx==0? +8 : seg->linedef->dy==0 ? -8 : 0;
+  wall.light=gld_CalcLightLevel(frontsector->lightlevel+rellight+(extralight<<5));
   wall.alpha=1.0f;
   wall.gltexture=NULL;
 
@@ -2226,7 +2133,10 @@ void gld_AddWall(seg_t *seg)
     {
       wall.ytop=255.0f;
       if (
-          (backsector->ceilingheight==backsector->floorheight) &&
+          // e6y
+          // Fix for HOM in the starting area on Memento Mori map29 and on map30.
+          // old code: (backsector->ceilingheight==backsector->floorheight) &&
+          (backsector->ceilingheight==backsector->floorheight||(backsector->ceilingheight<=frontsector->floorheight)) &&
           (backsector->ceilingpic==skyflatnum)
          )
       {
@@ -2236,9 +2146,13 @@ void gld_AddWall(seg_t *seg)
       }
       else
       {
-        if ( (texturetranslation[seg->sidedef->toptexture]!=R_TextureNumForName("-")) )
+        if ( (texturetranslation[seg->sidedef->toptexture]!=NO_TEXTURE) )
         {
-          wall.ybottom=(float)frontsector->ceilingheight/MAP_SCALE;
+          // e6y
+          // It corrects some problem with sky, but I do not remember which one
+          // old code: wall.ybottom=(float)frontsector->ceilingheight/MAP_SCALE;
+          wall.ybottom=(float)max(frontsector->ceilingheight,backsector->ceilingheight)/MAP_SCALE;
+
           SKYTEXTURE(frontsector->sky,backsector->sky);
           ADDWALL(&wall);
         }
@@ -2289,11 +2203,45 @@ void gld_AddWall(seg_t *seg)
         ceiling_height=min(seg->frontsector->ceilingheight,seg->backsector->ceilingheight)+(seg->sidedef->rowoffset);
         floor_height=ceiling_height-(wall.gltexture->realtexheight<<FRACBITS);
       }
+      // e6y
+      // The fix for wrong middle texture drawing
+      // if it exceeds the boundaries of its floor and ceiling
+      /*
       CALC_Y_VALUES(wall, lineheight, floor_height, ceiling_height);
       CALC_TEX_VALUES_MIDDLE2S(
         wall, seg, (LINE->flags & ML_DONTPEGBOTTOM)>0,
         segs[seg->iSegID].length, lineheight
       );
+      */
+      {
+        int floormax, ceilingmin, linelen;
+        float mip;
+        mip = (float)wall.gltexture->realtexheight/(float)wall.gltexture->buffer_height;
+//        if ( (texturetranslation[seg->sidedef->bottomtexture]!=R_TextureNumForName("-")) )
+        if (seg->sidedef->bottomtexture)
+          floormax=max(seg->frontsector->floorheight,seg->backsector->floorheight);
+        else
+          floormax=floor_height;
+        if (seg->sidedef->toptexture)
+          ceilingmin=min(seg->frontsector->ceilingheight,seg->backsector->ceilingheight);
+        else
+          ceilingmin=ceiling_height;
+        linelen=abs(ceiling_height-floor_height);
+        wall.ytop=((float)min(ceilingmin, ceiling_height)/(float)MAP_SCALE);
+        wall.ybottom=((float)max(floormax, floor_height)/(float)MAP_SCALE);
+        wall.flag=GLDWF_M2S;
+        wall.ul=OU((wall),(seg))+(0.0f);
+        wall.ur=OU(wall,(seg))+((segs[seg->iSegID].length)/(float)wall.gltexture->buffer_width);
+        if (floormax<=floor_height)
+          wall.vb=1.0f;
+        else
+          wall.vb=mip*((float)(ceiling_height - floormax))/linelen;
+        if (ceilingmin>=ceiling_height)
+          wall.vt=0.0f;
+        else
+          wall.vt=mip*((float)(ceiling_height - ceilingmin))/linelen;
+      }
+
       if (seg->linedef->tranlump >= 0 && general_translucency)
         wall.alpha=(float)tran_filter_pct/100.0f;
       ADDWALL(&wall);
@@ -2317,7 +2265,7 @@ bottomtexture:
       }
       else
       {
-        if ( (texturetranslation[seg->sidedef->bottomtexture]!=R_TextureNumForName("-")) )
+        if ( (texturetranslation[seg->sidedef->bottomtexture]!=NO_TEXTURE) )
         {
           wall.ytop=(float)frontsector->floorheight/MAP_SCALE;
           SKYTEXTURE(frontsector->sky,backsector->sky);
@@ -2621,8 +2569,6 @@ void gld_AddSprite(vissprite_t *vspr)
  *               *
  *****************/
 
-extern int rendered_visplanes, rendered_segs, rendered_vissprites;
-
 void gld_DrawScene(player_t *player)
 {
   int i,j,k,count;
@@ -2730,7 +2676,6 @@ void gld_DrawScene(player_t *player)
 
 void gld_PreprocessLevel(void)
 {
-  void gld_Precache(void);
   if (precache)
     gld_Precache();
   gld_PreprocessSectors();
