@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -60,8 +62,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 
-int broken_pipe;
-
 /* Most of the following has been rewritten by Lee Killough
  *
  * I_GetTime
@@ -113,7 +113,6 @@ void I_Init(void)
 
   {
     /* killough 2/21/98: avoid sound initialization if no sound & no music */
-    extern boolean nomusicparm, nosfxparm;
     if (!(nomusicparm && nosfxparm))
       I_InitSound();
   }
@@ -125,14 +124,6 @@ static void I_SignalHandler(int s)
 {
   char buf[2048];
 
-#ifdef SIGPIPE
-  /* CPhipps - report but don't crash on SIGPIPE */
-  if (s == SIGPIPE) {
-    fprintf(stderr, "Broken pipe\n");
-    broken_pipe = 1;
-    return;
-  }
-#endif
   signal(s,SIG_IGN);  /* Ignore future instances of this signal.*/
 
   strcpy(buf,"Exiting on signal: ");
@@ -234,7 +225,7 @@ static void I_EndDoom(void)
 
   if (lump != -1)
   {
-    const char (*endoom)[2] = (void*)W_CacheLumpNum(lump);
+    const char (*endoom)[2] = (const void*)W_CacheLumpNum(lump);
     int i, l = W_LumpLength(lump) / 2;
 
     /* cph - colour ENDOOM by rain */
@@ -260,10 +251,10 @@ static void I_EndDoom(void)
       {
         if (!(i % 80))
         {
-          /* reset everything when we start a new line */
+          /* reset color but not bold when we start a new line */
           oldbg = -1;
           oldcolor = -1;
-          printf("\e[0m\n");
+          printf("\e[39m\e[49m\n");
         }
         /* foreground color */
         bold = 0;
@@ -302,7 +293,7 @@ static void I_EndDoom(void)
     lprintf(LO_INFO,"\b"); /* hack workaround for extra newline at bottom of screen */
     lprintf(LO_INFO,"\r");
     if (endoom_mode & endoom_nonasciichars)
-      putchar('\017'); /* restore primary charset */
+      printf("%c",'\017'); /* restore primary charset */
 #endif /* _WIN32 */
     W_UnlockLumpNum(lump);
   }
@@ -332,7 +323,7 @@ void I_SafeExit(int rc)
     }
 }
 
-void I_Quit (void)
+static void I_Quit (void)
 {
   if (!has_exited)
     has_exited=1;   /* Prevent infinitely recursive exits -- killough */
@@ -376,6 +367,8 @@ int main(int argc, char **argv)
   lprintf(LO_INFO,"\n");
   PrintVer();
 
+  /* cph - Z_Close must be done after I_Quit, so we register it first. */
+  atexit(Z_Close);
   /*
      killough 1/98:
 
@@ -397,11 +390,7 @@ int main(int argc, char **argv)
   atexit(I_Quit);
 #ifndef _DEBUG
   signal(SIGSEGV, I_SignalHandler);
-#ifdef SIGPIPE
-  signal(SIGPIPE, I_SignalHandler); /* CPhipps - add SIGPIPE, as this is fatal */
-#endif
   signal(SIGTERM, I_SignalHandler);
-  signal(SIGILL,  I_SignalHandler);
   signal(SIGFPE,  I_SignalHandler);
   signal(SIGILL,  I_SignalHandler);
   signal(SIGINT,  I_SignalHandler);  /* killough 3/6/98: allow CTRL-BRK during init */
@@ -409,12 +398,7 @@ int main(int argc, char **argv)
 #endif
 
   /* cphipps - call to video specific startup code */
-  if (!(M_CheckParm("-nodraw") && M_CheckParm("-nosound")))
-    I_PreInitGraphics();
-
-  /* 2/2/98 Stan
-   * Must call this here.  It's required by both netgames and i_video.c.
-   */
+  I_PreInitGraphics();
 
   D_DoomMain ();
   return 0;
