@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2002 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -30,7 +32,6 @@
  *-----------------------------------------------------------------------------*/
 
 #include "doomstat.h"
-#include "c_runcmd.h"
 #include "g_game.h"
 #include "r_data.h"
 #include "p_inter.h"
@@ -41,9 +42,12 @@
 #include "sounds.h"
 #include "dstrings.h"
 #include "r_main.h"
+#include "p_map.h"
 #include "d_deh.h"  // Ty 03/27/98 - externalized strings
+/* cph 2006/07/23 - needs direct access to thinkercap */
+#include "p_tick.h"
 
-#define plyr (&players[consoleplayer])     /* the console player */
+#define plyr (players+consoleplayer)     /* the console player */
 
 //-----------------------------------------------------------------------------
 //
@@ -143,7 +147,7 @@ struct cheat_s cheat[] = {
   {"idbeholds",  "Berserk",           not_net | not_demo,
    cheat_pw,  pw_strength        },
 
-  {"idbeholdi",  "Invisibility",      not_net | not_demo,  
+  {"idbeholdi",  "Invisibility",      not_net | not_demo,
    cheat_pw,  pw_invisibility    },
 
   {"idbeholdr",  "Radiation Suit",    not_net | not_demo,
@@ -179,7 +183,7 @@ struct cheat_s cheat[] = {
   {"tnthom",     NULL,                always,
    cheat_hom      },     // killough 2/07/98: HOM autodetector
 
-  {"tntkey",     NULL,                not_net | not_demo, 
+  {"tntkey",     NULL,                not_net | not_demo,
    cheat_tntkey   },     // killough 2/16/98: generalized key cheats
 
   {"tntkeyr",    NULL,                not_net | not_demo,
@@ -191,13 +195,13 @@ struct cheat_s cheat[] = {
   {"tntkeyb",    NULL,                not_net | not_demo,
    cheat_tntkeyx  },
 
-  {"tntkeyrc",   NULL,                not_net | not_demo, 
+  {"tntkeyrc",   NULL,                not_net | not_demo,
    cheat_tntkeyxx, it_redcard    },
 
   {"tntkeyyc",   NULL,                not_net | not_demo,
    cheat_tntkeyxx, it_yellowcard },
 
-  {"tntkeybc",   NULL,                not_net | not_demo, 
+  {"tntkeybc",   NULL,                not_net | not_demo,
    cheat_tntkeyxx, it_bluecard   },
 
   {"tntkeyrs",   NULL,                not_net | not_demo,
@@ -234,7 +238,7 @@ struct cheat_s cheat[] = {
    cheat_pitch},         // killough 2/21/98: pitched sound toggle
 
   // killough 2/21/98: reduce RSI injury by adding simpler alias sequences:
-  {"tntran",     NULL,                always, 
+  {"tntran",     NULL,                always,
    cheat_tnttran    },   // killough 2/21/98: same as tnttran
 
   {"tntamo",     NULL,                not_net | not_demo,
@@ -249,7 +253,7 @@ struct cheat_s cheat[] = {
   {"tntice",     NULL,                not_net | not_demo,
    cheat_friction   },   // phares 3/10/98: toggle variable friction effects
 
-  {"tntpush",    NULL,                not_net | not_demo, 
+  {"tntpush",    NULL,                not_net | not_demo,
    cheat_pushers    },   // phares 3/10/98: toggle pushers
 
   {NULL}                 // end-of-list marker
@@ -261,38 +265,38 @@ static void cheat_mus(buf)
 char buf[3];
 {
   int musnum;
-  
+
   //jff 3/20/98 note: this cheat allowed in netgame/demorecord
 
   //jff 3/17/98 avoid musnum being negative and crashing
   if (!isdigit(buf[0]) || !isdigit(buf[1]))
     return;
 
-  doom_printf(s_STSTR_MUS); // Ty 03/27/98 - externalized
-  
+  plyr->message = s_STSTR_MUS; // Ty 03/27/98 - externalized
+
   if (gamemode == commercial)
     {
       musnum = mus_runnin + (buf[0]-'0')*10 + buf[1]-'0' - 1;
-          
+
       //jff 4/11/98 prevent IDMUS00 in DOOMII and IDMUS36 or greater
       if (musnum < mus_runnin ||  ((buf[0]-'0')*10 + buf[1]-'0') > 35)
-        doom_printf(s_STSTR_NOMUS); // Ty 03/27/98 - externalized
+        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
       else
         {
-          S_ChangeMusicNum(musnum, 1);
+          S_ChangeMusic(musnum, 1);
           idmusnum = musnum; //jff 3/17/98 remember idmus number for restore
         }
     }
   else
     {
       musnum = mus_e1m1 + (buf[0]-'1')*9 + (buf[1]-'1');
-          
+
       //jff 4/11/98 prevent IDMUS0x IDMUSx0 in DOOMI and greater than introa
       if (buf[0] < '1' || buf[1] < '1' || ((buf[0]-'1')*9 + buf[1]-'1') > 31)
-        doom_printf(s_STSTR_NOMUS); // Ty 03/27/98 - externalized
+        plyr->message = s_STSTR_NOMUS; // Ty 03/27/98 - externalized
       else
         {
-          S_ChangeMusicNum(musnum, 1);
+          S_ChangeMusic(musnum, 1);
           idmusnum = musnum; //jff 3/17/98 remember idmus number for restore
         }
     }
@@ -303,7 +307,7 @@ static void cheat_choppers()
 {
   plyr->weaponowned[wp_chainsaw] = true;
   plyr->powers[pw_invulnerability] = true;
-  doom_printf(s_STSTR_CHOPPERS); // Ty 03/27/98 - externalized
+  plyr->message = s_STSTR_CHOPPERS; // Ty 03/27/98 - externalized
 }
 
 static void cheat_god()
@@ -313,12 +317,12 @@ static void cheat_god()
     {
       if (plyr->mo)
         plyr->mo->health = god_health;  // Ty 03/09/98 - deh
-          
+
       plyr->health = god_health;
-      doom_printf(s_STSTR_DQDON); // Ty 03/27/98 - externalized
+      plyr->message = s_STSTR_DQDON; // Ty 03/27/98 - externalized
     }
-  else 
-    doom_printf(s_STSTR_DQDOFF); // Ty 03/27/98 - externalized
+  else
+    plyr->message = s_STSTR_DQDOFF; // Ty 03/27/98 - externalized
 }
 
 // CPhipps - new health and armour cheat codes
@@ -328,7 +332,7 @@ static void cheat_health()
     if (plyr->mo)
       plyr->mo->health = mega_health;
     plyr->health = mega_health;
-    doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
+    plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
   }
 }
 
@@ -336,7 +340,7 @@ static void cheat_megaarmour()
 {
   plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
   plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
-  doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
+  plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
 }
 
 static void cheat_fa()
@@ -352,18 +356,18 @@ static void cheat_fa()
 
   plyr->armorpoints = idfa_armor;      // Ty 03/09/98 - deh
   plyr->armortype = idfa_armor_class;  // Ty 03/09/98 - deh
-        
+
   // You can't own weapons that aren't in the game // phares 02/27/98
   for (i=0;i<NUMWEAPONS;i++)
     if (!(((i == wp_plasma || i == wp_bfg) && gamemode == shareware) ||
           (i == wp_supershotgun && gamemode != commercial)))
       plyr->weaponowned[i] = true;
-        
+
   for (i=0;i<NUMAMMO;i++)
     if (i!=am_cell || gamemode!=shareware)
       plyr->ammo[i] = plyr->maxammo[i];
 
-  doom_printf(s_STSTR_FAADDED);
+  plyr->message = s_STSTR_FAADDED;
 }
 
 static void cheat_k()
@@ -373,7 +377,7 @@ static void cheat_k()
     if (!plyr->cards[i])     // only print message if at least one key added
       {                      // however, caller may overwrite message anyway
         plyr->cards[i] = true;
-        doom_printf("Keys Added");
+        plyr->message = "Keys Added";
       }
 }
 
@@ -381,7 +385,7 @@ static void cheat_kfa()
 {
   cheat_k();
   cheat_fa();
-  doom_printf(STSTR_KFAADDED);
+  plyr->message = STSTR_KFAADDED;
 }
 
 static void cheat_noclip()
@@ -389,12 +393,12 @@ static void cheat_noclip()
   // Simplified, accepting both "noclip" and "idspispopd".
   // no clipping mode cheat
 
-  doom_printf((plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ? 
-    s_STSTR_NCON : s_STSTR_NCOFF); // Ty 03/27/98 - externalized
+  plyr->message = (plyr->cheats ^= CF_NOCLIP) & CF_NOCLIP ?
+    s_STSTR_NCON : s_STSTR_NCOFF; // Ty 03/27/98 - externalized
 }
 
 // 'behold?' power-up cheats (modified for infinite duration -- killough)
-static void cheat_pw(pw)
+static void cheat_pw(int pw)
 {
   if (plyr->powers[pw])
     plyr->powers[pw] = pw!=pw_strength && pw!=pw_allmap;  // killough
@@ -404,24 +408,23 @@ static void cheat_pw(pw)
       if (pw != pw_strength)
         plyr->powers[pw] = -1;      // infinite duration -- killough
     }
-  doom_printf(s_STSTR_BEHOLDX); // Ty 03/27/98 - externalized
+  plyr->message = s_STSTR_BEHOLDX; // Ty 03/27/98 - externalized
 }
 
 // 'behold' power-up menu
 static void cheat_behold()
 {
-  doom_printf(s_STSTR_BEHOLD); // Ty 03/27/98 - externalized 
+  plyr->message = s_STSTR_BEHOLD; // Ty 03/27/98 - externalized
 }
 
 // 'clev' change-level cheat
-static void cheat_clev(buf)
-char buf[3];
+static void cheat_clev(char buf[3])
 {
   int epsd, map;
 
   if (gamemode == commercial)
     {
-      epsd = 1; //jff was 0, but espd is 1-based 
+      epsd = 1; //jff was 0, but espd is 1-based
       map = (buf[0] - '0')*10 + buf[1] - '0';
     }
   else
@@ -442,16 +445,16 @@ char buf[3];
 
   idmusnum = -1; //jff 3/17/98 revert to normal level music on IDCLEV
 
-  doom_printf(s_STSTR_CLEV); // Ty 03/27/98 - externalized
+  plyr->message = s_STSTR_CLEV; // Ty 03/27/98 - externalized
 
-  G_DeferedInitNewNum(gameskill, epsd, map);
+  G_DeferedInitNew(gameskill, epsd, map);
 }
 
 // 'mypos' for player position
 // killough 2/7/98: simplified using dprintf and made output more user-friendly
 static void cheat_mypos()
 {
-  doom_printf("Position (%d,%d,%d)\tAngle %-.0f", 
+  doom_printf("Position (%d,%d,%d)\tAngle %-.0f",
           players[consoleplayer].mo->x >> FRACBITS,
           players[consoleplayer].mo->y >> FRACBITS,
           players[consoleplayer].mo->z >> FRACBITS,
@@ -470,16 +473,16 @@ static void cheat_comp()
 {
   // CPhipps - modified for new compatibility system
   compatibility_level++; compatibility_level %= MAX_COMPATIBILITY_LEVEL;
-  doom_printf("New compatibility level:\n%s", 
-	      comp_lev_str[compatibility_level]);
+  doom_printf("New compatibility level:\n%s",
+        comp_lev_str[compatibility_level]);
 }
 
 // variable friction cheat
 static void cheat_friction()
 {
-  doom_printf(                      // Ty 03/27/98 - *not* externalized
-    (variable_friction = !variable_friction) ? "Variable Friction enabled" : 
-                                               "Variable Friction disabled");
+  plyr->message =                       // Ty 03/27/98 - *not* externalized
+    (variable_friction = !variable_friction) ? "Variable Friction enabled" :
+                                               "Variable Friction disabled";
 }
 
 
@@ -487,16 +490,16 @@ static void cheat_friction()
 // phares 3/10/98
 static void cheat_pushers()
 {
-  doom_printf(                     // Ty 03/27/98 - *not* externalized
-    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled");
+  plyr->message =                      // Ty 03/27/98 - *not* externalized
+    (allow_pushers = !allow_pushers) ? "Pushers enabled" : "Pushers disabled";
 }
 
 // translucency cheat
 static void cheat_tnttran()
 {
-  doom_printf(                     // Ty 03/27/98 - *not* externalized
+  plyr->message =                      // Ty 03/27/98 - *not* externalized
     (general_translucency = !general_translucency) ? "Translucency enabled" :
-                                                     "Translucency disabled");
+                                                     "Translucency disabled";
 
   // killough 3/1/98, 4/11/98: cache translucency map on a demand basis
   if (general_translucency && !main_tranmap)
@@ -517,10 +520,11 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
 
   // killough 7/20/98: kill friendly monsters only if no others to kill
   uint_64_t mask = MF_FRIEND;
+  P_MapStart();
   do
     while ((currentthinker = P_NextThinker(currentthinker,th_all)) != NULL)
     if (currentthinker->function == P_MobjThinker &&
-	!(((mobj_t *) currentthinker)->flags & mask) && // killough 7/20/98
+  !(((mobj_t *) currentthinker)->flags & mask) && // killough 7/20/98
         (((mobj_t *) currentthinker)->flags & MF_COUNTKILL ||
          ((mobj_t *) currentthinker)->type == MT_SKULL))
       { // killough 3/6/98: kill even if PE is dead
@@ -536,6 +540,7 @@ static void cheat_massacre()    // jff 2/01/98 kill all monsters
           }
       }
   while (!killcount && mask ? mask=0, 1 : 0); // killough 7/20/98
+  P_MapEnd();
   // killough 3/22/98: make more intelligent about plural
   // Ty 03/27/98 - string(s) *not* externalized
   doom_printf("%d Monster%s Killed", killcount, killcount==1 ? "" : "s");
@@ -554,41 +559,41 @@ static void cheat_ddt()
 static void cheat_hom()
 {
   extern int autodetect_hom;           // Ty 03/27/98 - *not* externalized
-  doom_printf((autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
-    "HOM Detection Off");
+  plyr->message = (autodetect_hom = !autodetect_hom) ? "HOM Detection On" :
+    "HOM Detection Off";
 }
 
 // killough 3/6/98: -fast parameter toggle
 static void cheat_fast()
 {
-  doom_printf((fastparm = !fastparm) ? "Fast Monsters On" : 
-    "Fast Monsters Off");  // Ty 03/27/98 - *not* externalized
+  plyr->message = (fastparm = !fastparm) ? "Fast Monsters On" :
+    "Fast Monsters Off";  // Ty 03/27/98 - *not* externalized
   G_SetFastParms(fastparm); // killough 4/10/98: set -fast parameter correctly
 }
 
 // killough 2/16/98: keycard/skullkey cheat functions
 static void cheat_tntkey()
 {
-  doom_printf("Red, Yellow, Blue");  // Ty 03/27/98 - *not* externalized
+  plyr->message = "Red, Yellow, Blue";  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_tntkeyx()
 {
-  doom_printf("Card, Skull");        // Ty 03/27/98 - *not* externalized
+  plyr->message = "Card, Skull";        // Ty 03/27/98 - *not* externalized
 }
 
-static void cheat_tntkeyxx(key)
+static void cheat_tntkeyxx(int key)
 {
-  doom_printf((plyr->cards[key] = !plyr->cards[key]) ? 
-    "Key Added" : "Key Removed");  // Ty 03/27/98 - *not* externalized
+  plyr->message = (plyr->cards[key] = !plyr->cards[key]) ?
+    "Key Added" : "Key Removed";  // Ty 03/27/98 - *not* externalized
 }
 
 // killough 2/16/98: generalized weapon cheats
 
 static void cheat_tntweap()
 {                                   // Ty 03/27/98 - *not* externalized
-  doom_printf(gamemode==commercial ?           // killough 2/28/98
-    "Weapon number 1-9" : "Weapon number 1-8");
+  plyr->message = gamemode==commercial ?           // killough 2/28/98
+    "Weapon number 1-9" : "Weapon number 1-8";
 }
 
 static void cheat_tntweapx(buf)
@@ -605,11 +610,10 @@ char buf[3];
   else
     if (w >= 0 && w < NUMWEAPONS) {
       if ((plyr->weaponowned[w] = !plyr->weaponowned[w]))
-        doom_printf("Weapon Added");  // Ty 03/27/98 - *not* externalized
-      else 
+        plyr->message = "Weapon Added";  // Ty 03/27/98 - *not* externalized
+      else
         {
-          int P_SwitchWeapon(player_t *player);
-          doom_printf("Weapon Removed"); // Ty 03/27/98 - *not* externalized
+          plyr->message = "Weapon Removed"; // Ty 03/27/98 - *not* externalized
           if (w==plyr->readyweapon)         // maybe switch if weapon removed
             plyr->pendingweapon = P_SwitchWeapon(plyr);
         }
@@ -619,7 +623,7 @@ char buf[3];
 // killough 2/16/98: generalized ammo cheats
 static void cheat_tntammo()
 {
-  doom_printf("Ammo 1-4, Backpack");  // Ty 03/27/98 - *not* externalized
+  plyr->message = "Ammo 1-4, Backpack";  // Ty 03/27/98 - *not* externalized
 }
 
 static void cheat_tntammox(buf)
@@ -628,35 +632,33 @@ char buf[1];
   int a = *buf - '1';
   if (*buf == 'b')  // Ty 03/27/98 - strings *not* externalized
     if ((plyr->backpack = !plyr->backpack))
-      for (doom_printf("Backpack Added"),   a=0 ; a<NUMAMMO ; a++)
+      for (plyr->message = "Backpack Added",   a=0 ; a<NUMAMMO ; a++)
         plyr->maxammo[a] <<= 1;
     else
-      for (doom_printf("Backpack Removed"), a=0 ; a<NUMAMMO ; a++)
+      for (plyr->message = "Backpack Removed", a=0 ; a<NUMAMMO ; a++)
         {
           if (plyr->ammo[a] > (plyr->maxammo[a] >>= 1))
             plyr->ammo[a] = plyr->maxammo[a];
         }
   else
     if (a>=0 && a<NUMAMMO)  // Ty 03/27/98 - *not* externalized
-      { // killough 5/5/98: switch plasma and rockets for now -- KLUDGE 
+      { // killough 5/5/98: switch plasma and rockets for now -- KLUDGE
         a = a==am_cell ? am_misl : a==am_misl ? am_cell : a;  // HACK
-        doom_printf((plyr->ammo[a] = !plyr->ammo[a]) ? 
-          plyr->ammo[a] = plyr->maxammo[a], "Ammo Added" : "Ammo Removed");
+        plyr->message = (plyr->ammo[a] = !plyr->ammo[a]) ?
+          plyr->ammo[a] = plyr->maxammo[a], "Ammo Added" : "Ammo Removed";
       }
 }
 
 static void cheat_smart()
 {
-  extern int monsters_remember;  // Ty 03/27/98 - *not* externalized
-  doom_printf((monsters_remember = !monsters_remember) ? 
-    "Smart Monsters Enabled" : "Smart Monsters Disabled");
+  plyr->message = (monsters_remember = !monsters_remember) ?
+    "Smart Monsters Enabled" : "Smart Monsters Disabled";
 }
 
 static void cheat_pitch()
 {
-  extern int pitched_sounds;  // Ty 03/27/98 - *not* externalized
-  doom_printf((pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
-    "Pitch Effects Disabled");
+  plyr->message=(pitched_sounds = !pitched_sounds) ? "Pitch Effects Enabled" :
+    "Pitch Effects Disabled";
 }
 
 //-----------------------------------------------------------------------------
@@ -697,7 +699,8 @@ boolean M_FindCheats(int key)
       for (i=0;cheat[i].cheat;i++)
         {
           uint_64_t c=0, m=0;
-          const unsigned char *p;
+          const char *p;
+
           for (p=cheat[i].cheat; *p; p++)
             {
               unsigned key = tolower(*p)-'a';  // convert to 0-31
@@ -728,7 +731,7 @@ boolean M_FindCheats(int key)
           ret = 1;                        // responder has eaten key
         }
       else
-        if (!matchedbefore)               // allow only one cheat at a time 
+        if (!matchedbefore)               // allow only one cheat at a time
           {
             matchedbefore = ret = 1;      // responder has eaten key
             cheat[i].func(cheat[i].arg);  // call cheat handler
@@ -736,28 +739,3 @@ boolean M_FindCheats(int key)
     }
   return ret;
 }
-
-CONSOLE_COMMAND(noclip, cf_notnet|cf_level)
-{
-	cheat_noclip();
-}
-
-CONSOLE_COMMAND(god, cf_notnet|cf_level)
-{
-	cheat_god();
-}
-
-/*
-CONSOLE_NETCMD(nuke, cf_server|cf_level, netcmd_nuke)
-{
-	cheat_massacre();
-}
-*/
-
-void Cheat_AddCommands()
-{
-   C_AddCommand(god);
-   C_AddCommand(noclip);
-   //C_AddCommand(nuke);
-}
-
