@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -52,6 +54,7 @@
 #include "m_menu.h"
 #include "am_map.h"
 #include "w_wad.h"
+#include "i_system.h"
 #include "i_sound.h"
 #include "i_video.h"
 #include "v_video.h"
@@ -65,89 +68,67 @@
 #include "lprintf.h"
 #include "d_main.h"
 
-//
-// M_DrawText
-// Returns the final X coordinate
-// HU_Init must have been called to init the font
-//
-extern patchnum_t hu_font[HU_FONTSIZE];
+/* cph - disk icon not implemented */
+static inline void I_BeginRead(void) {}
+static inline void I_EndRead(void) {}
 
-int M_DrawText(int x,int y,boolean direct,char* string)
+/*
+ * M_WriteFile
+ *
+ * killough 9/98: rewritten to use stdio and to flash disk icon
+ */
+
+boolean M_WriteFile(char const *name, void *source, int length)
 {
-  int c;
-  int w;
+  FILE *fp;
 
-  while (*string) {
-    c = toupper(*string) - HU_FONTSTART;
-    string++;
-    if (c < 0 || c> HU_FONTSIZE) {
-      x += 4;
-      continue;
+  errno = 0;
+
+  if (!(fp = fopen(name, "wb")))       // Try opening file
+    return 0;                          // Could not open file for writing
+
+  I_BeginRead();                       // Disk icon on
+  length = fwrite(source, 1, length, fp) == (size_t)length;   // Write data
+  fclose(fp);
+  I_EndRead();                         // Disk icon off
+
+  if (!length)                         // Remove partially written file
+    remove(name);
+
+  return length;
+}
+
+/*
+ * M_ReadFile
+ *
+ * killough 9/98: rewritten to use stdio and to flash disk icon
+ */
+
+int M_ReadFile(char const *name, byte **buffer)
+{
+  FILE *fp;
+
+  if ((fp = fopen(name, "rb")))
+    {
+      size_t length;
+
+      I_BeginRead();
+      fseek(fp, 0, SEEK_END);
+      length = ftell(fp);
+      fseek(fp, 0, SEEK_SET);
+      *buffer = Z_Malloc(length, PU_STATIC, 0);
+      if (fread(*buffer, 1, length, fp) == length)
+        {
+          fclose(fp);
+          I_EndRead();
+          return length;
+        }
+      fclose(fp);
     }
 
-    w = SHORT (hu_font[c].width);
-    if (x+w > SCREENWIDTH)
-      break;
-
-    // proff/nicolas 09/20/98 -- changed for hi-res
-    // CPhipps - patch drawing updated, reformatted
-    V_DrawNumPatch(x, y, 0, hu_font[c].lumpnum, CR_DEFAULT, VPT_STRETCH);
-    x+=w;
-  }
-
-  return x;
-}
-
-//
-// M_WriteFile
-//
-
-boolean M_WriteFile(char const* name,void* source,int length)
-{
-  int handle;
-  int count;
-
-  handle = open ( name, O_WRONLY | O_CREAT | O_TRUNC | O_BINARY, 0666);
-
-  if (handle == -1)
-    return false;
-
-  count = write (handle, source, length);
-  close (handle);
-
-  if (count < length) {
-    unlink(name); // CPhipps - no corrupt data files around, they only confuse people.
-    return false;
-  }
-
-  return true;
-}
-
-
-//
-// M_ReadFile
-//
-
-int M_ReadFile(char const* name,byte** buffer)
-{
-  int handle, count, length;
-  struct stat fileinfo;
-  byte   *buf;
-
-  handle = open (name, O_RDONLY | O_BINARY, 0666);
-  if ((handle == -1) || (fstat (handle,&fileinfo) == -1))
-    I_Error ("M_ReadFile: Couldn't read file %s", name);
-
-  length = fileinfo.st_size;
-  buf = Z_Malloc (length, PU_STATIC, NULL);
-  count = read (handle, buf, length);
-  close (handle);
-
-  if (count < length)
-    I_Error ("M_ReadFile: Couldn't read file %s", name);
-
-  *buffer = buf;
-  return length;
+  /* cph 2002/08/10 - this used to return 0 on error, but that's ambiguous,
+   * because we could have a legit 0-length file. So make it -1. */
+  return -1;
 }
 
 //
@@ -163,10 +144,8 @@ extern int mousebforward;
 
 extern int viewwidth;
 extern int viewheight;
-extern int fake_contrast;
 #ifdef GL_DOOM
 extern int gl_nearclip;
-extern int gl_farclip;
 extern int gl_colorbuffer_bits;
 extern int gl_depthbuffer_bits;
 extern char *gl_tex_filter_string;
@@ -177,7 +156,6 @@ extern int gl_use_paletted_texture;
 extern int gl_use_shared_texture_palette;
 extern int gl_sprite_offset;
 #endif
-extern int mouseSensitivity_horiz,mouseSensitivity_vert;  // killough
 
 extern int realtic_clock_rate;         // killough 4/13/98: adjustable timer
 extern int tran_filter_pct;            // killough 2/21/98
@@ -186,16 +164,12 @@ extern int screenblocks;
 extern int showMessages;
 
 #ifndef DJGPP
-const char* musserver_filename;
-const char* sndserver_filename;
-const char* snd_device;
 int         mus_pause_opt; // 0 = kill music, 1 = pause, 2 = continue
 #endif
 
 extern const char* chat_macros[];
 
 extern int endoom_mode;
-int X_opt;
 
 extern const char* S_music_files[]; // cournia
 
@@ -207,7 +181,7 @@ int map_point_coordinates;
 default_t defaults[] =
 {
   {"Misc settings",{NULL},{0},UL,UL,def_none,ss_none},
-  {"default_compatibility_level",{&default_compatibility_level},
+  {"default_compatibility_level",{(int*)&default_compatibility_level},
    {-1},-1,MAX_COMPATIBILITY_LEVEL-1,
    def_int,ss_none}, // compatibility level" - CPhipps
   {"realtic_clock_rate",{&realtic_clock_rate},{100},0,UL,
@@ -255,6 +229,10 @@ default_t defaults[] =
    def_bool, ss_enem, &monster_friction},
   {"help_friends",{&default_help_friends}, {1}, 0, 1,
    def_bool, ss_enem, &help_friends},
+  {"allow_pushers",{&default_allow_pushers},{1},0,1,
+   def_bool,ss_weap, &allow_pushers},
+  {"variable_friction",{&default_variable_friction},{1},0,1,
+   def_bool,ss_weap, &variable_friction},
 #ifdef DOGS
   {"player_helpers",{&default_dogs}, {0}, 0, 3,
    def_bool, ss_enem },
@@ -264,6 +242,7 @@ default_t defaults[] =
    def_bool, ss_enem, &dog_jumping},
 #endif
    /* End of MBF AI extras */
+
   {"sts_always_red",{&sts_always_red},{1},0,1, // no color changes on status bar
    def_bool,ss_stat},
   {"sts_pct_always_gray",{&sts_pct_always_gray},{0},0,1, // 2/23/98 chg default
@@ -315,29 +294,20 @@ default_t defaults[] =
   {"music_volume",{&snd_MusicVolume},{8},0,15, def_int,ss_none},
   {"mus_pause_opt",{&mus_pause_opt},{2},0,2, // CPhipps - music pausing
    def_int, ss_none}, // 0 = kill music when paused, 1 = pause music, 2 = let music continue
-  {"sounddev", {NULL,&snd_device}, {0,"/dev/dsp"},UL,UL,
-   def_str,ss_none}, // sound output device (UNIX)
   {"snd_channels",{&default_numChannels},{8},1,32,
    def_int,ss_none}, // number of audio events simultaneously // killough
 
   {"Video settings",{NULL},{0},UL,UL,def_none,ss_none},
-  // CPhipps - default screensize for targets that support high-res
-#ifndef GL_DOOM
-  {"screen_width",{&desired_screenwidth},{320}, 320, 1600,
-   def_int,ss_none},
-  {"screen_height",{&desired_screenheight},{200},200,1200,
-   def_int,ss_none},
+#if ((defined GL_DOOM) && (defined _MSC_VER))
+  {"videomode",{&default_videomode},{VID_MODEGL}, VID_MODE8, VID_MODEGL, def_int,ss_none},
 #else
-  /* proff - 640x480 for OpenGL */
-  {"screen_width",{&desired_screenwidth},{640}, 320, 1600,
-   def_int,ss_none},
-  {"screen_height",{&desired_screenheight},{480},200,1200,
-   def_int,ss_none},
-  {"gl_sprite_offset",{&gl_sprite_offset},{0}, 0, 5,
-   def_int,ss_none}, // amount to bring items out of floor (GL) Mead 8/13/03
+  {"videomode",{&default_videomode},{VID_MODE8}, VID_MODE8, VID_MODEGL, def_int,ss_none},
 #endif
-  {"fake_contrast",{&fake_contrast},{1},0,1,
-   def_bool,ss_none}, /* cph - allow crappy fake contrast to be disabled */
+  /* 640x480 default resolution */
+  {"screen_width",{&desired_screenwidth},{640}, 320, MAX_SCREENWIDTH,
+   def_int,ss_none},
+  {"screen_height",{&desired_screenheight},{480},200,MAX_SCREENHEIGHT,
+   def_int,ss_none},
   {"use_fullscreen",{&use_fullscreen},{1},0,1, /* proff 21/05/2000 */
    def_bool,ss_none},
 #ifndef DISABLE_DOUBLEBUFFER
@@ -352,15 +322,11 @@ default_t defaults[] =
    def_int,ss_none},
   {"usegamma",{&usegamma},{3},0,4, //jff 3/6/98 fix erroneous upper limit in range
    def_int,ss_none}, // gamma correction level // killough 1/18/98
-  {"X_options",{&X_opt},{0},0,3, // CPhipps - misc X options
-   def_hex,ss_none}, // X options, see l_video_x.c
 
 #ifdef GL_DOOM
   {"OpenGL settings",{NULL},{0},UL,UL,def_none,ss_none},
   {"gl_nearclip",{&gl_nearclip},{5},0,UL,
    def_int,ss_none}, /* near clipping plane pos */
-  {"gl_farclip",{&gl_farclip},{6400},0,UL,
-   def_int,ss_none}, /* far clipping plane pos */
   {"gl_colorbuffer_bits",{&gl_colorbuffer_bits},{16},16,32,
    def_int,ss_none},
   {"gl_depthbuffer_bits",{&gl_depthbuffer_bits},{16},16,32,
@@ -377,6 +343,10 @@ default_t defaults[] =
    def_bool,ss_none},
   {"gl_use_shared_texture_palette",{&gl_use_shared_texture_palette},{0},0,1,
    def_bool,ss_none},
+#ifdef GL_DOOM
+  {"gl_sprite_offset",{&gl_sprite_offset},{0}, 0, 5,
+   def_int,ss_none}, // amount to bring items out of floor (GL) Mead 8/13/03
+#endif
 #endif
 
   {"Mouse settings",{NULL},{0},UL,UL,def_none,ss_none},
@@ -624,6 +594,8 @@ default_t defaults[] =
    def_colour,ss_auto}, // color used for the single player arrow
   {"mapcolor_me",   {&mapcolor_me}, {112},0,255, // green
    def_colour,ss_auto}, // your (player) colour
+  {"mapcolor_enemy",   {&mapcolor_enemy}, {177},0,255,
+   def_colour,ss_auto},
   {"mapcolor_frnd",   {&mapcolor_frnd}, {112},0,255,
    def_colour,ss_auto},
   //jff 3/9/98 add option to not show secrets til after found
@@ -933,8 +905,23 @@ void M_LoadDefaults (void)
   i = M_CheckParm ("-config");
   if (i && i < myargc-1)
     defaultfile = myargv[i+1];
-  else
-    defaultfile = basedefault;
+  else {
+    const char* exedir = I_DoomExeDir();
+    defaultfile = malloc(PATH_MAX+1);
+    /* get config file from same directory as executable */
+#ifdef HAVE_SNPRINTF
+    snprintf((char *)defaultfile, PATH_MAX,
+#else
+    sprintf ((char *)defaultfile,
+#endif
+            "%s%s%sboom.cfg", exedir, HasTrailingSlash(exedir) ? "" : "/", 
+#if ((defined GL_DOOM) && (defined _MSC_VER))
+            "gl"
+#else
+            "pr"
+#endif
+            );
+  }
 
   lprintf (LO_CONFIRM, " default file: %s\n",defaultfile);
 
@@ -1013,6 +1000,51 @@ void M_LoadDefaults (void)
 // CPhipps - nasty but better than nothing
 static boolean screenshot_write_error;
 
+#ifdef HAVE_LIBPNG
+
+#include <png.h>
+
+static void error_fn(png_structp p, png_const_charp s)
+{ screenshot_write_error = true; lprintf(LO_ERROR, "%s\n", s); }
+
+static void WritePNGfile(FILE* fp, const byte* data,
+       const int width, const int height, const byte* palette)
+{
+  png_structp png_ptr;
+  png_infop info_ptr;
+  boolean gl = !palette;
+
+  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, png_error_ptr_NULL, error_fn, NULL);
+  png_set_compression_level(png_ptr, 2);
+  if (png_ptr == NULL) { screenshot_write_error = true; return; }
+  info_ptr = png_create_info_struct(png_ptr);
+  if (info_ptr != NULL) {
+    png_init_io(png_ptr, fp);
+    png_set_IHDR(png_ptr, info_ptr, width, height, 8, !gl ? PNG_COLOR_TYPE_PALETTE : PNG_COLOR_TYPE_RGB, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
+
+    if (palette)
+      png_set_PLTE(png_ptr, info_ptr, (png_colorp)palette, PNG_MAX_PALETTE_LENGTH);
+    {
+      png_time t;
+      png_convert_from_time_t(&t, time(NULL));
+      png_set_tIME(png_ptr, info_ptr, &t);
+    }
+
+    { /* Now write the image header and data */
+      int y;
+
+      png_write_info(png_ptr, info_ptr);
+      for (y = 0; y < height; y++)
+        png_write_row(png_ptr, data + (gl ? height - y - 1 : y)*width*(gl ? 3 : 1));
+    }
+
+    png_write_end(png_ptr, info_ptr);
+  }
+  png_destroy_write_struct(&png_ptr,  png_infopp_NULL);
+}
+
+#else /* HAVE_LIBPNG */
+
 // jff 3/30/98 types and data structures for BMP output of screenshots
 //
 // killough 5/2/98:
@@ -1073,47 +1105,6 @@ static void SafeWrite(const void *data, size_t size, size_t number, FILE *st)
     screenshot_write_error = true; // CPhipps - made non-fatal
 }
 
-#ifndef GL_DOOM
-#ifdef HAVE_LIBPNG
-
-#include <png.h>
-
-static void error_fn(png_structp p, png_const_charp s)
-{ screenshot_write_error = true; lprintf(LO_ERROR, "%s\n", s); }
-
-static void WritePNGfile(FILE* fp, const byte* data,
-       const int width, const int height, const byte* palette)
-{
-  png_structp png_ptr;
-  png_infop info_ptr;
-
-  png_ptr = png_create_write_struct(PNG_LIBPNG_VER_STRING, png_error_ptr_NULL, error_fn, NULL);
-  if (png_ptr == NULL) { screenshot_write_error = true; return; }
-  info_ptr = png_create_info_struct(png_ptr);
-  if (info_ptr != NULL) {
-    png_init_io(png_ptr, fp);
-    png_set_IHDR(png_ptr, info_ptr, width, height, 8, PNG_COLOR_TYPE_PALETTE, PNG_INTERLACE_NONE, PNG_COMPRESSION_TYPE_BASE, PNG_FILTER_TYPE_BASE);
-
-    png_set_PLTE(png_ptr, info_ptr, (png_colorp)palette, PNG_MAX_PALETTE_LENGTH);
-    {
-      png_time t;
-      png_convert_from_time_t(&t, time(NULL));
-      png_set_tIME(png_ptr, info_ptr, &t);
-    }
-
-    { /* Now write the image header and data */
-      int y;
-
-      png_write_info(png_ptr, info_ptr);
-      for (y = 0; y < height; y++)
-	png_write_row(png_ptr, data + y*width);
-    }
-
-    png_write_end(png_ptr, info_ptr);
-  }
-  png_destroy_write_struct(&png_ptr,  png_infopp_NULL);
-}
-#else
 //
 // WriteBMPfile
 // jff 3/30/98 Add capability to write a .BMP file (256 color uncompressed)
@@ -1153,6 +1144,9 @@ static void WriteBMPfile(FILE* st, const byte* data,
   bmih.biClrImportant = LONG(256);
 
   {
+    int gtlump = (W_CheckNumForName)("GAMMATBL",ns_prboom);
+    register const byte * const gtable = (const byte *)W_CacheLumpNum(gtlump) + 256*usegamma;
+
     // write the header
     SafeWrite(&bmfh.bfType,sizeof(bmfh.bfType),1,st);
     SafeWrite(&bmfh.bfSize,sizeof(bmfh.bfSize),1,st);
@@ -1174,23 +1168,22 @@ static void WriteBMPfile(FILE* st, const byte* data,
 
     // write the palette, in blue-green-red order, gamma corrected
     for (i=0;i<768;i+=3) {
-      c=gammatable[usegamma][palette[i+2]];
+      c=gtable[palette[i+2]];
       SafeWrite(&c,sizeof(char),1,st);
-      c=gammatable[usegamma][palette[i+1]];
+      c=gtable[palette[i+1]];
       SafeWrite(&c,sizeof(char),1,st);
-      c=gammatable[usegamma][palette[i+0]];
+      c=gtable[palette[i+0]];
       SafeWrite(&c,sizeof(char),1,st);
       SafeWrite(&zero,sizeof(char),1,st);
     }
 
     for (i = 0 ; i < height ; i++)
       SafeWrite(data+(height-1-i)*width,sizeof(byte),wid,st);
+
+    W_UnlockLumpNum(gtlump);
   }
 }
 
-#endif /* !HAVE_LIBPNG */
-#else /* GL_DOOM */
-
 //
 // WriteTGAfile
 // proff 05/15/2000 Add capability to write a .TGA file (24bit color uncompressed)
@@ -1239,7 +1232,8 @@ static void WriteTGAfile(FILE* st, const byte* data,
     }
   }
 }
-#endif /* GL_DOOM */
+
+#endif /* !HAVE_LIBPNG */
 
 //
 // M_ScreenShot
@@ -1258,12 +1252,10 @@ static void WriteTGAfile(FILE* st, const byte* data,
 
 void M_DoScreenShot (const char* fname)
 {
-  byte       *linear;
+  screeninfo_t screenshot;
   FILE	*fp = fopen(fname,"wb");
-#ifndef GL_DOOM
   const byte *pal;
   int        pplump = W_GetNumForName("PLAYPAL");
-#endif
 
   if (!fp) {
     doom_printf("Error opening %s", fname);
@@ -1271,36 +1263,52 @@ void M_DoScreenShot (const char* fname)
   }
   screenshot_write_error = false;
 
+  if (V_GetMode() == VID_MODEGL) {
+    screenshot.width = screens[0].width;
+    screenshot.height = screens[0].height;
+    screenshot.pitch = screens[0].width*3;
+    screenshot.not_on_heap = false;
+    V_AllocScreen(&screenshot);
+    // munge planar buffer to linear
+    // CPhipps - use a malloc()ed buffer instead of screens[2]
 #ifdef GL_DOOM
-  // munge planar buffer to linear
-  // CPhipps - use a malloc()ed buffer instead of screens[2]
-  gld_ReadScreen(linear = malloc(SCREENWIDTH * SCREENHEIGHT * 3));
-
-  // save the bmp file
-
-  WriteTGAfile
-    (fp, linear, SCREENWIDTH, SCREENHEIGHT);
-#else
-  // munge planar buffer to linear
-  // CPhipps - use a malloc()ed buffer instead of screens[2]
-  I_ReadScreen(linear = malloc(SCREENWIDTH * SCREENHEIGHT));
-
-  // killough 4/18/98: make palette stay around (PU_CACHE could cause crash)
-  pal = W_CacheLumpNum (pplump);
-
-  // save the bmp file
-
-#ifdef HAVE_LIBPNG
-  WritePNGfile(fp, linear, SCREENWIDTH, SCREENHEIGHT, pal + 3*256*st_palette);
-#else
-  WriteBMPfile
-    (fp, linear, SCREENWIDTH, SCREENHEIGHT, pal + 3*256*st_palette);
+    gld_ReadScreen(screenshot.data);
 #endif
 
-  // cph - free the palette
-  W_UnlockLumpNum(pplump);
-#endif
-  free(linear);
+    // save the bmp file
+
+  #ifdef HAVE_LIBPNG
+    WritePNGfile(fp, screenshot.data, SCREENWIDTH, SCREENHEIGHT, NULL);
+  #else
+    WriteTGAfile
+      (fp, screenshot.data, SCREENWIDTH, SCREENHEIGHT);
+  #endif
+  } else {
+    screenshot.width = screens[0].width;
+    screenshot.height = screens[0].height;
+    screenshot.pitch = screens[0].width;
+    screenshot.not_on_heap = false;
+    V_AllocScreen(&screenshot);
+    // munge planar buffer to linear
+    // CPhipps - use a malloc()ed buffer instead of screens[2]
+    I_ReadScreen(&screenshot);
+
+    // killough 4/18/98: make palette stay around (PU_CACHE could cause crash)
+    pal = W_CacheLumpNum (pplump);
+
+    // save the bmp file
+
+  #ifdef HAVE_LIBPNG
+    WritePNGfile(fp, screenshot.data, SCREENWIDTH, SCREENHEIGHT, pal + 3*256*st_palette);
+  #else
+    WriteBMPfile
+      (fp, screenshot.data, SCREENWIDTH, SCREENHEIGHT, pal + 3*256*st_palette);
+  #endif
+
+    // cph - free the palette
+    W_UnlockLumpNum(pplump);
+  }
+  V_FreeScreen(&screenshot);
   // 1/18/98 killough: replace "SCREEN SHOT" acknowledgement with sfx
 
   if (screenshot_write_error)
@@ -1320,26 +1328,22 @@ void M_ScreenShot(void)
 
   startshot = shot; // CPhipps - prevent infinite loop
 
-  do
-#ifdef GL_DOOM
-    sprintf(lbmname,"doom%02d.TGA", shot++);
-#else
+  do {
 #ifdef HAVE_LIBPNG
     sprintf(lbmname,"doom%02d.png", shot++);
 #else
-    sprintf(lbmname,"doom%02d.bmp", shot++);
+    if (V_GetMode() == VID_MODEGL) {
+      sprintf(lbmname,"doom%02d.tga", shot++);
+    } else {
+      sprintf(lbmname,"doom%02d.bmp", shot++);
+    }
 #endif
-#endif
-  while (!access(lbmname,0) && (shot != startshot) && (shot < 10000));
+  } while (!access(lbmname,0) && (shot != startshot) && (shot < 10000));
 
   if (!access(lbmname,0)) screenshot_write_error = true;
 
   if (screenshot_write_error) {
-#ifdef GL_DOOM
-    doom_printf ("M_ScreenShot: Couldn't create a TGA");
-#else
-    doom_printf ("M_ScreenShot: Couldn't create a BMP");
-#endif
+    doom_printf ("M_ScreenShot: Couldn't create screenshot");
     // killough 4/18/98
     return;
   }
