@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2001 by
+ *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,17 +23,15 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
- * DESCRIPTION: 
+ * DESCRIPTION:
  *   Door animation code (opening/closing)
  *
  *-----------------------------------------------------------------------------*/
 
-#include "z_zone.h"
 #include "doomstat.h"
-#include "g_game.h"
 #include "p_spec.h"
 #include "p_tick.h"
 #include "s_sound.h"
@@ -39,6 +39,7 @@
 #include "r_main.h"
 #include "dstrings.h"
 #include "d_deh.h"  // Ty 03/27/98 - externalized
+#include "lprintf.h"
 
 ///////////////////////////////////////////////////////////////
 //
@@ -49,11 +50,11 @@
 //
 // T_VerticalDoor
 //
-// Passed a door structure containing all info about the door. 
+// Passed a door structure containing all info about the door.
 // See P_SPEC.H for fields.
 // Returns nothing.
 //
-// jff 02/08/98 all cases with labels beginning with gen added to support 
+// jff 02/08/98 all cases with labels beginning with gen added to support
 // generalized line type behaviors.
 
 void T_VerticalDoor (vldoor_t* door)
@@ -63,7 +64,7 @@ void T_VerticalDoor (vldoor_t* door)
   // Is the door waiting, going up, or going down?
   switch(door->direction)
   {
-    case plat_stop:
+    case 0:
       // Door is waiting
       if (!--door->topcountdown)  // downcount and check
       {
@@ -71,24 +72,24 @@ void T_VerticalDoor (vldoor_t* door)
         {
           case blazeRaise:
           case genBlazeRaise:
-            door->direction = plat_down; // time to go back down
+            door->direction = -1; // time to go back down
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
             break;
 
           case normal:
           case genRaise:
-            door->direction = plat_down; // time to go back down
+            door->direction = -1; // time to go back down
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
             break;
 
           case close30ThenOpen:
           case genCdO:
-            door->direction = plat_up;  // time to go back up
+            door->direction = 1;  // time to go back up
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
 
           case genBlazeCdO:
-            door->direction = plat_up;  // time to go back up
+            door->direction = 1;  // time to go back up
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
             break;
 
@@ -105,7 +106,7 @@ void T_VerticalDoor (vldoor_t* door)
         switch(door->type)
         {
           case raiseIn5Mins:
-            door->direction = plat_up;  // time to raise then
+            door->direction = 1;  // time to raise then
             door->type = normal;  // door acts just like normal 1 DR door now
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
@@ -116,7 +117,7 @@ void T_VerticalDoor (vldoor_t* door)
       }
       break;
 
-    case plat_down:
+    case -1:
       // Door is moving down
       res = T_MovePlane
             (
@@ -129,7 +130,10 @@ void T_VerticalDoor (vldoor_t* door)
             );
 
       /* killough 10/98: implement gradual lighting effects */
-      if (door->lighttag && door->topheight - door->sector->floorheight)
+      // e6y: "Tagged doors don't trigger special lighting" handled wrong
+      // http://sourceforge.net/tracker/index.php?func=detail&aid=1411400&group_id=148658&atid=772943
+      // Old code: if (door->lighttag && door->topheight - door->sector->floorheight)
+      if (door->lighttag && door->topheight - door->sector->floorheight && compatibility_level >= mbf_compatibility)
         EV_LightTurnOnPartway(door->line,
                               FixedDiv(door->sector->ceilingheight -
                                        door->sector->floorheight,
@@ -146,7 +150,7 @@ void T_VerticalDoor (vldoor_t* door)
           case blazeClose:
           case genBlazeRaise:
           case genBlazeClose:
-            door->sector->ceilingdata = NULL;  //jff 2/22/98 
+            door->sector->ceilingdata = NULL;  //jff 2/22/98
             P_RemoveThinker (&door->thinker);  // unlink and free
             // killough 4/15/98: remove double-closing sound of blazing doors
             if (comp[comp_blazing])
@@ -163,19 +167,23 @@ void T_VerticalDoor (vldoor_t* door)
 
           // close then open doors start waiting
           case close30ThenOpen:
-            door->direction = plat_stop;
+            door->direction = 0;
             door->topcountdown = TICRATE*30;
             break;
 
           case genCdO:
           case genBlazeCdO:
-            door->direction = plat_stop;
+            door->direction = 0;
             door->topcountdown = door->topwait; // jff 5/8/98 insert delay
             break;
 
           default:
             break;
         }
+        // e6y: "Tagged doors don't trigger special lighting" handled wrong
+        // http://sourceforge.net/tracker/index.php?func=detail&aid=1411400&group_id=148658&atid=772943
+        if (door->lighttag && door->topheight - door->sector->floorheight && compatibility_level < mbf_compatibility)
+          EV_LightTurnOnPartway(door->line,0);
       }
       /* jff 1/31/98 turn lighting off in tagged sectors of manual doors
        * killough 10/98: replaced with gradual lighting code
@@ -190,15 +198,23 @@ void T_VerticalDoor (vldoor_t* door)
           case close:          // Close types do not bounce, merely wait
             break;
 
+          case blazeRaise:
+          case genBlazeRaise:
+            door->direction = 1;
+	    if (!comp[comp_blazing]) {
+	      S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
+	      break;
+	    }
+
           default:             // other types bounce off the obstruction
-              door->direction = plat_up;
+            door->direction = 1;
             S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
             break;
         }
       }
       break;
 
-    case plat_up:
+    case 1:
       // Door is moving up
       res = T_MovePlane
             (
@@ -211,7 +227,10 @@ void T_VerticalDoor (vldoor_t* door)
             );
 
       /* killough 10/98: implement gradual lighting effects */
-      if (door->lighttag && door->topheight - door->sector->floorheight)
+      // e6y: "Tagged doors don't trigger special lighting" handled wrong
+      // http://sourceforge.net/tracker/index.php?func=detail&aid=1411400&group_id=148658&atid=772943
+      // Old code: if (door->lighttag && door->topheight - door->sector->floorheight)
+      if (door->lighttag && door->topheight - door->sector->floorheight && compatibility_level >= mbf_compatibility)
         EV_LightTurnOnPartway(door->line,
                               FixedDiv(door->sector->ceilingheight -
                                        door->sector->floorheight,
@@ -227,7 +246,7 @@ void T_VerticalDoor (vldoor_t* door)
           case normal:
           case genRaise:
           case genBlazeRaise:
-            door->direction = plat_stop; // wait at top with delay
+            door->direction = 0; // wait at top with delay
             door->topcountdown = door->topwait;
             break;
 
@@ -247,7 +266,11 @@ void T_VerticalDoor (vldoor_t* door)
         }
 
         /* jff 1/31/98 turn lighting on in tagged sectors of manual doors
-	 * killough 10/98: replaced with gradual lighting code */
+   * killough 10/98: replaced with gradual lighting code */
+        // e6y: "Tagged doors don't trigger special lighting" handled wrong
+        // http://sourceforge.net/tracker/index.php?func=detail&aid=1411400&group_id=148658&atid=772943
+        if (door->lighttag && door->topheight - door->sector->floorheight && compatibility_level < mbf_compatibility)
+          EV_LightTurnOnPartway(door->line,FRACUNIT);
       }
       break;
   }
@@ -264,15 +287,20 @@ void T_VerticalDoor (vldoor_t* door)
 //
 // Handle opening a tagged locked door
 //
-// Passed the line activating the door, the type of door, 
+// Passed the line activating the door, the type of door,
 // and the thing that activated the line
 // Returns true if a thinker created
 //
-int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
+int EV_DoLockedDoor
+( line_t* line,
+  vldoor_e  type,
+  mobj_t* thing )
 {
-  player_t *p = thing->player;
+  player_t* p;
 
-  if (!p)          // only players can open locked doors
+  // only players can open locked doors
+  p = thing->player;
+  if (!p)
     return 0;
 
   // check type of linedef, and if key is possessed to open it
@@ -282,8 +310,7 @@ int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
     case 133:
       if (!p->cards[it_bluecard] && !p->cards[it_blueskull])
       {
-                //sf: player_printf
-          player_printf(p, s_PD_BLUEO);       // Ty 03/27/98 - externalized
+        p->message = s_PD_BLUEO;             // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
@@ -293,7 +320,7 @@ int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
     case 135:
       if (!p->cards[it_redcard] && !p->cards[it_redskull])
       {
-          player_printf(p, s_PD_REDO);             // Ty 03/27/98 - externalized
+        p->message = s_PD_REDO;              // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
@@ -303,11 +330,11 @@ int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
     case 137:
       if (!p->cards[it_yellowcard] && !p->cards[it_yellowskull])
       {
-          player_printf(p, s_PD_YELLOWO);             // Ty 03/27/98 - externalized
+        p->message = s_PD_YELLOWO;           // Ty 03/27/98 - externalized
         S_StartSound(p->mo,sfx_oof);         // killough 3/20/98
         return 0;
       }
-      break;  
+      break;
   }
 
   // got the key, so open the door
@@ -320,16 +347,20 @@ int EV_DoLockedDoor(line_t *line, vldoor_e type, mobj_t *thing)
 //
 // Handle opening a tagged door
 //
-// Passed the line activating the door and the type of door 
+// Passed the line activating the door and the type of door
 // Returns true if a thinker created
 //
-
-int EV_DoDoor(line_t *line, vldoor_e type)
+int EV_DoDoor
+( line_t* line,
+  vldoor_e  type )
 {
-  int secnum = -1, rtn = 0;
+  int   secnum,rtn;
   sector_t* sec;
   vldoor_t* door;
 
+  secnum = -1;
+  rtn = 0;
+
   // open all doors with the same tag as the activating line
   while ((secnum = P_FindSectorFromLineTag(line,secnum)) >= 0)
   {
@@ -337,7 +368,7 @@ int EV_DoDoor(line_t *line, vldoor_e type)
     // if the ceiling already moving, don't start the door action
     if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
         continue;
- 
+
     // new door thinker
     rtn = 1;
     door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
@@ -351,50 +382,50 @@ int EV_DoDoor(line_t *line, vldoor_e type)
     door->speed = VDOORSPEED;
     door->line = line; // jff 1/31/98 remember line that triggered us
     door->lighttag = 0; /* killough 10/98: no light effects with tagged doors */
-    
+
     // setup door parameters according to type of door
     switch(type)
     {
       case blazeClose:
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
-          door->direction = plat_down;
+        door->direction = -1;
         door->speed = VDOORSPEED * 4;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdcls);
         break;
-      
+
       case close:
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
-          door->direction = plat_down;
+        door->direction = -1;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
         break;
-      
+
       case close30ThenOpen:
         door->topheight = sec->ceilingheight;
-          door->direction = plat_down;
+        door->direction = -1;
         S_StartSound((mobj_t *)&door->sector->soundorg,sfx_dorcls);
         break;
-      
+
       case blazeRaise:
       case blazeOpen:
-          door->direction = plat_up;
+        door->direction = 1;
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         door->speed = VDOORSPEED * 4;
         if (door->topheight != sec->ceilingheight)
           S_StartSound((mobj_t *)&door->sector->soundorg,sfx_bdopn);
         break;
-      
+
       case normal:
       case open:
-          door->direction = plat_up;
+        door->direction = 1;
         door->topheight = P_FindLowestCeilingSurrounding(sec);
         door->topheight -= 4*FRACUNIT;
         if (door->topheight != sec->ceilingheight)
           S_StartSound((mobj_t *)&door->sector->soundorg,sfx_doropn);
         break;
-      
+
       default:
         break;
     }
@@ -402,120 +433,9 @@ int EV_DoDoor(line_t *line, vldoor_e type)
   return rtn;
 }
 
-//
-// EV_OpenDoor
-//
-// sf: for FraggleScript functions
-// allows greater control over how the door behaves
-//
-
-void EV_OpenDoor(int sectag, int speed, int wait_time)
-{
-  vldoor_e door_type;
-  int secnum = -1;
-  vldoor_t *door;
-
-  if(speed < 1) speed = 1;
-  
-  // find out door type first
-
-  if(wait_time)               // door closes afterward
-    {
-      if(speed >= 4)              // blazing ?
-	door_type = blazeRaise;
-      else
-	door_type = normal;
-    }
-  else
-    {
-      if(speed >= 4)              // blazing ?
-	door_type = blazeOpen;
-      else
-	door_type = open;
-    }
-
-  // open door in all the sectors with the specified tag
-
-  while ((secnum = P_FindSectorFromTag(sectag, secnum)) >= 0)
-    {
-      sector_t *sec = &sectors[secnum];
-      // if the ceiling already moving, don't start the door action
-      if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
-        continue;
-
-      // new door thinker
-      door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
-      P_AddThinker(&door->thinker);
-      sec->ceilingdata = door;
-
-      door->thinker.function = T_VerticalDoor;
-      door->sector = sec;
-      door->type = door_type;
-      door->topwait = wait_time;
-      door->speed = VDOORSPEED * speed;
-      door->line = NULL;   // not triggered by a line
-      door->lighttag = 0;  // no lighting effect
-      door->topheight = P_FindLowestCeilingSurrounding(sec) - 4*FRACUNIT;
-      door->direction = plat_up;
-
-      if (door->topheight != sec->ceilingheight)
-	S_StartSound((mobj_t *)&door->sector->soundorg,
-		     speed >= 4 ? sfx_bdopn : sfx_doropn);
-    }
-}
 
 //
-// EV_CloseDoor
-//
-// sf: also for FraggleScript functions
-//
-
-void EV_CloseDoor(int sectag, int speed)
-{
-  vldoor_e door_type;
-  int secnum = -1;
-  vldoor_t *door;
-
-  if(speed < 1) speed = 1;
-  
-  // find out door type first
-
-  if(speed >= 4)              // blazing ?
-    door_type = blazeClose;
-  else
-    door_type = close;
-  
-  // open door in all the sectors with the specified tag
-
-  while ((secnum = P_FindSectorFromTag(sectag, secnum)) >= 0)
-    {
-      sector_t *sec = &sectors[secnum];
-      // if the ceiling already moving, don't start the door action
-      if (P_SectorActive(ceiling_special,sec)) //jff 2/22/98
-        continue;
-
-      // new door thinker
-      door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
-      P_AddThinker(&door->thinker);
-      sec->ceilingdata = door;
-
-      door->thinker.function = T_VerticalDoor;
-      door->sector = sec;
-      door->type = door_type;
-      door->speed = VDOORSPEED * speed;
-      door->line = NULL;   // not triggered by a line
-      door->lighttag = 0;  // no lighting effect
-      door->topheight = P_FindLowestCeilingSurrounding(sec) - 4*FRACUNIT;
-      door->direction = plat_down;
-
-      S_StartSound((mobj_t *)&door->sector->soundorg,
-		   speed >= 4 ? sfx_bdcls : sfx_dorcls);
-    }  
-}
-
-
-//
-// EV_VerticalDoor 
+// EV_VerticalDoor
 //
 // Handle opening a door manually, no tag value
 //
@@ -524,8 +444,9 @@ void EV_CloseDoor(int sectag, int speed)
 //
 // jff 2/12/98 added int return value, fixed all returns
 //
-
-int EV_VerticalDoor(line_t *line, mobj_t *thing)
+int EV_VerticalDoor
+( line_t* line,
+  mobj_t* thing )
 {
   player_t* player;
   int   secnum;
@@ -534,7 +455,7 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
 
   //  Check for locks
   player = thing->player;
-  
+
   switch(line->special)
   {
     case 26: // Blue Lock
@@ -543,31 +464,31 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
         return 0;
       if (!player->cards[it_bluecard] && !player->cards[it_blueskull])
       {
-          player_printf(player, s_PD_BLUEK);             // Ty 03/27/98 - externalized
+          player->message = s_PD_BLUEK;         // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
-  
+
     case 27: // Yellow Lock
     case 34:
       if ( !player )
           return 0;
       if (!player->cards[it_yellowcard] && !player->cards[it_yellowskull])
       {
-          player_printf(player, s_PD_YELLOWK);             // Ty 03/27/98 - externalized
+          player->message = s_PD_YELLOWK;       // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
       break;
-  
+
     case 28: // Red Lock
     case 33:
       if ( !player )
           return 0;
       if (!player->cards[it_redcard] && !player->cards[it_redskull])
       {
-          player_printf(player, s_PD_REDK);             // Ty 03/27/98 - externalized
+          player->message = s_PD_REDK;          // Ty 03/27/98 - externalized
           S_StartSound(player->mo,sfx_oof);     // killough 3/20/98
           return 0;
       }
@@ -576,9 +497,9 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
     default:
       break;
   }
-  
+
   // if the wrong side of door is pushed, give oof sound
-  if (line->sidenum[1]==-1)                     // killough
+  if (line->sidenum[1]==NO_INDEX)                     // killough
   {
     S_StartSound(player->mo,sfx_oof);           // killough 3/20/98
     return 0;
@@ -589,8 +510,11 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
   secnum = sec-sectors;
 
   /* if door already has a thinker, use it
-   * cph 2001/04/05 - 
-   * Original Doom didn't distinguish floor/lighting/ceiling 
+   * cph 2001/04/05 -
+   * Ok, this is a disaster area. We're assuming that sec->ceilingdata
+   *  is a vldoor_t! What if this door is controlled by both DR lines
+   *  and by switches? I don't know how to fix that.
+   * Secondly, original Doom didn't distinguish floor/lighting/ceiling
    *  actions, so we need to do the same in demo compatibility mode.
    */
   door = sec->ceilingdata;
@@ -598,21 +522,46 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
     if (!door) door = sec->floordata;
     if (!door) door = sec->lightingdata;
   }
-  if (door) {
-    /* If the current action is a T_VerticalDoor and we're back in
-     * EV_VerticalDoor, it must have been a repeatable line, so I've dropped
-     * that check. For old demos we have to emulate the old buggy behavior and
+  /* If this is a repeatable line, and the door is already moving, then we can just reverse the current action. Note that in prboom 2.3.0 I erroneously removed the if-this-is-repeatable check, hence the prboom_4_compatibility clause below (foolishly assumed that already moving implies repeatable - but it could be moving due to another switch, e.g. lv19-509) */
+  if (door &&
+	  ((compatibility_level == prboom_4_compatibility) ||
+	   (line->special == 1) || (line->special == 117) || (line->special == 26) || (line->special == 27) || (line->special == 28)
+	  )
+     ) {
+    /* For old demos we have to emulate the old buggy behavior and
      * mess up non-T_VerticalDoor actions.
      */
     if (compatibility_level < prboom_4_compatibility || 
         door->thinker.function == T_VerticalDoor) {
+      /* cph - we are writing outval to door->direction iff it is non-zero */
+      signed int outval = 0;
+
       /* An already moving repeatable door which is being re-pressed, or a
        * monster is trying to open a closing door - so change direction
+       * DEMOSYNC: we only read door->direction now if it really is a door.
        */
-      if (door->direction == -1) {
-        door->direction = 1; return 1; /* go back up */
+      if (door->thinker.function == T_VerticalDoor && door->direction == -1) {
+        outval = 1; /* go back up */
       } else if (player) {
-        door->direction = -1; return 1; /* go back down */
+        outval = -1; /* go back down */
+      }
+
+      /* Write this to the thinker. In demo compatibility mode, we might be 
+       *  overwriting a field of a non-vldoor_t thinker - we need to add any 
+       *  other thinker types here if any demos depend on specific fields
+       *  being corrupted by this.
+       */
+      if (outval) {
+        if (door->thinker.function == T_VerticalDoor) {
+          door->direction = outval;
+        } else if (door->thinker.function == T_PlatRaise) {
+          plat_t* p = (plat_t*)door;
+          p->wait = outval;
+        } else {
+          lprintf(LO_DEBUG, "EV_VerticalDoor: unknown thinker.function in thinker corruption emulation");
+        }
+
+        return 1;
       }
     }
     /* Either we're in prboom >=v2.3 and it's not a door, or it's a door but
@@ -633,14 +582,14 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
       S_StartSound((mobj_t *)&sec->soundorg,sfx_doropn);
       break;
   }
-  
+
   // new door thinker
   door = Z_Malloc (sizeof(*door), PU_LEVSPEC, 0);
   P_AddThinker (&door->thinker);
   sec->ceilingdata = door; //jff 2/22/98
   door->thinker.function = T_VerticalDoor;
   door->sector = sec;
-  door->direction = plat_up;
+  door->direction = 1;
   door->speed = VDOORSPEED;
   door->topwait = VDOORWAIT;
   door->line = line; // jff 1/31/98 remember line that triggered us
@@ -680,7 +629,7 @@ int EV_VerticalDoor(line_t *line, mobj_t *thing)
       door->lighttag = 0;   // killough 10/98
       break;
   }
-  
+
   // find the top and bottom of the movement range
   door->topheight = P_FindLowestCeilingSurrounding(sec);
   door->topheight -= 4*FRACUNIT;
@@ -715,7 +664,7 @@ void P_SpawnDoorCloseIn30 (sector_t* sec)
 
   door->thinker.function = T_VerticalDoor;
   door->sector = sec;
-  door->direction = plat_stop;
+  door->direction = 0;
   door->type = normal;
   door->speed = VDOORSPEED;
   door->topcountdown = 30 * 35;
@@ -731,13 +680,14 @@ void P_SpawnDoorCloseIn30 (sector_t* sec)
 // Passed the sector of the door, whose type specified the door action
 // Returns nothing
 //
-
-void P_SpawnDoorRaiseIn5Mins(sector_t *sec, int secnum)
+void P_SpawnDoorRaiseIn5Mins
+( sector_t* sec,
+  int   secnum )
 {
   vldoor_t* door;
 
   door = Z_Malloc ( sizeof(*door), PU_LEVSPEC, 0);
-  
+
   P_AddThinker (&door->thinker);
 
   sec->ceilingdata = door; //jff 2/22/98
