@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2006 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -415,7 +417,9 @@ boolean PIT_CheckLine (line_t* ld)
 	spechit = realloc(spechit,sizeof *spechit*spechit_max); // killough
       }
       spechit[numspechit++] = ld;
-      if (numspechit >= 8 && demo_compatibility) SpechitOverrun(ld); // e6y: Spechits overrun emulation code
+      // e6y: Spechits overrun emulation code
+      if (numspechit >= 8 && demo_compatibility)
+        SpechitOverrun(ld);
     }
 
   return true;
@@ -1010,9 +1014,7 @@ boolean P_ThingHeightClip (mobj_t* thing)
 
 /* killough 8/2/98: make variables static */
 static fixed_t   bestslidefrac;
-static fixed_t   secondslidefrac;
 static line_t*   bestslideline;
-static line_t*   secondslideline;
 static mobj_t*   slidemo;
 static fixed_t   tmxmove;
 static fixed_t   tmymove;
@@ -1166,8 +1168,6 @@ isblocking:
 
   if (in->frac < bestslidefrac)
     {
-    secondslidefrac = bestslidefrac;
-    secondslideline = bestslideline;
     bestslidefrac = in->frac;
     bestslideline = li;
     }
@@ -2215,27 +2215,57 @@ void P_MapEnd(void) {
 // http://www.doomworld.com/vb/showthread.php?s=&threadid=35214
 static void SpechitOverrun(line_t *ld)
 {
-  int addr = 0x01C09C98 + (ld - lines) * 0x3E;
+  //int addr = 0x01C09C98 + (ld - lines) * 0x3E;
+  int addr = 0x00C09C98 + (ld - lines) * 0x3E;
 
-  switch(numspechit)
+  if (compatibility_level == dosdoom_compatibility || compatibility_level == tasdoom_compatibility)
   {
-    case 8: break; /* numspechit, not significant it seems - cph */
+    // e6y
+    // There are no more desyncs in the following dosdoom demos: 
+    // flsofdth.wad\fod3uv.lmp - http://www.doomworld.com/sda/flsofdth.htm
+    // hr.wad\hf181430.lmp - http://www.doomworld.com/tas/hf181430.zip
+    // hr.wad\hr181329.lmp - http://www.doomworld.com/tas/hr181329.zip
+    // icarus.wad\ic09uv.lmp - http://competn.doom2.net/pub/sda/i-o/icuvlmps.zip
+
+    switch(numspechit)
+    {
+    case 8: break; /* strange cph's code */
     case 9: 
+      tmfloorz = addr;
+      break;
     case 10:
-    case 11:
-    case 12:
-      tmbbox[numspechit-9] = addr;
-      break;
-    case 13: 
-      crushchange = addr; 
-      break;
-    case 14: 
-      nofit = addr; 
+      tmceilingz = addr;
       break;
+      
     default:
-      fprintf(stderr, "SpechitOverrun: Warning: unable to emulate"
-                      " an overrun where numspechit=%i\n",
-                      numspechit);
+        lprintf(LO_ERROR, "SpechitOverrun: Warning: unable to emulate"
+                          " an overrun where numspechit=%i\n",
+                          numspechit);
       break;
+    }
+  }
+  else
+  {
+    switch(numspechit)
+    {
+      case 8: break; /* numspechit, not significant it seems - cph */
+      case 9: 
+      case 10:
+      case 11:
+      case 12:
+        tmbbox[numspechit-9] = addr;
+        break;
+      case 13: 
+        nofit = addr;
+        break;
+      case 14: 
+        crushchange = addr;
+        break;
+      default:
+        lprintf(LO_ERROR, "SpechitOverrun: Warning: unable to emulate"
+                          " an overrun where numspechit=%i\n",
+                          numspechit);
+        break;
+    }
   }
 }
