@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2006 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -44,6 +46,7 @@
 #include "g_game.h"
 #include "p_inter.h"
 #include "lprintf.h"
+#include "r_demo.h"
 
 //
 // P_SetMobjState
@@ -132,7 +135,7 @@ void P_ExplodeMissile (mobj_t* mo)
 // Attempts to move something if it has momentum.
 //
 
-void P_XYMovement (mobj_t* mo)
+static void P_XYMovement (mobj_t* mo)
   {
   player_t *player;
   fixed_t xmove, ymove;
@@ -395,7 +398,7 @@ static void P_ZMovement (mobj_t* mo)
       FixedMul(mo->momz, (fixed_t)(FRACUNIT*.45)) ;
 
     /* Bring it to rest below a certain speed */
-    if (abs(mo->momz) <= mo->info->mass*(GRAVITY*4/256))
+    if (D_abs(mo->momz) <= mo->info->mass*(GRAVITY*4/256))
       mo->momz = 0;
   }
 
@@ -474,7 +477,7 @@ floater:
       {
   fixed_t delta;
   if (P_AproxDistance(mo->x - mo->target->x, mo->y - mo->target->y) <
-      abs(delta = mo->target->z + (mo->height>>1) - mo->z)*3)
+      D_abs(delta = mo->target->z + (mo->height>>1) - mo->z)*3)
     mo->z += delta < 0 ? -FLOATSPEED : FLOATSPEED;
       }
 
@@ -593,7 +596,7 @@ floater:
 // P_NightmareRespawn
 //
 
-void P_NightmareRespawn(mobj_t* mobj)
+static void P_NightmareRespawn(mobj_t* mobj)
   {
   fixed_t      x;
   fixed_t      y;
@@ -815,8 +818,6 @@ mobj_t* P_SpawnMobj(fixed_t x,fixed_t y,fixed_t z,mobjtype_t type)
     mobj->ceilingz - mobj->height : z;
 
   mobj->thinker.function = P_MobjThinker;
-  mobj->above_thing = 0;                                            // phares
-  mobj->below_thing = 0;                                            // phares
 
   mobj->target = mobj->tracer = mobj->lastenemy = NULL;
   P_AddThinker (&mobj->thinker);
@@ -837,7 +838,7 @@ int        iquetail;
 //
 
 void P_RemoveMobj (mobj_t* mobj)
-  {
+{
   if ((mobj->flags & MF_SPECIAL)
       && !(mobj->flags & MF_DROPPED)
       && (mobj->type != MT_INV)
@@ -887,9 +888,8 @@ void P_RemoveMobj (mobj_t* mobj)
   }
   // free block
 
-//  P_RemoveThinker ((thinker_t*)mobj);
   P_RemoveThinker (&mobj->thinker);
-  }
+}
 
 
 /*
@@ -900,7 +900,7 @@ void P_RemoveMobj (mobj_t* mobj)
  * killough 8/24/98: rewrote to use hashing
  */
 
-int P_FindDoomedNum(unsigned type)
+static PUREFUNC int P_FindDoomedNum(unsigned type)
 {
   static struct { int first, next; } *hash;
   register int i;
@@ -995,7 +995,7 @@ void P_RespawnSpecials (void)
 
 extern byte playernumtotrans[MAXPLAYERS];
 
-void P_SpawnPlayer (mapthing_t* mthing)
+void P_SpawnPlayer (int n, const mapthing_t* mthing)
   {
   player_t* p;
   fixed_t   x;
@@ -1006,14 +1006,20 @@ void P_SpawnPlayer (mapthing_t* mthing)
 
   // not playing?
 
-  if (!playeringame[mthing->type-1])
+  if (!playeringame[n])
     return;
 
-  p = &players[mthing->type-1];
+  p = &players[n];
 
   if (p->playerstate == PST_REBORN)
     G_PlayerReborn (mthing->type-1);
 
+  /* cph 2001/08/14 - use the options field of memorised player starts to
+   * indicate whether the start really exists in the level.
+   */
+  if (!mthing->options)
+    I_Error("P_SpawnPlayer: attempt to spawn player at unavailable start point");
+  
   x    = mthing->x << FRACBITS;
   y    = mthing->y << FRACBITS;
   z    = ONFLOORZ;
@@ -1021,8 +1027,7 @@ void P_SpawnPlayer (mapthing_t* mthing)
 
   // set color translations for player sprites
 
-  if (mthing->type > 0)
-    mobj->flags |= playernumtotrans[mthing->type-1]<<MF_TRANSSHIFT;
+  mobj->flags |= playernumtotrans[n]<<MF_TRANSSHIFT;
 
   mobj->angle      = ANG45 * (mthing->angle/45);
   mobj->player     = p;
@@ -1055,8 +1060,36 @@ void P_SpawnPlayer (mapthing_t* mthing)
     ST_Start(); // wake up the status bar
     HU_Start(); // wake up the heads up text
     }
+    R_SmoothPlaying_Reset(p); // e6y
   }
 
+/*
+ * P_IsDoomnumAllowed()
+ * Based on code taken from P_LoadThings() in src/p_setup.c  Return TRUE
+ * if the thing in question is expected to be available in the gamemode used.
+ */
+
+boolean P_IsDoomnumAllowed(int doomnum)
+{
+  // Do not spawn cool, new monsters if !commercial
+  if (gamemode != commercial)
+    switch(doomnum)
+      {
+      case 64:  // Archvile
+      case 65:  // Former Human Commando
+      case 66:  // Revenant
+      case 67:  // Mancubus
+      case 68:  // Arachnotron
+      case 69:  // Hell Knight
+      case 71:  // Pain Elemental
+      case 84:  // Wolf SS
+      case 88:  // Boss Brain
+      case 89:  // Boss Shooter
+        return false;
+      }
+
+  return true;
+}
 
 //
 // P_SpawnMapThing
@@ -1064,7 +1097,7 @@ void P_SpawnPlayer (mapthing_t* mthing)
 // already be in host byte order.
 //
 
-void P_SpawnMapThing (mapthing_t* mthing)
+void P_SpawnMapThing (const mapthing_t* mthing)
   {
   int     i;
   //int     bit;
@@ -1072,6 +1105,7 @@ void P_SpawnMapThing (mapthing_t* mthing)
   fixed_t x;
   fixed_t y;
   fixed_t z;
+  int options = mthing->options; /* cph 2001/07/07 - make writable copy */
 
   // killough 2/26/98: Ignore type-0 things as NOPs
   // phares 5/14/98: Ignore Player 5-8 starts (for now)
@@ -1096,11 +1130,11 @@ void P_SpawnMapThing (mapthing_t* mthing)
 
   if (demo_compatibility ||
       (compatibility_level >= lxdoom_1_compatibility  &&
-       mthing->options & MTF_RESERVED)) {
+       options & MTF_RESERVED)) {
     if (!demo_compatibility) // cph - Add warning about bad thing flags
       lprintf(LO_WARN, "P_SpawnMapThing: correcting bad flags (%u) (thing type %d)\n",
-        mthing->options, mthing->type);
-    mthing->options &= MTF_EASY|MTF_NORMAL|MTF_HARD|MTF_AMBUSH|MTF_NOTSINGLE;
+        options, mthing->type);
+    options &= MTF_EASY|MTF_NORMAL|MTF_HARD|MTF_AMBUSH|MTF_NOTSINGLE;
   }
 
   // count deathmatch start positions
@@ -1125,6 +1159,7 @@ void P_SpawnMapThing (mapthing_t* mthing)
       deathmatch_p = deathmatchstarts + offset;
       }
     memcpy(deathmatch_p++, mthing, sizeof(*mthing));
+    (deathmatch_p-1)->options = 1;
     return;
 	}
     }
@@ -1141,42 +1176,61 @@ void P_SpawnMapThing (mapthing_t* mthing)
     players[mthing->type-1].secretcount = 1;
 
     // killough 10/98: force it to be a friend
-    mthing->options |= MTF_FRIEND;
-    i = MT_DOGS;
+    options |= MTF_FRIEND;
+    if(HelperThing != -1) // haleyjd 9/22/99: deh substitution
+    {
+      int type = HelperThing - 1;
+      if(type >= 0 && type < NUMMOBJTYPES)
+      {
+        i = type;
+      }
+      else
+      {
+        doom_printf("Invalid value %i for helper, ignored.", HelperThing);
+        i = MT_DOGS;
+      }
+    }
+    else {
+      i = MT_DOGS;
+    }
     goto spawnit;
   }
 #endif
 
-
-    // save spots for respawning in network games
-
+    // save spots for respawning in coop games
     playerstarts[mthing->type-1] = *mthing;
+    /* cph 2006/07/24 - use the otherwise-unused options field to flag that
+     * this start is present (so we know which elements of the array are filled
+     * in, in effect). Also note that the call below to P_SpawnPlayer must use
+     * the playerstarts version with this field set */
+    playerstarts[mthing->type-1].options = 1;
+
     if (!deathmatch)
-      P_SpawnPlayer (mthing);
+      P_SpawnPlayer (mthing->type-1, &playerstarts[mthing->type-1]);
     return;
     }
 
   // check for apropriate skill level
 
   /* jff "not single" thing flag */
-  if (!netgame && mthing->options & MTF_NOTSINGLE)
+  if (!netgame && options & MTF_NOTSINGLE)
     return;
 
   //jff 3/30/98 implement "not deathmatch" thing flag
 
-  if (netgame && deathmatch && mthing->options & MTF_NOTDM)
+  if (netgame && deathmatch && options & MTF_NOTDM)
     return;
 
   //jff 3/30/98 implement "not cooperative" thing flag
 
-  if (netgame && !deathmatch && mthing->options & MTF_NOTCOOP)
+  if (netgame && !deathmatch && options & MTF_NOTCOOP)
     return;
 
   // killough 11/98: simplify
   if (gameskill == sk_baby || gameskill == sk_easy ?
-      !(mthing->options & MTF_EASY) :
+      !(options & MTF_EASY) :
       gameskill == sk_hard || gameskill == sk_nightmare ?
-      !(mthing->options & MTF_HARD) : !(mthing->options & MTF_NORMAL))
+      !(options & MTF_HARD) : !(options & MTF_NORMAL))
     return;
 
   // find which type to spawn
@@ -1224,7 +1278,7 @@ spawnit:
     mobj->tics = 1 + (P_Random (pr_spawnthing) % mobj->tics);
 
   if (!(mobj->flags & MF_FRIEND) &&
-      mthing->options & MTF_FRIEND &&
+      options & MTF_FRIEND &&
       mbf_features)
     {
       mobj->flags |= MF_FRIEND;            // killough 10/98:
@@ -1239,7 +1293,7 @@ spawnit:
     totalitems++;
 
   mobj->angle = ANG45 * (mthing->angle/45);
-  if (mthing->options & MTF_AMBUSH)
+  if (options & MTF_AMBUSH)
     mobj->flags |= MF_AMBUSH;
   }
 
