@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -202,25 +204,19 @@
 // Hmm ???.
 #define MF_TRANSSHIFT 26
 
-    // proff 11/19/98: Andy Baker's stealth monsters - unused yet
-    //Stealth Mode - Creatures that dissappear and reappear.
-
-#define MF_STEALTH      (uint_64_t)(0x0000000010000000)
-
-    // proff 11/19/98: 3 (4 counting opaque) levels of translucency
-    // not very good to set the next one in this enum, should be seperate
-#define MF_TRANSLUCBITS (uint_64_t)(0x0000000060000000)
-#define MF_TRANSLUC25   (uint_64_t)(0x0000000020000000)
-#define MF_TRANSLUC50   (uint_64_t)(0x0000000040000000)
-#define MF_TRANSLUC75   (uint_64_t)(0x0000000060000000)
+#define MF_UNUSED2      (uint_64_t)(0x0000000010000000)
+#define MF_UNUSED3      (uint_64_t)(0x0000000020000000)
 
     // Translucent sprite?                                          // phares
 #define MF_TRANSLUCENT  (uint_64_t)(0x0000000040000000)
 
+// this is free            LONGLONG(0x0000000100000000)
+
 // these are greater than an int. That's why the flags below are now uint_64_t
-#define MF_TOUCHY       (uint_64_t)(0x0000000100000000)
-#define MF_BOUNCES      (uint_64_t)(0x0000000200000000)
-#define MF_FRIEND       (uint_64_t)(0x0000000400000000)
+
+#define MF_TOUCHY          LONGLONG(0x0000000100000000)
+#define MF_BOUNCES         LONGLONG(0x0000000200000000)
+#define MF_FRIEND          LONGLONG(0x0000000400000000)
 
 // killough 9/15/98: Same, but internal flags, not intended for .deh
 // (some degree of opaqueness is good, to avoid compatibility woes)
@@ -232,6 +228,7 @@ enum {
 
 // Map Object definition.
 //
+//
 // killough 2/20/98:
 //
 // WARNING: Special steps must be taken in p_saveg.c if C pointers are added to
@@ -335,23 +332,9 @@ typedef struct mobj_s
     // Thing being chased/attacked for tracers.
     struct mobj_s*      tracer;
 
-    //proff 11/22/98: Andy Baker's stealth monsters
-    boolean             invisible;
-
     // new field: last known enemy -- killough 2/15/98
     struct mobj_s*      lastenemy;
 
-    // Are we above a Thing? above_thing points to the Thing        // phares
-    // if so, otherwise it's zero.                                  //   |
-                                                                    //   V
-    struct mobj_s* above_thing;
-
-    // Are we below a Thing? below_thing points to the Thing
-    // if so, otherwise it's zero.
-                                                                    //   ^
-    struct mobj_s* below_thing;                                     //   |
-                                                                    // phares
-
     // killough 8/2/98: friction properties part of sectors,
     // not objects -- removed friction properties from here
 
@@ -400,7 +383,9 @@ void    P_SpawnPuff(fixed_t x, fixed_t y, fixed_t z);
 void    P_SpawnBlood(fixed_t x, fixed_t y, fixed_t z, int damage);
 mobj_t  *P_SpawnMissile(mobj_t *source, mobj_t *dest, mobjtype_t type);
 void    P_SpawnPlayerMissile(mobj_t *source, mobjtype_t type);
-void    P_SpawnMapThing (mapthing_t*  mthing);
+boolean P_IsDoomnumAllowed(int doomnum);
+void    P_SpawnMapThing (const mapthing_t*  mthing);
+void    P_SpawnPlayer(int n, const mapthing_t *mthing);
 void    P_CheckMissileSpawn(mobj_t*);  // killough 8/2/98
 void    P_ExplodeMissile(mobj_t*);    // killough
 #endif
