@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -29,7 +31,6 @@
  *
  *-----------------------------------------------------------------------------*/
 
-#include "z_zone.h"
 #include "doomstat.h"
 #include "m_random.h"
 #include "r_main.h"
@@ -46,7 +47,7 @@ platlist_t *activeplats;       // killough 2/14/98: made global again
 // Action routine to move a plat up and down
 //
 // Passed a plat structure containing all pertinent information about the move
-// No return value.
+// No return
 //
 // jff 02/08/98 all cases with labels beginning with gen added to support
 // generalized line type behaviors.
@@ -65,7 +66,7 @@ void T_PlatRaise(plat_t* plat)
       if (plat->type == raiseAndChange
           || plat->type == raiseToNearestAndChange)
       {
-        if (!(leveltime&7) && !silentmove(plat->sector)) // sf: silentmove
+        if (!(leveltime&7))
           S_StartSound((mobj_t *)&plat->sector->soundorg, sfx_stnmov);
       }
 
@@ -74,7 +75,6 @@ void T_PlatRaise(plat_t* plat)
       {
         plat->count = plat->wait;
         plat->status = down;
-        if(!silentmove(plat->sector))    // sf: silentmove
         S_StartSound((mobj_t *)&plat->sector->soundorg, sfx_pstart);
       }
       else  // else handle reaching end of up stroke
@@ -86,7 +86,6 @@ void T_PlatRaise(plat_t* plat)
           {
             plat->count = plat->wait;
             plat->status = waiting;
-            if(!silentmove(plat->sector)) // sf: silentmove
             S_StartSound((mobj_t *)&plat->sector->soundorg, sfx_pstop);
           }
           else // else go into stasis awaiting next toggle activation
@@ -123,7 +122,6 @@ void T_PlatRaise(plat_t* plat)
         {                           // is silent, instant, no waiting
           plat->count = plat->wait;
           plat->status = waiting;
-          if(!silentmove(plat->sector)) // sf: silentmove
           S_StartSound((mobj_t *)&plat->sector->soundorg,sfx_pstop);
         }
         else // instant toggles go into stasis awaiting next activation
@@ -160,7 +158,6 @@ void T_PlatRaise(plat_t* plat)
           plat->status = down;   // if at top, start down
 
         // make plat start sound
-        if(!silentmove(plat->sector))    // sf: silentmove
         S_StartSound((mobj_t *)&plat->sector->soundorg,sfx_pstart);
       }
       break; //jff 1/27/98 don't pickup code added later to in_stasis
@@ -248,7 +245,6 @@ int EV_DoPlat
         //jff 3/14/98 clear old field as well
         sec->oldspecial = 0;
 
-        if(!silentmove(sec)) //sf: silentmove
         S_StartSound((mobj_t *)&sec->soundorg,sfx_stnmov);
         break;
 
@@ -259,7 +255,6 @@ int EV_DoPlat
         plat->wait = 0;
         plat->status = up;
 
-        if(!silentmove(sec)) //sf: silentmove
         S_StartSound((mobj_t *)&sec->soundorg,sfx_stnmov);
         break;
 
@@ -273,7 +268,6 @@ int EV_DoPlat
         plat->high = sec->floorheight;
         plat->wait = 35*PLATWAIT;
         plat->status = down;
-        if(!silentmove(sec))    // sf: silentmove
         S_StartSound((mobj_t *)&sec->soundorg,sfx_pstart);
         break;
 
@@ -287,7 +281,6 @@ int EV_DoPlat
         plat->high = sec->floorheight;
         plat->wait = 35*PLATWAIT;
         plat->status = down;
-        if(!silentmove(sec))    // sf: silentmove
         S_StartSound((mobj_t *)&sec->soundorg,sfx_pstart);
         break;
 
@@ -306,7 +299,6 @@ int EV_DoPlat
         plat->wait = 35*PLATWAIT;
         plat->status = P_Random(pr_plats)&1;
 
-        if(!silentmove(sec))    // sf: silentmove
         S_StartSound((mobj_t *)&sec->soundorg,sfx_pstart);
         break;
 
