@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -29,7 +31,6 @@
  *
  *-----------------------------------------------------------------------------*/
 
-#include "z_zone.h"
 #include "doomstat.h"
 #include "r_main.h"
 #include "p_maputl.h"
@@ -90,6 +91,7 @@ void P_UnArchivePlayers (void)
 
         // will be set when unarc thinker
         players[i].mo = NULL;
+        players[i].message = NULL;
         players[i].attacker = NULL;
 
         for (j=0 ; j<NUMPSPRITES ; j++)
@@ -113,18 +115,18 @@ void P_ArchiveWorld (void)
 
   // killough 3/22/98: fix bug caused by hoisting save_p too early
   // killough 10/98: adjust size for changes below
-  size_t size = 
-    (sizeof(short)*5 + sizeof sec->floorheight + sizeof sec->ceilingheight) 
+  size_t size =
+    (sizeof(short)*5 + sizeof sec->floorheight + sizeof sec->ceilingheight)
     * numsectors + sizeof(short)*3*numlines + 4;
 
   for (i=0; i<numlines; i++)
     {
-      if (lines[i].sidenum[0] != -1)
+      if (lines[i].sidenum[0] != NO_INDEX)
         size +=
-	  sizeof(short)*3 + sizeof si->textureoffset + sizeof si->rowoffset;
-      if (lines[i].sidenum[1] != -1)
-	size +=
-	  sizeof(short)*3 + sizeof si->textureoffset + sizeof si->rowoffset;
+    sizeof(short)*3 + sizeof si->textureoffset + sizeof si->rowoffset;
+      if (lines[i].sidenum[1] != NO_INDEX)
+  size +=
+    sizeof(short)*3 + sizeof si->textureoffset + sizeof si->rowoffset;
     }
 
   CheckSaveGame(size); // killough
@@ -159,17 +161,17 @@ void P_ArchiveWorld (void)
       *put++ = li->tag;
 
       for (j=0; j<2; j++)
-        if (li->sidenum[j] != -1)
+        if (li->sidenum[j] != NO_INDEX)
           {
-	    si = &sides[li->sidenum[j]];
+      si = &sides[li->sidenum[j]];
 
-	    // killough 10/98: save full sidedef offsets,
-	    // preserving fractional scroll offsets
+      // killough 10/98: save full sidedef offsets,
+      // preserving fractional scroll offsets
 
-	    memcpy(put, &si->textureoffset, sizeof si->textureoffset);
-	    put = (void *)((char *) put + sizeof si->textureoffset);
-	    memcpy(put, &si->rowoffset, sizeof si->rowoffset);
-	    put = (void *)((char *) put + sizeof si->rowoffset);
+      memcpy(put, &si->textureoffset, sizeof si->textureoffset);
+      put = (void *)((char *) put + sizeof si->textureoffset);
+      memcpy(put, &si->rowoffset, sizeof si->rowoffset);
+      put = (void *)((char *) put + sizeof si->rowoffset);
 
             *put++ = si->toptexture;
             *put++ = si->bottomtexture;
@@ -225,16 +227,16 @@ void P_UnArchiveWorld (void)
       li->special = *get++;
       li->tag = *get++;
       for (j=0 ; j<2 ; j++)
-        if (li->sidenum[j] != -1)
+        if (li->sidenum[j] != NO_INDEX)
           {
             side_t *si = &sides[li->sidenum[j]];
 
-	    // killough 10/98: load full sidedef offsets, including fractions
+      // killough 10/98: load full sidedef offsets, including fractions
 
-	    memcpy(&si->textureoffset, get, sizeof si->textureoffset);
-	    get = (void *)((char *) get + sizeof si->textureoffset);
-	    memcpy(&si->rowoffset, get, sizeof si->rowoffset);
-	    get = (void *)((char *) get + sizeof si->rowoffset);
+      memcpy(&si->textureoffset, get, sizeof si->textureoffset);
+      get = (void *)((char *) get + sizeof si->textureoffset);
+      memcpy(&si->rowoffset, get, sizeof si->rowoffset);
+      get = (void *)((char *) get + sizeof si->rowoffset);
 
             si->toptexture = *get++;
             si->bottomtexture = *get++;
@@ -301,8 +303,12 @@ void P_ArchiveThinkers (void)
   /* check that enough room is available in savegame buffer
    * - killough 2/14/98
    * cph - use number_of_thinkers saved by P_ThinkerToIndex above
+   * Andreas Dehmel: size per object is sizeof(mobj_t) - 2*sizeof(void*) plus
+   * padded type (4) plus 5*sizeof(void*), i.e. sizeof(mobj_t) + 4 +
+   * 3*sizeof(void*)
+   * cph - +1 for the tc_end
    */
-  CheckSaveGame(number_of_thinkers*(sizeof(mobj_t)+4));
+  CheckSaveGame(number_of_thinkers*(sizeof(mobj_t)+4+3*sizeof(void*)) +1);
 
   // save off the current thinkers
   for (th = thinkercap.next ; th != &thinkercap ; th=th->next)
@@ -313,8 +319,25 @@ void P_ArchiveThinkers (void)
         *save_p++ = tc_mobj;
         PADSAVEP();
         mobj = (mobj_t *)save_p;
-        memcpy (mobj, th, sizeof(*mobj));
-        save_p += sizeof(*mobj);
+	/* cph 2006/07/30 - 
+	 * The end of mobj_t changed from
+	 *  boolean invisible;
+	 *  mobj_t* lastenemy;
+	 *  mobj_t* above_monster;
+	 *  mobj_t* below_monster;
+	 *  void* touching_sectorlist;
+	 * to
+	 *  mobj_t* lastenemy;
+	 *  void* touching_sectorlist;
+	 * at prboom 2.4.4. There is code here to preserve the savegame format.
+	 *
+	 * touching_sectorlist is reconstructed anyway, so we now leave off the
+	 * last 2 words of mobj_t, write 5 words of 0 and then write lastenemy
+	 * into the second of these.
+	 */
+        memcpy (mobj, th, sizeof(*mobj) - 2*sizeof(void*));
+        save_p += sizeof(*mobj) - 2*sizeof(void*);
+	memset (save_p, 0, 5*sizeof(void*));
         mobj->state = (state_t *)(mobj->state - states);
 
         // killough 2/14/98: convert pointers into indices.
@@ -337,22 +360,13 @@ void P_ArchiveThinkers (void)
         // monsters from going to sleep after killing monsters and not
         // seeing player anymore.
 
-        if (mobj->lastenemy)
-          mobj->lastenemy = mobj->lastenemy->thinker.function ==
-            P_MobjThinker ?
-            (mobj_t *) mobj->lastenemy->thinker.prev : NULL;
+        if (((mobj_t*)th)->lastenemy && ((mobj_t*)th)->lastenemy->thinker.function == P_MobjThinker) {
+	  memcpy (save_p + sizeof(void*), &(((mobj_t*)th)->lastenemy->thinker.prev), sizeof(void*));
+	}
 
         // killough 2/14/98: end changes
 
-        if (mobj->above_thing)                                      // phares
-          mobj->above_thing = mobj->above_thing->thinker.function ==
-            P_MobjThinker ?
-            (mobj_t *) mobj->above_thing->thinker.prev : NULL;
-
-        if (mobj->below_thing)
-          mobj->below_thing = mobj->below_thing->thinker.function ==
-            P_MobjThinker ?
-            (mobj_t *) mobj->below_thing->thinker.prev : NULL;      // phares
+	save_p += 5*sizeof(void*);
 
         if (mobj->player)
           mobj->player = (player_t *)((mobj->player-players) + 1);
@@ -367,11 +381,11 @@ void P_ArchiveThinkers (void)
     CheckSaveGame(numsectors * sizeof(mobj_t *));       // killough 9/14/98
     for (i = 0; i < numsectors; i++)
       {
-	mobj_t *target = sectors[i].soundtarget;
-	if (target)
-	  target = (mobj_t *) target->thinker.prev;
-	memcpy(save_p, &target, sizeof target);
-	save_p += sizeof target;
+  mobj_t *target = sectors[i].soundtarget;
+  if (target)
+    target = (mobj_t *) target->thinker.prev;
+  memcpy(save_p, &target, sizeof target);
+  save_p += sizeof target;
       }
   }
 }
@@ -396,12 +410,22 @@ static void P_SetNewTarget(mobj_t **mop, mobj_t *targ)
 // 2/14/98 killough: substantially modified to fix savegame bugs
 //
 
+// savegame file stores ints in the corresponding * field; this function
+// safely casts them back to int.
+static int P_GetMobj(mobj_t* mi, size_t s)
+{
+  size_t i = (size_t)mi;
+  if (i >= s) I_Error("Corrupt savegame");
+  return i;
+}
+
 void P_UnArchiveThinkers (void)
 {
   thinker_t *th;
   mobj_t    **mobj_p;    // killough 2/14/98: Translation table
   size_t    size;        // killough 2/14/98: size of or index into table
 
+  totallive = 0;
   // killough 3/26/98: Load boss brain state
   memcpy(&brain, save_p, sizeof brain);
   save_p += sizeof brain;
@@ -424,7 +448,8 @@ void P_UnArchiveThinkers (void)
     for (size = 1; *save_p++ == tc_mobj; size++)  // killough 2/14/98
       {                     // skip all entries, adding up count
         PADSAVEP();
-        save_p += sizeof(mobj_t);
+	/* cph 2006/07/30 - see comment below for change in layout of mobj_t */
+        save_p += sizeof(mobj_t)+3*sizeof(void*);
       }
 
     if (*--save_p != tc_end)
@@ -444,8 +469,27 @@ void P_UnArchiveThinkers (void)
       mobj_p[size] = mobj;
 
       PADSAVEP();
-      memcpy (mobj, save_p, sizeof(mobj_t));
-      save_p += sizeof(mobj_t);
+      /* cph 2006/07/30 - 
+       * The end of mobj_t changed from
+       *  boolean invisible;
+       *  mobj_t* lastenemy;
+       *  mobj_t* above_monster;
+       *  mobj_t* below_monster;
+       *  void* touching_sectorlist;
+       * to
+       *  mobj_t* lastenemy;
+       *  void* touching_sectorlist;
+       * at prboom 2.4.4. There is code here to preserve the savegame format.
+       *
+       * touching_sectorlist is reconstructed anyway, so we now read in all 
+       * but the last 5 words from the savegame (filling all but the last 2
+       * fields of our current mobj_t. We then pull lastenemy from the 2nd of
+       * the 5 leftover words, and skip the others.
+       */
+      memcpy (mobj, save_p, sizeof(mobj_t)-2*sizeof(void*));
+      save_p += sizeof(mobj_t)-sizeof(void*);
+      memcpy (&(mobj->lastenemy), save_p, sizeof(void*));
+      save_p += 4*sizeof(void*);
       mobj->state = states + (int) mobj->state;
 
       if (mobj->player)
@@ -461,6 +505,9 @@ void P_UnArchiveThinkers (void)
 
       mobj->thinker.function = P_MobjThinker;
       P_AddThinker (&mobj->thinker);
+
+      if (!((mobj->flags ^ MF_COUNTKILL) & (MF_FRIEND | MF_COUNTKILL | MF_CORPSE)))
+        totallive++;
     }
 
   // killough 2/14/98: adjust target and tracer fields, plus
@@ -472,31 +519,23 @@ void P_UnArchiveThinkers (void)
   for (th = thinkercap.next ; th != &thinkercap ; th=th->next)
     {
       P_SetNewTarget(&((mobj_t *) th)->target,
-        mobj_p[(size_t)((mobj_t *)th)->target]);
+        mobj_p[P_GetMobj(((mobj_t *)th)->target,size)]);
 
       P_SetNewTarget(&((mobj_t *) th)->tracer,
-        mobj_p[(size_t)((mobj_t *)th)->tracer]);
+        mobj_p[P_GetMobj(((mobj_t *)th)->tracer,size)]);
 
       P_SetNewTarget(&((mobj_t *) th)->lastenemy,
-        mobj_p[(size_t)((mobj_t *)th)->lastenemy]);
-
-      // phares: added two new fields for Sprite Height problem
-
-      P_SetNewTarget(&((mobj_t *) th)->above_thing,
-        mobj_p[(size_t)((mobj_t *)th)->above_thing]);
-
-      P_SetNewTarget(&((mobj_t *) th)->below_thing,
-        mobj_p[(size_t)((mobj_t *)th)->below_thing]);
+        mobj_p[P_GetMobj(((mobj_t *)th)->lastenemy,size)]);
     }
 
   {  // killough 9/14/98: restore soundtargets
     int i;
     for (i = 0; i < numsectors; i++)
       {
-	mobj_t *target;
-	memcpy(&target, save_p, sizeof target);
-	save_p += sizeof target;
-	P_SetNewTarget(&sectors[i].soundtarget, mobj_p[(size_t) target]);
+  mobj_t *target;
+  memcpy(&target, save_p, sizeof target);
+  save_p += sizeof target;
+  P_SetNewTarget(&sectors[i].soundtarget, mobj_p[(size_t) target]);
       }
   }
 
@@ -582,7 +621,7 @@ void P_ArchiveSpecials (void)
         th->function==T_FireFlicker? 4+sizeof(fireflicker_t) :
       0;
 
-  CheckSaveGame(size);          // killough
+  CheckSaveGame(size + 1);    // killough; cph: +1 for the tc_endspecials
 
   // save off the current thinkers
   for (th=thinkercap.next; th!=&thinkercap; th=th->next)
@@ -926,6 +965,7 @@ void P_UnArchiveRNG(void)
   save_p += sizeof rng;
 }
 
+// killough 2/22/98: Save/restore automap state
 // killough 2/22/98: Save/restore automap state
 void P_ArchiveMap(void)
 {
