@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -47,9 +49,8 @@
 #include "p_enemy.h"
 #include "s_sound.h"
 #include "lprintf.h" //jff 10/6/98 for debug outputs
-#ifdef GL_DOOM
-#include "gl_struct.h"
-#endif
+#include "v_video.h"
+#include "r_demo.h"
 
 //
 // MAP related Lookup tables.
@@ -163,7 +164,7 @@ mapthing_t playerstarts[MAXPLAYERS];
 //
 static void P_LoadVertexes (int lump)
 {
-  const byte *data; // cph - const
+  const mapvertex_t *data; // cph - const
   int i;
 
   // Determine number of lumps:
@@ -174,14 +175,15 @@ static void P_LoadVertexes (int lump)
   vertexes = Z_Malloc(numvertexes*sizeof(vertex_t),PU_LEVEL,0);
 
   // Load data into cache.
-  data = W_CacheLumpNum(lump); // cph - wad handling updated
+  // cph 2006/07/29 - cast to mapvertex_t here, making the loop below much neater
+  data = (const mapvertex_t *)W_CacheLumpNum(lump);
 
   // Copy and convert vertex coordinates,
   // internal representation as fixed.
   for (i=0; i<numvertexes; i++)
     {
-      vertexes[i].x = SHORT(((mapvertex_t *) data)[i].x)<<FRACBITS;
-      vertexes[i].y = SHORT(((mapvertex_t *) data)[i].y)<<FRACBITS;
+      vertexes[i].x = SHORT(data[i].x)<<FRACBITS;
+      vertexes[i].y = SHORT(data[i].y)<<FRACBITS;
     }
 
   // Free buffer memory.
@@ -200,25 +202,24 @@ static void P_LoadVertexes (int lump)
 
 static void P_LoadVertexes2(int lump, int gllump)
 {
-  const byte      *data, *gldata;
+  const byte         *gldata;
   int                 i;
-  mapvertex_t*        ml;
+  const mapvertex_t*  ml;
 
   firstglvertex = W_LumpLength(lump) / sizeof(mapvertex_t);
   numvertexes   = W_LumpLength(lump) / sizeof(mapvertex_t);
-  data      = W_CacheLumpNum(lump);
 
   if (gllump >= 0)  // check for glVertices
   {
     gldata = W_CacheLumpNum(gllump);
 
-    if (*(int *)gldata == gNd2) // 32 bit GL_VERT format (16.16 fixed)
+    if (*(const int *)gldata == gNd2) // 32 bit GL_VERT format (16.16 fixed)
     {
-      mapglvertex_t*  mgl;
+      const mapglvertex_t*  mgl;
 
       numvertexes += (W_LumpLength(gllump) - GL_VERT_OFFSET)/sizeof(mapglvertex_t);
       vertexes   = Z_Malloc (numvertexes*sizeof(vertex_t),PU_LEVEL,0);
-      mgl      = (mapglvertex_t *) (gldata + GL_VERT_OFFSET);
+      mgl      = (const mapglvertex_t *) (gldata + GL_VERT_OFFSET);
 
       for (i = firstglvertex; i < numvertexes; i++)
       {
@@ -231,7 +232,7 @@ static void P_LoadVertexes2(int lump, int gllump)
     {
       numvertexes += W_LumpLength(gllump)/sizeof(mapvertex_t);
       vertexes     = Z_Malloc (numvertexes*sizeof(vertex_t),PU_LEVEL,0);
-      ml       = (mapvertex_t *)gldata;
+      ml       = (const mapvertex_t *)gldata;
 
       for (i = firstglvertex; i < numvertexes; i++)
       {
@@ -243,7 +244,7 @@ static void P_LoadVertexes2(int lump, int gllump)
     W_UnlockLumpNum(gllump);
   }
 
-  ml = (mapvertex_t*)data;
+  ml = (const mapvertex_t*) W_CacheLumpNum(lump);
 
   for (i=0; i < firstglvertex; i++)
   {
@@ -298,34 +299,33 @@ static int GetOffset(vertex_t *v1, vertex_t *v2)
 static void P_LoadSegs (int lump)
 {
   int  i;
-  const byte *data; // cph - const
+  const mapseg_t *data; // cph - const
 
   numsegs = W_LumpLength(lump) / sizeof(mapseg_t);
   segs = Z_Calloc(numsegs,sizeof(seg_t),PU_LEVEL,0);
-  data = W_CacheLumpNum(lump); // cph - wad lump handling updated
+  data = (const mapseg_t *)W_CacheLumpNum(lump); // cph - wad lump handling updated
 
   for (i=0; i<numsegs; i++)
     {
       seg_t *li = segs+i;
-      mapseg_t *ml = (mapseg_t *) data + i;
+      const mapseg_t *ml = data + i;
+      unsigned short v1, v2;
 
       int side, linedef;
       line_t *ldef;
 
-#ifdef GL_DOOM
       li->iSegID = i; // proff 11/05/2000: needed for OpenGL
-#endif
 
-      li->v1 = &vertexes[SHORT(ml->v1)];
-      li->v2 = &vertexes[SHORT(ml->v2)];
+      v1 = (unsigned short)SHORT(ml->v1);
+      v2 = (unsigned short)SHORT(ml->v2);
+      li->v1 = &vertexes[v1];
+      li->v2 = &vertexes[v2];
 
       li->miniseg = false; // figgi -- there are no minisegs in classic BSP nodes
-#ifdef GL_DOOM
       li->length  = GetDistance(li->v2->x - li->v1->x, li->v2->y - li->v1->y);
-#endif
       li->angle = (SHORT(ml->angle))<<16;
       li->offset =(SHORT(ml->offset))<<16;
-      linedef = SHORT(ml->linedef);
+      linedef = (unsigned short)SHORT(ml->linedef);
       ldef = &lines[linedef];
       li->linedef = ldef;
       side = SHORT(ml->side);
@@ -333,7 +333,7 @@ static void P_LoadSegs (int lump)
       li->frontsector = sides[ldef->sidenum[side]].sector;
 
       // killough 5/3/98: ignore 2s flag if second sidedef missing:
-      if (ldef->flags & ML_TWOSIDED && ldef->sidenum[side^1]!=-1)
+      if (ldef->flags & ML_TWOSIDED && ldef->sidenum[side^1]!=NO_INDEX)
         li->backsector = sides[ldef->sidenum[side^1]].sector;
       else
         li->backsector = 0;
@@ -353,24 +353,20 @@ static void P_LoadSegs (int lump)
  *******************************************/
 static void P_LoadGLSegs(int lump)
 {
-  const byte  *data;
   int     i;
-  glseg_t   *ml;
+  const glseg_t   *ml;
   line_t    *ldef;
 
   numsegs = W_LumpLength(lump) / sizeof(glseg_t);
   segs = Z_Malloc(numsegs * sizeof(seg_t), PU_LEVEL, 0);
   memset(segs, 0, numsegs * sizeof(seg_t));
-  data = W_CacheLumpNum(lump);
+  ml = (const glseg_t*)W_CacheLumpNum(lump);
 
-  ml = (glseg_t*) data;
   for(i = 0; i < numsegs; i++)
   {             // check for gl-vertices
-    segs[i].v1 = &vertexes[SHORT(checkGLVertex(ml->v1))];
-    segs[i].v2 = &vertexes[SHORT(checkGLVertex(ml->v2))];
-#ifdef GL_DOOM
+    segs[i].v1 = &vertexes[checkGLVertex(SHORT(ml->v1))];
+    segs[i].v2 = &vertexes[checkGLVertex(SHORT(ml->v2))];
     segs[i].iSegID  = i;
-#endif
 
     if(ml->linedef != -1) // skip minisegs
     {
@@ -380,9 +376,7 @@ static void P_LoadGLSegs(int lump)
       segs[i].angle = R_PointToAngle2(segs[i].v1->x,segs[i].v1->y,segs[i].v2->x,segs[i].v2->y);
 
       segs[i].sidedef = &sides[ldef->sidenum[ml->side]];
-#ifdef GL_DOOM
       segs[i].length  = GetDistance(segs[i].v2->x - segs[i].v1->x, segs[i].v2->y - segs[i].v1->y);
-#endif
       segs[i].frontsector = sides[ldef->sidenum[ml->side]].sector;
       if (ldef->flags & ML_TWOSIDED)
         segs[i].backsector = sides[ldef->sidenum[ml->side^1]].sector;
@@ -399,9 +393,7 @@ static void P_LoadGLSegs(int lump)
       segs[i].miniseg = true;
       segs[i].angle  = 0;
       segs[i].offset  = 0;
-#ifdef GL_DOOM
       segs[i].length  = 0;
-#endif
       segs[i].linedef = NULL;
       segs[i].sidedef = NULL;
       segs[i].frontsector = NULL;
@@ -419,17 +411,18 @@ static void P_LoadGLSegs(int lump)
 
 static void P_LoadSubsectors (int lump)
 {
-  const byte *data; // cph - const*
+  /* cph 2006/07/29 - make data a const mapsubsector_t *, so the loop below is simpler & gives no constness warnings */
+  const mapsubsector_t *data;
   int  i;
 
   numsubsectors = W_LumpLength (lump) / sizeof(mapsubsector_t);
   subsectors = Z_Calloc(numsubsectors,sizeof(subsector_t),PU_LEVEL,0);
-  data = W_CacheLumpNum(lump); // cph - wad lump handling updated
+  data = (const mapsubsector_t *)W_CacheLumpNum(lump);
 
   for (i=0; i<numsubsectors; i++)
   {
-    subsectors[i].numlines  = (unsigned short)SHORT(((mapsubsector_t *) data)[i].numsegs );
-    subsectors[i].firstline = (unsigned short)SHORT(((mapsubsector_t *) data)[i].firstseg);
+    subsectors[i].numlines  = (unsigned short)SHORT(data[i].numsegs );
+    subsectors[i].firstline = (unsigned short)SHORT(data[i].firstseg);
   }
 
   W_UnlockLumpNum(lump); // cph - release the data
@@ -452,11 +445,9 @@ static void P_LoadSectors (int lump)
   for (i=0; i<numsectors; i++)
     {
       sector_t *ss = sectors + i;
-      const mapsector_t *ms = (mapsector_t *) data + i;
+      const mapsector_t *ms = (const mapsector_t *) data + i;
 
-#ifdef GL_DOOM
       ss->iSectorID=i; // proff 04/05/2000: needed for OpenGL
-#endif
       ss->floorheight = SHORT(ms->floorheight)<<FRACBITS;
       ss->ceilingheight = SHORT(ms->ceilingheight)<<FRACBITS;
       ss->floorpic = R_FlatNumForName(ms->floorpic);
@@ -511,7 +502,7 @@ static void P_LoadNodes (int lump)
   for (i=0; i<numnodes; i++)
     {
       node_t *no = nodes + i;
-      mapnode_t *mn = (mapnode_t *) data + i;
+      const mapnode_t *mn = (const mapnode_t *) data + i;
       int j;
 
       no->x = SHORT(mn->x)<<FRACBITS;
@@ -555,22 +546,8 @@ static void P_LoadThings (int lump)
       mt.type = SHORT(mt.type);
       mt.options = SHORT(mt.options);
 
-      // Do not spawn cool, new monsters if !commercial
-      if (gamemode != commercial)
-        switch(mt.type)
-          {
-          case 68:  // Arachnotron
-          case 64:  // Archvile
-          case 88:  // Boss Brain
-          case 89:  // Boss Shooter
-          case 69:  // Hell Knight
-          case 67:  // Mancubus
-          case 71:  // Pain Elemental
-          case 65:  // Former Human Commando
-          case 66:  // Revenant
-          case 84:  // Wolf SS
-            continue;
-          }
+      if (!P_IsDoomnumAllowed(mt.type))
+        continue;
 
       // Do spawn all other stuff.
       P_SpawnMapThing(&mt);
@@ -601,15 +578,15 @@ static void P_LoadLineDefs (int lump)
 
   for (i=0; i<numlines; i++)
     {
-      maplinedef_t *mld = (maplinedef_t *) data + i;
+      const maplinedef_t *mld = (const maplinedef_t *) data + i;
       line_t *ld = lines+i;
       vertex_t *v1, *v2;
 
-      ld->flags = SHORT(mld->flags);
+      ld->flags = (unsigned short)SHORT(mld->flags);
       ld->special = SHORT(mld->special);
       ld->tag = SHORT(mld->tag);
-      v1 = ld->v1 = &vertexes[SHORT(mld->v1)];
-      v2 = ld->v2 = &vertexes[SHORT(mld->v2)];
+      v1 = ld->v1 = &vertexes[(unsigned short)SHORT(mld->v1)];
+      v2 = ld->v2 = &vertexes[(unsigned short)SHORT(mld->v2)];
       ld->dx = v2->x - v1->x;
       ld->dy = v2->y - v1->y;
 
@@ -640,14 +617,12 @@ static void P_LoadLineDefs (int lump)
           ld->bbox[BOXTOP] = v1->y;
         }
 
-#ifdef GL_DOOM
       ld->iLineID=i; // proff 04/05/2000: needed for OpenGL
-#endif
       ld->sidenum[0] = SHORT(mld->sidenum[0]);
       ld->sidenum[1] = SHORT(mld->sidenum[1]);
 
       // killough 4/4/98: support special sidedef interpretation below
-      if (ld->sidenum[0] != -1 && ld->special)
+      if (ld->sidenum[0] != NO_INDEX && ld->special)
         sides[*ld->sidenum].special = ld->special;
     }
 
@@ -664,23 +639,34 @@ static void P_LoadLineDefs2(int lump)
   for (;i--;ld++)
     {
       { // cph 2002/07/20 - these errors are fatal if not fixed, so apply them in compatibility mode - a desync is better than a crash!
-  // killough 11/98: fix common wad errors (missing sidedefs):
-
-  if (ld->sidenum[0] == -1) {
-    ld->sidenum[0] = 0;  // Substitute dummy sidedef for missing right side
-    // cph - print a warning about the bug
-    lprintf(LO_WARN, "P_LoadSegs: linedef %d missing first sidedef\n",numlines-i);
-  }
-
-  if ((ld->sidenum[1] == -1) && (ld->flags & ML_TWOSIDED)) {
-    ld->flags &= ~ML_TWOSIDED;  // Clear 2s flag for missing left side
-    // cph - print a warning about the bug
-    lprintf(LO_WARN, "P_LoadSegs: linedef %d has two-sided flag set, but no second sidedef\n",numlines-i);
-  }
+        int j;
+        
+        for (j=0; j < 2; j++)
+        {
+          if (ld->sidenum[j] != NO_INDEX && ld->sidenum[j] >= numsides) {
+            ld->sidenum[j] = NO_INDEX;
+            lprintf(LO_WARN, "P_LoadSegs: linedef %d has out-of-range sidedef number\n",numlines-i-1);
+          }
+        }
+        
+        // killough 11/98: fix common wad errors (missing sidedefs):
+        
+        if (ld->sidenum[0] == NO_INDEX) {
+          ld->sidenum[0] = 0;  // Substitute dummy sidedef for missing right side
+          // cph - print a warning about the bug
+          lprintf(LO_WARN, "P_LoadSegs: linedef %d missing first sidedef\n",numlines-i-1);
+        }
+        
+        if ((ld->sidenum[1] == NO_INDEX) && (ld->flags & ML_TWOSIDED)) {
+          ld->flags &= ~ML_TWOSIDED;  // Clear 2s flag for missing left side
+          // cph - print a warning about the bug
+          lprintf(LO_WARN, "P_LoadSegs: linedef %d has two-sided flag set, but no second sidedef\n",numlines-i-1);
+        }
+        
       }
 
-      ld->frontsector = ld->sidenum[0]!=-1 ? sides[ld->sidenum[0]].sector : 0;
-      ld->backsector  = ld->sidenum[1]!=-1 ? sides[ld->sidenum[1]].sector : 0;
+      ld->frontsector = sides[ld->sidenum[0]].sector; //e6y: Can't be NO_INDEX here
+      ld->backsector  = ld->sidenum[1]!=NO_INDEX ? sides[ld->sidenum[1]].sector : 0;
       switch (ld->special)
         {                       // killough 4/11/98: handle special types
           int lump, j;
@@ -720,7 +706,7 @@ static void P_LoadSideDefs2(int lump)
 
   for (i=0; i<numsides; i++)
     {
-      register mapsidedef_t *msd = (mapsidedef_t *) data + i;
+      register const mapsidedef_t *msd = (const mapsidedef_t *) data + i;
       register side_t *sd = sides + i;
       register sector_t *sec;
 
@@ -757,9 +743,9 @@ static void P_LoadSideDefs2(int lump)
           break;
 
         default:                        // normal cases
-          sd->midtexture = R_TextureNumForName(msd->midtexture);
-          sd->toptexture = R_TextureNumForName(msd->toptexture);
-          sd->bottomtexture = R_TextureNumForName(msd->bottomtexture);
+          sd->midtexture = R_SafeTextureNumForName(msd->midtexture, i);
+          sd->toptexture = R_SafeTextureNumForName(msd->toptexture, i);
+          sd->bottomtexture = R_SafeTextureNumForName(msd->bottomtexture, i);
           break;
         }
     }
@@ -820,7 +806,7 @@ static void AddBlockLine
 // adds the line to all block lists touching the intersection.
 //
 
-void P_CreateBlockMap()
+static void P_CreateBlockMap(void)
 {
   int xorg,yorg;                 // blockmap origin (lower left)
   int nrows,ncols;               // blockmap dimensions
@@ -1174,7 +1160,7 @@ static void RejectOverrunAddInt(int k)
   }
 }
 
-void P_GroupLines (void)
+static void P_GroupLines (void)
 {
   register line_t *li;
   register sector_t *sector;
@@ -1317,7 +1303,7 @@ void P_GroupLines (void)
 // Firelines (TM) is a Rezistered Trademark of MBF Productions
 //
 
-void P_RemoveSlimeTrails(void)                // killough 10/98
+static void P_RemoveSlimeTrails(void)         // killough 10/98
 {
   byte *hit = calloc(1, numvertexes);         // Hitlist for vertices
   int i;
@@ -1441,14 +1427,14 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     P_LoadSubsectors(gl_lumpnum + ML_GL_SSECT);
     P_LoadNodes(gl_lumpnum + ML_GL_NODES);
     P_LoadGLSegs(gl_lumpnum + ML_GL_SEGS);
-    lprintf(LO_INFO,"Using glBSP nodes!\n");
+    lprintf(LO_DEBUG,"Using glBSP nodes!\n");
   }
   else
   {
     P_LoadSubsectors(lumpnum + ML_SSECTORS);
     P_LoadNodes(lumpnum + ML_NODES);
     P_LoadSegs(lumpnum + ML_SEGS);
-    lprintf(LO_INFO,"Using normal BSP nodes!\n");
+    lprintf(LO_DEBUG,"Using normal BSP nodes!\n");
   }
 
 #else
@@ -1492,8 +1478,12 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
   // a much simpler fix is in g_game.c -- killough 10/98
 
   bodyqueslot = 0;
+
+  /* cph - reset all multiplayer starts */
+  memset(playerstarts,0,sizeof(playerstarts));
   deathmatch_p = deathmatchstarts;
   P_MapStart();
+
   P_LoadThings(lumpnum+ML_THINGS);
 
   // if deathmatch, randomly spawn the active players
@@ -1522,10 +1512,14 @@ void P_SetupLevel(int episode, int map, int playermask, skill_t skill)
     R_PrecacheLevel();
 
 #ifdef GL_DOOM
- // proff 11/99: calculate all OpenGL specific tables etc.
-  gld_PreprocessLevel();
+  if (V_GetMode() == VID_MODEGL)
+  {
+    // proff 11/99: calculate all OpenGL specific tables etc.
+    gld_PreprocessLevel();
+  }
 #endif
 
+  R_SmoothPlaying_Reset(NULL); // e6y
 }
 
 //
