@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -37,7 +39,6 @@
  *
  *-----------------------------------------------------------------------------*/
 
-#include "z_zone.h"
 #include "doomstat.h"
 #include "p_spec.h"
 #include "p_tick.h"
@@ -57,7 +58,6 @@
 #include "d_deh.h"
 #include "r_plane.h"
 #include "lprintf.h"
-#include "t_script.h"
 
 //
 // Animating textures and planes
@@ -76,14 +76,7 @@ typedef struct
 //
 //      source animation definition
 //
-#ifdef _MSC_VER
-#define GCC_PACKED
-#else //_MSC_VER
-/* we might want to make this sensitive to whether configure has detected
-   a build with GCC */
-#define GCC_PACKED __attribute__((packed))
-#endif //_MSC_VER
-
+//
 #ifdef _MSC_VER // proff: This is the same as __attribute__ ((packed)) in GNUC
 #pragma pack(push)
 #pragma pack(1)
@@ -95,11 +88,11 @@ typedef struct
 
 typedef struct
 {
-  signed char istexture GCC_PACKED; //jff 3/23/98 make char for comparison // cph - make signed
-  char        endname[9] GCC_PACKED;           //  if false, it is a flat
-  char        startname[9] GCC_PACKED;
-  int         speed GCC_PACKED;
-} GCC_PACKED animdef_t; //jff 3/23/98 pack to read from memory
+  signed char istexture; //jff 3/23/98 make char for comparison // cph - make signed
+  char        endname[9];           //  if false, it is a flat
+  char        startname[9];
+  int         speed;
+} PACKEDATTR animdef_t; //jff 3/23/98 pack to read from memory
 
 #if defined(__MWERKS__)
 #pragma options align=reset
@@ -121,9 +114,6 @@ static void P_SpawnScrollers(void);
 static void P_SpawnFriction(void);    // phares 3/16/98
 static void P_SpawnPushers(void);     // phares 3/20/98
 
-extern int allow_pushers;
-extern int variable_friction;         // phares 3/20/98
-
 //
 // P_InitPicAnims
 //
@@ -215,8 +205,10 @@ void P_InitPicAnims (void)
 //
 // Note: if side=1 is specified, it must exist or results undefined
 //
-
-side_t *getSide(int currentSector, int line, int side)
+side_t* getSide
+( int           currentSector,
+  int           line,
+  int           side )
 {
   return &sides[ (sectors[currentSector].lines[line])->sidenum[side] ];
 }
@@ -231,8 +223,10 @@ side_t *getSide(int currentSector, int line, int side)
 //
 // Note: if side=1 is specified, it must exist or results undefined
 //
-
-sector_t *getSector(int currentSector, int line, int side)
+sector_t* getSector
+( int           currentSector,
+  int           line,
+  int           side )
 {
   return sides[ (sectors[currentSector].lines[line])->sidenum[side] ].sector;
 }
@@ -247,8 +241,9 @@ sector_t *getSector(int currentSector, int line, int side)
 // modified to return actual two-sidedness rather than presence
 // of 2S flag unless compatibility optioned
 //
-
-int twoSided(int sector, int line)
+int twoSided
+( int   sector,
+  int   line )
 {
   //jff 1/26/98 return what is actually needed, whether the line
   //has two sidedefs, rather than whether the 2S flag is set
@@ -256,7 +251,7 @@ int twoSided(int sector, int line)
   return comp[comp_model] ?
     (sectors[sector].lines[line])->flags & ML_TWOSIDED
     :
-    (sectors[sector].lines[line])->sidenum[1] != -1;
+    (sectors[sector].lines[line])->sidenum[1] != NO_INDEX;
 }
 
 
@@ -267,8 +262,9 @@ int twoSided(int sector, int line)
 //
 // Note: returns NULL if not two-sided line, or both sides refer to sector
 //
-
-sector_t *getNextSector(line_t *line, sector_t *sec)
+sector_t* getNextSector
+( line_t*       line,
+  sector_t*     sec )
 {
   //jff 1/26/98 check unneeded since line->backsector already
   //returns NULL if the line is not two sided, and does so from
@@ -381,7 +377,11 @@ fixed_t P_FindNextHighestFloor(sector_t *sec, int currentheight)
           height = other->floorheight;
       return height;
     }
-  return currentheight;
+  /* cph - my guess at doom v1.2 - 1.4beta compatibility here.
+   * If there are no higher neighbouring sectors, Heretic just returned
+   * heightlist[0] (local variable), i.e. noise off the stack. 0 is right for
+   * RETURN01 E1M2, so let's take that. */
+  return (compatibility_level < doom_1666_compatibility ? 0 : currentheight);
 }
 
 
@@ -495,7 +495,7 @@ fixed_t P_FindLowestCeilingSurrounding(sector_t* sec)
   fixed_t             height = INT_MAX;
 
   /* jff 3/12/98 avoid ovf in height calculations */
-  if (!comp[comp_model]) height = 32000*FRACUNIT; 
+  if (!comp[comp_model]) height = 32000*FRACUNIT;
 
   for (i=0 ;i < sec->linecount ; i++)
   {
@@ -531,7 +531,7 @@ fixed_t P_FindHighestCeilingSurrounding(sector_t* sec)
   /* jff 1/26/98 Fix initial value for floor to not act differently
    * in sections of wad that are below 0 units
    * jff 3/12/98 avoid ovf in height calculations */
-  if (!comp[comp_model]) height = -32000*FRACUNIT; 
+  if (!comp[comp_model]) height = -32000*FRACUNIT;
 
   for (i=0 ;i < sec->linecount ; i++)
   {
@@ -737,17 +737,6 @@ int P_FindLineFromLineTag(const line_t *line, int start)
   return start;
 }
 
-// sf: same thing but from just a number
-
-int P_FindSectorFromTag(const int tag, int start)
-{
-  start = start >= 0 ? sectors[start].nexttag :
-    sectors[(unsigned) tag % (unsigned) numsectors].firsttag;
-  while (start >= 0 && sectors[start].tag != tag)
-    start = sectors[start].nexttag;
-  return start;
-}
-
 // Hash the sector tags across the sectors and linedefs.
 static void P_InitTagLists(void)
 {
@@ -781,8 +770,9 @@ static void P_InitTagLists(void)
 // in a surrounding sector less than that passed. If no smaller light
 // level exists, the light level passed is returned.
 //
-
-int P_FindMinSurroundingLight(sector_t *sector, int max)
+int P_FindMinSurroundingLight
+( sector_t*     sector,
+  int           max )
 {
   int         i;
   int         min;
@@ -817,8 +807,9 @@ int P_FindMinSurroundingLight(sector_t *sector, int max)
 // jff 02/05/98 routine added to test for unlockability of
 //  generalized locked doors
 //
-
-boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
+boolean P_CanUnlockGenDoor
+( line_t* line,
+  player_t* player)
 {
   // does this line special distinguish between skulls and keys?
   int skulliscard = (line->special & LockedNKeys)>>LockedNKeysShift;
@@ -837,7 +828,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         !player->cards[it_yellowskull]
       )
       {
-          player_printf(player, s_PD_ANY);
+        player->message = s_PD_ANY; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -849,7 +840,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_redskull])
       )
       {
-          player_printf(player, skulliscard? s_PD_REDK : s_PD_REDC);
+        player->message = skulliscard? s_PD_REDK : s_PD_REDC; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -861,7 +852,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_blueskull])
       )
       {
-          player_printf(player, skulliscard ? s_PD_BLUEK : s_PD_BLUEC);
+        player->message = skulliscard? s_PD_BLUEK : s_PD_BLUEC; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -873,7 +864,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_yellowskull])
       )
       {
-          player_printf(player, skulliscard? s_PD_YELLOWK : s_PD_YELLOWC);
+        player->message = skulliscard? s_PD_YELLOWK : s_PD_YELLOWC; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -885,7 +876,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_redcard])
       )
       {
-          player_printf(player, skulliscard? s_PD_REDK : s_PD_REDS);
+        player->message = skulliscard? s_PD_REDK : s_PD_REDS; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -897,7 +888,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_bluecard])
       )
       {
-          player_printf(player, skulliscard? s_PD_BLUEK : s_PD_BLUES);
+        player->message = skulliscard? s_PD_BLUEK : s_PD_BLUES; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -909,7 +900,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         (!skulliscard || !player->cards[it_yellowcard])
       )
       {
-          player_printf(player, skulliscard? s_PD_YELLOWK : s_PD_YELLOWS);
+        player->message = skulliscard? s_PD_YELLOWK : s_PD_YELLOWS; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -928,7 +919,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         )
       )
       {
-          player_printf(player, s_PD_ALL6);
+        player->message = s_PD_ALL6; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -945,7 +936,7 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
         )
       )
       {
-          player_printf(player, s_PD_ALL3);
+        player->message = s_PD_ALL3; // Ty 03/27/98 - externalized
         S_StartSound(player->mo,sfx_oof);             // killough 3/20/98
         return false;
       }
@@ -966,21 +957,21 @@ boolean P_CanUnlockGenDoor(line_t *line, player_t *player)
 // jff 2/23/98 added to prevent old demos from
 //  succeeding in starting multiple specials on one sector
 //
-int P_SectorActive(special_e t,sector_t *sec)
+boolean PUREFUNC P_SectorActive(special_e t, const sector_t *sec)
 {
   if (demo_compatibility)  // return whether any thinker is active
-    return sec->floordata || sec->ceilingdata || sec->lightingdata;
+    return sec->floordata != NULL || sec->ceilingdata != NULL || sec->lightingdata != NULL;
   else
     switch (t)             // return whether thinker of same type is active
     {
       case floor_special:
-        return (int)sec->floordata;
+        return sec->floordata != NULL;
       case ceiling_special:
-        return (int)sec->ceilingdata;
+        return sec->ceilingdata != NULL;
       case lighting_special:
-        return (int)sec->lightingdata;
+        return sec->lightingdata != NULL;
     }
-  return 1; // don't know which special, must be active, shouldn't be here
+  return true; // don't know which special, must be active, shouldn't be here
 }
 
 
@@ -1057,16 +1048,7 @@ int P_CheckTag(line_t *line)
 
     case 48:                // Scrolling walls
     case 85:
-        // sf: scripting
-    case 272:   // WR
-    case 273:
-    case 274:   // W1
-    case 275:
-    case 276:   // SR
-    case 277:   // S1
-    case 278:   // GR
-    case 279:   // G1
-      return 1;
+      return 1;   // zero tag allowed
 
     default:
       break;
@@ -1084,7 +1066,7 @@ int P_CheckTag(line_t *line)
 // jff 3/14/98 added to simplify checks for whether sector is secret
 //  in automap and other places
 //
-boolean P_IsSecret(sector_t *sec)
+boolean PUREFUNC P_IsSecret(const sector_t *sec)
 {
   return (sec->special==9 || (sec->special&SECRET_MASK));
 }
@@ -1099,7 +1081,7 @@ boolean P_IsSecret(sector_t *sec)
 // jff 3/14/98 added to simplify checks for whether sector is secret
 //  in automap and other places
 //
-boolean P_WasSecret(sector_t *sec)
+boolean PUREFUNC P_WasSecret(const sector_t *sec)
 {
   return (sec->oldspecial==9 || (sec->oldspecial&SECRET_MASK));
 }
@@ -1157,7 +1139,11 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
     int (*linefunc)(line_t *line)=NULL;
 
     // check each range of generalized linedefs
-    if ((unsigned)line->special >= GenFloorBase)
+    if ((unsigned)line->special >= GenEnd)
+    {
+      // Out of range for GenFloors
+    }
+    else if ((unsigned)line->special >= GenFloorBase)
     {
       if (!thing->player)
         if ((line->special & FloorChange) || !(line->special & FloorModel))
@@ -1421,7 +1407,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
       // EXIT!
       // killough 10/98: prevent zombies from exiting levels
       if (!(thing->player && thing->player->health <= 0 && !comp[comp_zombie]))
-	G_ExitLevel ();
+  G_ExitLevel ();
       break;
 
     case 53:
@@ -1507,7 +1493,7 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
       // killough 10/98: prevent zombies from exiting levels
       // CPhipps - change for lxdoom's compatibility handling
       if (!(thing->player && thing->player->health <= 0 && !comp[comp_zombie]))
-	G_SecretExitLevel ();
+  G_SecretExitLevel ();
       break;
 
     case 125:
@@ -1692,29 +1678,6 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
     case 129:
       // Raise Floor Turbo
       EV_DoFloor(line,raiseFloorTurbo);
-      break;
-
-        // scripting ld types
-
-        // repeatable
-
-    case 273:  // console command (1sided)
-      if(side) break;
-
-    case 272:  // console command (2sided)
-      t_trigger = thing;
-      T_RunScript(line->tag);
-      break;
-
-        // once-only triggers
-
-    case 275:  // console command (1sided)
-      if(side) break;
-
-    case 274:  // console command (2sided)
-      t_trigger = thing;
-      T_RunScript(line->tag);
-      line->special = 0;        // clear trigger
       break;
 
       // Extended walk triggers
@@ -2037,7 +2000,9 @@ void P_CrossSpecialLine(line_t *line, int side, mobj_t *thing)
 // of the line, should the sector already be in motion when the line is
 // impacted. Change is qualified by demo_compatibility.
 //
-void P_ShootSpecialLine(mobj_t *thing, line_t *line)
+void P_ShootSpecialLine
+( mobj_t*       thing,
+  line_t*       line )
 {
   //jff 02/04/98 add check here for generalized linedef
   if (!demo_compatibility)
@@ -2047,7 +2012,11 @@ void P_ShootSpecialLine(mobj_t *thing, line_t *line)
     int (*linefunc)(line_t *line)=NULL;
 
     // check each range of generalized linedefs
-    if ((unsigned)line->special >= GenFloorBase)
+    if ((unsigned)line->special >= GenEnd)
+    {
+      // Out of range for GenFloors
+    }
+    else if ((unsigned)line->special >= GenFloorBase)
     {
       if (!thing->player)
         if ((line->special & FloorChange) || !(line->special & FloorModel))
@@ -2178,14 +2147,6 @@ void P_ShootSpecialLine(mobj_t *thing, line_t *line)
     //jff 1/30/98 added new gun linedefs here
     // killough 1/31/98: added demo_compatibility check, added inner switch
 
-        // sf: scripting
-    case 278:
-    case 279:
-      t_trigger = thing;
-      T_RunScript(line->tag);
-      if(line->special == 279) line->special = 0;       // clear if G1
-      break;
-
     default:
       if (!demo_compatibility)
         switch (line->special)
@@ -2224,7 +2185,9 @@ void P_ShootSpecialLine(mobj_t *thing, line_t *line)
 //
 void P_PlayerInSpecialSector (player_t* player)
 {
-  sector_t *sector = player->mo->subsector->sector;
+  sector_t*   sector;
+
+  sector = player->mo->subsector->sector;
 
   // Falling, not all the way down yet?
   // Sector specials don't apply in mid-air
@@ -2340,8 +2303,8 @@ void P_PlayerInSpecialSector (player_t* player)
 //  levelFragLimit, levelFragLimitCount
 //
 
-boolean         levelTimer;
-int             levelTimeCount;
+static boolean  levelTimer;
+static int      levelTimeCount;
 boolean         levelFragLimit;      // Ty 03/18/98 Added -frags support
 int             levelFragLimitCount; // Ty 03/18/98 Added -frags support
 
@@ -3060,7 +3023,7 @@ static void Add_Pusher(int type, int x_mag, int y_mag, mobj_t* source, int affec
 
 pusher_t* tmpusher; // pusher structure for blockmap searches
 
-boolean PIT_PushThing(mobj_t* thing)
+static boolean PIT_PushThing(mobj_t* thing)
 {
   /* killough 10/98: made more general */
   if (!mbf_features ?
@@ -3300,5 +3263,3 @@ static void P_SpawnPushers(void)
 // phares 3/20/98: End of Pusher effects
 //
 ////////////////////////////////////////////////////////////////////////////
-
-
