@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -37,6 +39,7 @@
 #include "p_map.h"
 #include "p_spec.h"
 #include "p_user.h"
+#include "r_demo.h"
 
 // Index of the special effects (INVUL inverse) map.
 
@@ -76,7 +79,7 @@ void P_Thrust(player_t* player,angle_t angle,fixed_t move)
  * reduced at a regular rate, even on ice (where the player coasts).
  */
 
-void P_Bob(player_t *player, angle_t angle, fixed_t move)
+static void P_Bob(player_t *player, angle_t angle, fixed_t move)
 {
   player->momx += FixedMul(move,finecosine[angle >>= ANGLETOFINESHIFT]);
   player->momy += FixedMul(move,finesine[angle]);
@@ -106,13 +109,13 @@ void P_CalcHeight (player_t* player)
    * it causes bobbing jerkiness when the player moves from ice to non-ice,
    * and vice-versa.
    */
-  player->bob = !mbf_features ? 
+  player->bob = !mbf_features ?
     (FixedMul (player->mo->momx, player->mo->momx)
      + FixedMul (player->mo->momy,player->mo->momy))>>2 :
-    player_bobbing ? (FixedMul(player->momx,player->momx) + 
+    player_bobbing ? (FixedMul(player->momx,player->momx) +
         FixedMul(player->momy,player->momy))>>2 : 0;
 
-  if (player->bob > MAXBOB)                             
+  if (player->bob > MAXBOB)
     player->bob = MAXBOB;
 
   if (!onground || player->cheats & CF_NOMOMENTUM)
@@ -182,6 +185,10 @@ void P_MovePlayer (player_t* player)
   mo->angle += cmd->angleturn << 16;
   onground = mo->z <= mo->floorz;
 
+  // e6y
+  if (demo_smoothturns && player == &players[displayplayer])
+    R_SmoothPlaying_Add(cmd->angleturn << 16);
+
   // killough 10/98:
   //
   // We must apply thrust to the player and bobbing separately, to avoid
@@ -192,30 +199,30 @@ void P_MovePlayer (player_t* player)
   if (cmd->forwardmove | cmd->sidemove) // killough 10/98
     {
       if (onground || mo->flags & MF_BOUNCES) // killough 8/9/98
-    	{
-    	  int friction, movefactor = P_GetMoveFactor(mo, &friction);
+      {
+        int friction, movefactor = P_GetMoveFactor(mo, &friction);
 
-	      // killough 11/98:
-	      // On sludge, make bobbing depend on efficiency.
-	      // On ice, make it depend on effort.
+        // killough 11/98:
+        // On sludge, make bobbing depend on efficiency.
+        // On ice, make it depend on effort.
 
-	      int bobfactor =
-	        friction < ORIG_FRICTION ? movefactor : ORIG_FRICTION_FACTOR;
+        int bobfactor =
+          friction < ORIG_FRICTION ? movefactor : ORIG_FRICTION_FACTOR;
 
-	      if (cmd->forwardmove)
-	      {
-	        P_Bob(player,mo->angle,cmd->forwardmove*bobfactor);
-	        P_Thrust(player,mo->angle,cmd->forwardmove*movefactor);
-	      }
+        if (cmd->forwardmove)
+        {
+          P_Bob(player,mo->angle,cmd->forwardmove*bobfactor);
+          P_Thrust(player,mo->angle,cmd->forwardmove*movefactor);
+        }
 
-	      if (cmd->sidemove)
-	      {
-	        P_Bob(player,mo->angle-ANG90,cmd->sidemove*bobfactor);
-	        P_Thrust(player,mo->angle-ANG90,cmd->sidemove*movefactor);
-	      }
-	    }
+        if (cmd->sidemove)
+        {
+          P_Bob(player,mo->angle-ANG90,cmd->sidemove*bobfactor);
+          P_Thrust(player,mo->angle-ANG90,cmd->sidemove*movefactor);
+        }
+      }
       if (mo->state == states+S_PLAY)
-      	P_SetMobjState(mo,S_PLAY_RUN1);
+        P_SetMobjState(mo,S_PLAY_RUN1);
     }
 }
 
@@ -275,6 +282,7 @@ void P_DeathThink (player_t* player)
 
   if (player->cmd.buttons & BT_USE)
     player->playerstate = PST_REBORN;
+  R_SmoothPlaying_Reset(player); // e6y
   }
 
 
