@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -48,12 +50,13 @@ extern sector_t *backsector;
 extern drawseg_t *drawsegs;
 extern unsigned maxdrawsegs;
 
+extern byte solidcol[MAX_SCREENWIDTH];
+
 extern drawseg_t *ds_p;
 
 void R_ClearClipSegs(void);
 void R_ClearDrawSegs(void);
 void R_RenderBSPNode(int bspnum);
-int R_DoorClosed(void);   /* killough 1/17/98 */
 
 /* killough 4/13/98: fake floors/ceilings for deep water / fake ceilings: */
 sector_t *R_FakeFlat(sector_t *, sector_t *, int *, int *, boolean);
