@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -55,6 +57,7 @@
 #include "r_things.h"
 #include "r_sky.h"
 #include "r_plane.h"
+#include "v_video.h"
 #include "lprintf.h"
 
 #define MAXVISPLANES 128    /* must be a power of 2 */
@@ -71,13 +74,13 @@ visplane_t *floorplane, *ceilingplane;
   ((unsigned)((picnum)*3+(lightlevel)+(height)*7) & (MAXVISPLANES-1))
 
 size_t maxopenings;
-short *openings,*lastopening;
+int *openings,*lastopening; // dropoff overflow
 
 // Clip values are the solid pixel bounding the range.
 //  floorclip starts out SCREENHEIGHT
 //  ceilingclip starts out -1
 
-short floorclip[MAX_SCREENWIDTH], ceilingclip[MAX_SCREENWIDTH];
+int floorclip[MAX_SCREENWIDTH], ceilingclip[MAX_SCREENWIDTH]; // dropoff overflow
 
 // spanstart holds the start of a plane span; initialized to 0 at start
 
@@ -87,7 +90,7 @@ static int spanstart[MAX_SCREENHEIGHT];                // killough 2/8/98
 // texture mapping
 //
 
-static lighttable_t **planezlight;
+static const lighttable_t **planezlight;
 static fixed_t planeheight;
 
 // killough 2/8/98: make variables static
@@ -114,7 +117,7 @@ void R_InitPlanes (void)
 //
 // Uses global vars:
 //  planeheight
-//  ds_source
+//  dsvars.source
 //  basexscale
 //  baseyscale
 //  viewx
@@ -140,38 +143,37 @@ static void R_MapPlane(int y, int x1, int x2)
     {
       cachedheight[y] = planeheight;
       distance = cacheddistance[y] = FixedMul (planeheight, yslope[y]);
-      ds_xstep = cachedxstep[y] = FixedMul (distance,basexscale);
-      ds_ystep = cachedystep[y] = FixedMul (distance,baseyscale);
+      dsvars.xstep = cachedxstep[y] = FixedMul (distance,basexscale);
+      dsvars.ystep = cachedystep[y] = FixedMul (distance,baseyscale);
     }
   else
     {
       distance = cacheddistance[y];
-      ds_xstep = cachedxstep[y];
-      ds_ystep = cachedystep[y];
+      dsvars.xstep = cachedxstep[y];
+      dsvars.ystep = cachedystep[y];
     }
 
   length = FixedMul (distance,distscale[x1]);
   angle = (viewangle + xtoviewangle[x1])>>ANGLETOFINESHIFT;
 
   // killough 2/28/98: Add offsets
-  ds_xfrac =  viewx + FixedMul(finecosine[angle], length) + xoffs;
-  ds_yfrac = -viewy - FixedMul(finesine[angle],   length) + yoffs;
+  dsvars.xfrac =  viewx + FixedMul(finecosine[angle], length) + xoffs;
+  dsvars.yfrac = -viewy - FixedMul(finesine[angle],   length) + yoffs;
 
-  if (!(ds_colormap = fixedcolormap))
+  if (!(dsvars.colormap = fixedcolormap))
     {
       index = distance >> LIGHTZSHIFT;
       if (index >= MAXLIGHTZ )
         index = MAXLIGHTZ-1;
-      ds_colormap = planezlight[index];
+      dsvars.colormap = planezlight[index];
     }
 
-  ds_y = y;
-  ds_x1 = x1;
-  ds_x2 = x2;
+  dsvars.y = y;
+  dsvars.x1 = x1;
+  dsvars.x2 = x2;
 
-#ifndef GL_DOOM
-  R_DrawSpan();
-#endif
+  if (V_GetMode() != VID_MODEGL)
+    R_DrawSpan();
 }
 
 //
@@ -182,7 +184,6 @@ static void R_MapPlane(int y, int x1, int x2)
 void R_ClearPlanes(void)
 {
   int i;
-  angle_t angle;
 
   // opening / clipping determination
   for (i=0 ; i<viewwidth ; i++)
@@ -197,12 +198,9 @@ void R_ClearPlanes(void)
   // texture calculation
   memset (cachedheight, 0, sizeof(cachedheight));
 
-  // left to right mapping
-  angle = (viewangle-ANG90)>>ANGLETOFINESHIFT;
-
   // scale will be unit scale at SCREENWIDTH/2 distance
-  basexscale = FixedDiv (finecosine[angle],centerxfrac);
-  baseyscale = -FixedDiv (finesine[angle],centerxfrac);
+  basexscale = FixedDiv (viewsin,projection);
+  baseyscale = FixedDiv (viewcos,projection);
 }
 
 // New function, by Lee Killough
@@ -297,7 +295,7 @@ visplane_t *R_CheckPlane(visplane_t *pl, int start, int stop)
   else
     unionh  = pl->maxx, intrh  = stop;
 
-  for (x=intrl ; x <= intrh && pl->top[x] == 0xffff; x++)
+  for (x=intrl ; x <= intrh && pl->top[x] == 0xffffffffu; x++) // dropoff overflow
     ;
 
   if (x > intrh) { /* Can use existing plane; extend range */
@@ -311,7 +309,7 @@ visplane_t *R_CheckPlane(visplane_t *pl, int start, int stop)
 // R_MakeSpans
 //
 
-static void R_MakeSpans(int x, int t1, int b1, int t2, int b2)
+static void R_MakeSpans(int x, unsigned int t1, unsigned int b1, unsigned int t2, unsigned int b2) // dropoff overflow
 {
   for (; t1 < t2 && t1 <= b1; t1++)
     R_MapPlane(t1, spanstart[t1], x-1);
@@ -331,6 +329,7 @@ static void R_DoDrawPlane(visplane_t *pl)
   if (pl->minx <= pl->maxx) {
     if (pl->picnum == skyflatnum || pl->picnum & PL_SKYFLAT) { // sky flat
       int texture;
+      const rpatch_t *tex_patch;
       angle_t an, flip;
 
       // killough 10/98: allow skies to come from sidedefs.
@@ -360,7 +359,7 @@ static void R_DoDrawPlane(visplane_t *pl)
 
         // Vertical offset allows careful sky positioning.
 
-        dc_texturemid = s->rowoffset - 28*FRACUNIT;
+        dcvars.texturemid = s->rowoffset - 28*FRACUNIT;
 
         // We sometimes flip the picture horizontally.
         //
@@ -372,7 +371,7 @@ static void R_DoDrawPlane(visplane_t *pl)
       }
       else
       {    // Normal Doom sky, only one allowed per level
-        dc_texturemid = skytexturemid;    // Default y-offset
+        dcvars.texturemid = skytexturemid;    // Default y-offset
         texture = skytexture;             // Default texture
         flip = 0;                         // Doom flips it
       }
@@ -381,26 +380,30 @@ static void R_DoDrawPlane(visplane_t *pl)
        * Because of this hack, sky is not affected by INVUL inverse mapping.
        * Until Boom fixed this. Compat option added in MBF. */
 
-      if (comp[comp_skymap] || !(dc_colormap = fixedcolormap))
-	dc_colormap = fullcolormap;          // killough 3/20/98
-      //dc_texturemid = skytexturemid;
-      dc_texheight = textureheight[skytexture]>>FRACBITS; // killough
+      if (comp[comp_skymap] || !(dcvars.colormap = fixedcolormap))
+        dcvars.colormap = fullcolormap;          // killough 3/20/98
+      //dcvars.texturemid = skytexturemid;
+      dcvars.texheight = textureheight[skytexture]>>FRACBITS; // killough
       // proff 09/21/98: Changed for high-res
-      dc_iscale = FRACUNIT*200/viewheight;
+      dcvars.iscale = FRACUNIT*200/viewheight;
+
+      tex_patch = R_CacheTextureCompositePatchNum(texture);
 
   // killough 10/98: Use sky scrolling offset, and possibly flip picture
-        for (x = pl->minx; (dc_x = x) <= pl->maxx; x++)
-          if ((dc_yl = pl->top[x]) <= (dc_yh = pl->bottom[x]))
+        for (x = pl->minx; (dcvars.x = x) <= pl->maxx; x++)
+          if ((dcvars.yl = pl->top[x]) != -1 && dcvars.yl <= (dcvars.yh = pl->bottom[x])) // dropoff overflow
             {
-              dc_source = R_GetColumn(texture, ((an + xtoviewangle[x])^flip) >>
-              ANGLETOSKYSHIFT);
+              dcvars.source = R_GetTextureColumn(tex_patch, ((an + xtoviewangle[x])^flip) >> ANGLETOSKYSHIFT);
               colfunc();
             }
+
+      R_UnlockTextureCompositePatchNum(texture);
+
     } else {     // regular flat
 
       int stop, light;
 
-      ds_source = W_CacheLumpNum(firstflat + flattranslation[pl->picnum]);
+      dsvars.source = W_CacheLumpNum(firstflat + flattranslation[pl->picnum]);
 
       xoffs = pl->xoffs;  // killough 2/28/98: Add offsets
       yoffs = pl->yoffs;
@@ -415,7 +418,7 @@ static void R_DoDrawPlane(visplane_t *pl)
 
       stop = pl->maxx + 1;
       planezlight = zlight[light];
-      pl->top[pl->minx-1] = pl->top[stop] = 0xffff;
+      pl->top[pl->minx-1] = pl->top[stop] = 0xffffffffu; // dropoff overflow
 
       for (x = pl->minx ; x <= stop ; x++)
   R_MakeSpans(x,pl->top[x-1],pl->bottom[x-1],pl->top[x],pl->bottom[x]);
