@@ -2,12 +2,14 @@
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2006 by
+ *  Copyright (C) 1999-2004 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
  *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
@@ -34,14 +36,13 @@
 #include "doomstat.h"
 #include "r_main.h"
 #include "r_bsp.h"
+#include "r_segs.h"
 #include "r_plane.h"
 #include "r_things.h"
 #include "r_draw.h"
 #include "w_wad.h"
+#include "v_video.h"
 #include "lprintf.h"
-#ifdef GL_DOOM
-#include "gl_struct.h"
-#endif
 
 // OPTIMIZE: closed two sided lines as single sided
 
@@ -61,7 +62,6 @@ static fixed_t  toptexheight, midtexheight, bottomtexheight; // cph
 angle_t         rw_normalangle; // angle to line origin
 int             rw_angle1;
 fixed_t         rw_distance;
-lighttable_t    **walllights;
 
 //
 // regular wall
@@ -75,6 +75,7 @@ static fixed_t  rw_scalestep;
 static fixed_t  rw_midtexturemid;
 static fixed_t  rw_toptexturemid;
 static fixed_t  rw_bottomtexturemid;
+static int      rw_lightlevel;
 static int      worldtop;
 static int      worldbottom;
 static int      worldhigh;
@@ -87,7 +88,7 @@ static fixed_t  topfrac;
 static fixed_t  topstep;
 static fixed_t  bottomfrac;
 static fixed_t  bottomstep;
-static short    *maskedtexturecol;
+static int      *maskedtexturecol; // dropoff overflow
 
 //
 // R_ScaleFromGlobalAngle
@@ -114,14 +115,11 @@ static fixed_t R_ScaleFromGlobalAngle(angle_t visangle)
 // R_RenderMaskedSegRange
 //
 
-int fake_contrast;
-
 void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 {
-  column_t *col;
-  int      lightnum;
   int      texnum;
   sector_t tempsec;      // killough 4/13/98
+  const rpatch_t *patch;
 
   // Calculate light table.
   // Use different light tables
@@ -150,21 +148,7 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
     texnum = texturetranslation[texnum];
 
   // killough 4/13/98: get correct lightlevel for 2s normal textures
-  lightnum = (R_FakeFlat(frontsector, &tempsec, NULL, NULL, false)
-              ->lightlevel >> LIGHTSEGSHIFT)+extralight;
-
-  /* cph - ...what is this for? adding contrast to rooms?
-   * It looks crap in outdoor areas */
-  if (fake_contrast) {
-    if (curline->v1->y == curline->v2->y)
-      lightnum--;
-    else
-      if (curline->v1->x == curline->v2->x)
-        lightnum++;
-  }
-
-  walllights = lightnum >= LIGHTLEVELS ? scalelight[LIGHTLEVELS-1] :
-    lightnum <  0           ? scalelight[0] : scalelight[lightnum];
+  rw_lightlevel = R_FakeFlat(frontsector, &tempsec, NULL, NULL, false) ->lightlevel;
 
   maskedtexturecol = ds->maskedtexturecol;
 
@@ -176,41 +160,35 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
   // find positioning
   if (curline->linedef->flags & ML_DONTPEGBOTTOM)
     {
-      dc_texturemid = frontsector->floorheight > backsector->floorheight
+      dcvars.texturemid = frontsector->floorheight > backsector->floorheight
         ? frontsector->floorheight : backsector->floorheight;
-      dc_texturemid = dc_texturemid + textureheight[texnum] - viewz;
+      dcvars.texturemid = dcvars.texturemid + textureheight[texnum] - viewz;
     }
   else
     {
-      dc_texturemid =frontsector->ceilingheight<backsector->ceilingheight
+      dcvars.texturemid =frontsector->ceilingheight<backsector->ceilingheight
         ? frontsector->ceilingheight : backsector->ceilingheight;
-      dc_texturemid = dc_texturemid - viewz;
+      dcvars.texturemid = dcvars.texturemid - viewz;
     }
 
-  dc_texturemid += curline->sidedef->rowoffset;
+  dcvars.texturemid += curline->sidedef->rowoffset;
 
   if (fixedcolormap)
-    dc_colormap = fixedcolormap;
+    dcvars.colormap = fixedcolormap;
+
+  patch = R_CacheTextureCompositePatchNum(texnum);
 
   // draw the columns
-  for (dc_x = x1 ; dc_x <= x2 ; dc_x++, spryscale += rw_scalestep)
-    if (maskedtexturecol[dc_x] != SHRT_MAX)
+  for (dcvars.x = x1 ; dcvars.x <= x2 ; dcvars.x++, spryscale += rw_scalestep)
+    if (maskedtexturecol[dcvars.x] != INT_MAX) // dropoff overflow
       {
-        if (!fixedcolormap)      // calculate lighting
-          {
-            unsigned index = spryscale>>LIGHTSCALESHIFT;
-
-            if (index >=  MAXLIGHTSCALE )
-              index = MAXLIGHTSCALE-1;
-
-            dc_colormap = walllights[index];
-          }
+        dcvars.colormap = R_ColourMap(rw_lightlevel,spryscale);
 
         // killough 3/2/98:
         //
         // This calculation used to overflow and cause crashes in Doom:
         //
-        // sprtopscreen = centeryfrac - FixedMul(dc_texturemid, spryscale);
+        // sprtopscreen = centeryfrac - FixedMul(dcvars.texturemid, spryscale);
         //
         // This code fixes it, by using double-precision intermediate
         // arithmetic and by skipping the drawing of 2s normals whose
@@ -218,14 +196,14 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 
         {
           int_64_t t = ((int_64_t) centeryfrac << FRACBITS) -
-            (int_64_t) dc_texturemid * spryscale;
+            (int_64_t) dcvars.texturemid * spryscale;
           if (t + (int_64_t) textureheight[texnum] * spryscale < 0 ||
               t > (int_64_t) MAX_SCREENHEIGHT << FRACBITS*2)
             continue;        // skip if the texture is out of screen's range
           sprtopscreen = (long)(t >> FRACBITS);
         }
 
-        dc_iscale = 0xffffffffu / (unsigned) spryscale;
+        dcvars.iscale = 0xffffffffu / (unsigned) spryscale;
 
         // killough 1/25/98: here's where Medusa came in, because
         // it implicitly assumed that the column was all one patch.
@@ -236,15 +214,21 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
         // when forming multipatched textures (see r_data.c).
 
         // draw the texture
-        col = (column_t *)((byte *)
-                           R_GetColumn(texnum,maskedtexturecol[dc_x]) - 3);
-        R_DrawMaskedColumn (col);
-        maskedtexturecol[dc_x] = SHRT_MAX;
+        R_DrawMaskedColumn(
+          patch, 
+          R_GetPatchColumnWrapped(patch, maskedtexturecol[dcvars.x])
+        );
+
+        maskedtexturecol[dcvars.x] = INT_MAX; // dropoff overflow
       }
 
   // Except for main_tranmap, mark others purgable at this point
   if (curline->linedef->tranlump > 0 && general_translucency)
     W_UnlockLumpNum(curline->linedef->tranlump-1); // cph - unlock it
+
+  R_UnlockTextureCompositePatchNum(texnum);
+
+  curline = NULL; /* cph 2001/11/18 - must clear curline now we're done with it, so R_ColourMap doesn't try using it for other things */
 }
 
 //
@@ -254,13 +238,13 @@ void R_RenderMaskedSegRange(drawseg_t *ds, int x1, int x2)
 // CALLED: CORE LOOPING ROUTINE.
 //
 
-extern byte solidcol[MAX_SCREENWIDTH];
 #define HEIGHTBITS 12
 #define HEIGHTUNIT (1<<HEIGHTBITS)
 static int didsolidcol; /* True if at least one column was marked solid */
 
 static void R_RenderSegLoop (void)
 {
+  const rpatch_t *tex_patch;
   fixed_t  texturecolumn = 0;   // shut up compiler warning
   rendered_segs++;
   for ( ; rw_x < rw_stopx ; rw_x++)
@@ -312,34 +296,30 @@ static void R_RenderSegLoop (void)
       // texturecolumn and lighting are independent of wall tiers
       if (segtextured)
         {
-          unsigned index;
-
           // calculate texture offset
           angle_t angle =(rw_centerangle+xtoviewangle[rw_x])>>ANGLETOFINESHIFT;
 
           texturecolumn = rw_offset-FixedMul(finetangent[angle],rw_distance);
           texturecolumn >>= FRACBITS;
-          // calculate lighting
-          index = rw_scale>>LIGHTSCALESHIFT;
 
-          if (index >=  MAXLIGHTSCALE )
-            index = MAXLIGHTSCALE-1;
-
-          dc_colormap = walllights[index];
-          dc_x = rw_x;
-          dc_iscale = 0xffffffffu / (unsigned)rw_scale;
+          dcvars.colormap = R_ColourMap(rw_lightlevel,rw_scale);
+          dcvars.x = rw_x;
+          dcvars.iscale = 0xffffffffu / (unsigned)rw_scale;
         }
 
       // draw the wall tiers
       if (midtexture)
         {
 
-          dc_yl = yl;     // single sided line
-          dc_yh = yh;
-          dc_texturemid = rw_midtexturemid;
-          dc_source = R_GetColumn(midtexture, texturecolumn);
-    dc_texheight = midtexheight;
+          dcvars.yl = yl;     // single sided line
+          dcvars.yh = yh;
+          dcvars.texturemid = rw_midtexturemid;
+          tex_patch = R_CacheTextureCompositePatchNum(midtexture);
+          dcvars.source = R_GetTextureColumn(tex_patch, texturecolumn);
+          dcvars.texheight = midtexheight;
           colfunc ();
+          R_UnlockTextureCompositePatchNum(midtexture);
+          tex_patch = NULL;
           ceilingclip[rw_x] = viewheight;
           floorclip[rw_x] = -1;
         }
@@ -358,12 +338,15 @@ static void R_RenderSegLoop (void)
 
               if (mid >= yl)
                 {
-                  dc_yl = yl;
-                  dc_yh = mid;
-                  dc_texturemid = rw_toptexturemid;
-                  dc_source = R_GetColumn(toptexture,texturecolumn);
-      dc_texheight = toptexheight;
+                  dcvars.yl = yl;
+                  dcvars.yh = mid;
+                  dcvars.texturemid = rw_toptexturemid;
+                  tex_patch = R_CacheTextureCompositePatchNum(toptexture);
+                  dcvars.source = R_GetTextureColumn(tex_patch,texturecolumn);
+                  dcvars.texheight = toptexheight;
                   colfunc ();
+                  R_UnlockTextureCompositePatchNum(toptexture);
+                  tex_patch = NULL;
                   ceilingclip[rw_x] = mid;
                 }
               else
@@ -387,13 +370,15 @@ static void R_RenderSegLoop (void)
 
               if (mid <= yh)
                 {
-                  dc_yl = mid;
-                  dc_yh = yh;
-                  dc_texturemid = rw_bottomtexturemid;
-                  dc_source = R_GetColumn(bottomtexture,
-                                          texturecolumn);
-                  dc_texheight = bottomtexheight;
+                  dcvars.yl = mid;
+                  dcvars.yh = yh;
+                  dcvars.texturemid = rw_bottomtexturemid;
+                  tex_patch = R_CacheTextureCompositePatchNum(bottomtexture);
+                  dcvars.source = R_GetTextureColumn(tex_patch, texturecolumn);
+                  dcvars.texheight = bottomtexheight;
                   colfunc ();
+                  R_UnlockTextureCompositePatchNum(bottomtexture);
+                  tex_patch = NULL;
                   floorclip[rw_x] = mid;
                 }
               else
@@ -449,8 +434,7 @@ static fixed_t R_PointToDist(fixed_t x, fixed_t y)
 void R_StoreWallRange(const int start, const int stop)
 {
   fixed_t hyp;
-  fixed_t sineval;
-  angle_t distangle, offsetangle;
+  angle_t offsetangle;
 
   if (ds_p == drawsegs+maxdrawsegs)   // killough 1/98 -- fix 2s line HOM
     {
@@ -465,10 +449,14 @@ void R_StoreWallRange(const int start, const int stop)
     curline->linedef->flags |= ML_MAPPED;
 
 #ifdef GL_DOOM
-  // proff 11/99: the rest of the calculations is not needed for OpenGL
-  ds_p++->curline = curline;
-  gld_AddWall(curline);
-  return;
+  if (V_GetMode() == VID_MODEGL)
+  {
+    // proff 11/99: the rest of the calculations is not needed for OpenGL
+    ds_p++->curline = curline;
+    gld_AddWall(curline);
+
+    return;
+  }
 #endif
 
 
@@ -486,16 +474,14 @@ void R_StoreWallRange(const int start, const int stop)
   // calculate rw_distance for scale calculation
   rw_normalangle = curline->angle + ANG90;
 
-  offsetangle = D_abs(rw_normalangle-rw_angle1);
+  offsetangle = rw_normalangle-rw_angle1;
 
-  if (offsetangle > ANG90)
+  if (D_abs(offsetangle) > ANG90)
     offsetangle = ANG90;
 
-  distangle = ANG90 - offsetangle;
   hyp = (viewx==curline->v1->x && viewy==curline->v1->y)?
     0 : R_PointToDist (curline->v1->x, curline->v1->y);
-  sineval = finesine[distangle>>ANGLETOFINESHIFT];
-  rw_distance = FixedMul(hyp, sineval);
+  rw_distance = FixedMul(hyp, finecosine[offsetangle>>ANGLETOFINESHIFT]);
 
   ds_p->x1 = rw_x = start;
   ds_p->x2 = stop;
@@ -503,15 +489,15 @@ void R_StoreWallRange(const int start, const int stop)
   rw_stopx = stop+1;
 
   {     // killough 1/6/98, 2/1/98: remove limit on openings
-    extern short *openings;
+    extern int *openings; // dropoff overflow
     extern size_t maxopenings;
     size_t pos = lastopening - openings;
     size_t need = (rw_stopx - start)*4 + pos;
     if (need > maxopenings)
       {
         drawseg_t *ds;                //jff 8/9/98 needed for fix from ZDoom
-        short *oldopenings = openings;
-        short *oldlast = lastopening;
+        int *oldopenings = openings; // dropoff overflow
+        int *oldlast = lastopening; // dropoff overflow
 
         do
           maxopenings = maxopenings ? maxopenings*2 : 16384;
@@ -706,48 +692,13 @@ void R_StoreWallRange(const int start, const int stop)
 
   if (segtextured)
     {
-      offsetangle = rw_normalangle-rw_angle1;
-
-      if (offsetangle > ANG180)
-        offsetangle = 0-offsetangle;
-
-      if (offsetangle > ANG90)
-        offsetangle = ANG90;
-
-      sineval = finesine[offsetangle >>ANGLETOFINESHIFT];
-      rw_offset = FixedMul (hyp, sineval);
-
-      if (rw_normalangle-rw_angle1 < ANG180)
-        rw_offset = -rw_offset;
+      rw_offset = FixedMul (hyp, -finesine[offsetangle >>ANGLETOFINESHIFT]);
 
       rw_offset += sidedef->textureoffset + curline->offset;
 
       rw_centerangle = ANG90 + viewangle - rw_normalangle;
 
-      // calculate light table
-      //  use different light tables
-      //  for horizontal / vertical / diagonal
-      // OPTIMIZE: get rid of LIGHTSEGSHIFT globally
-      if (!fixedcolormap)
-        {
-          int lightnum = (frontsector->lightlevel >> LIGHTSEGSHIFT)+extralight;
-
-    /* cph - ...what is this for? adding contrast to rooms?
-     * It looks crap in outdoor areas */
-    if (fake_contrast) {
-          if (curline->v1->y == curline->v2->y)
-            lightnum--;
-          else if (curline->v1->x == curline->v2->x)
-            lightnum++;
-    }
-
-          if (lightnum < 0)
-            walllights = scalelight[0];
-          else if (lightnum >= LIGHTLEVELS)
-            walllights = scalelight[LIGHTLEVELS-1];
-          else
-            walllights = scalelight[lightnum];
-        }
+      rw_lightlevel = frontsector->lightlevel;
     }
 
   // if a floor / ceiling plane is on the wrong side of the view
@@ -832,13 +783,13 @@ void R_StoreWallRange(const int start, const int stop)
   // save sprite clipping info
   if ((ds_p->silhouette & SIL_TOP || maskedtexture) && !ds_p->sprtopclip)
     {
-      memcpy (lastopening, ceilingclip+start, 2*(rw_stopx-start));
+      memcpy (lastopening, ceilingclip+start, sizeof(int)*(rw_stopx-start)); // dropoff overflow
       ds_p->sprtopclip = lastopening - start;
       lastopening += rw_stopx - start;
     }
   if ((ds_p->silhouette & SIL_BOTTOM || maskedtexture) && !ds_p->sprbottomclip)
     {
-      memcpy (lastopening, floorclip+start, 2*(rw_stopx-start));
+      memcpy (lastopening, floorclip+start, sizeof(int)*(rw_stopx-start)); // dropoff overflow
       ds_p->sprbottomclip = lastopening - start;
       lastopening += rw_stopx - start;
     }
