@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
- *  Copyright (C) 1999-2001 by
+ *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -29,7 +31,6 @@
  *
  *-----------------------------------------------------------------------------*/
 
-#include "z_zone.h"
 #include "doomstat.h"
 #include "w_wad.h"
 #include "r_main.h"
@@ -67,9 +68,8 @@ fixed_t pspriteyscale;
 // constant arrays
 //  used for psprite clipping and initializing clipping
 
-short negonearray[MAX_SCREENWIDTH];        // killough 2/8/98:
-short screenheightarray[MAX_SCREENWIDTH];  // change to MAX_*
-int lefthanded=0;
+int negonearray[MAX_SCREENWIDTH];        // killough 2/8/98: // dropoff overflow
+int screenheightarray[MAX_SCREENWIDTH];  // change to MAX_* // dropoff overflow
 
 //
 // INITIALIZATION FUNCTIONS
@@ -77,7 +77,7 @@ int lefthanded=0;
 
 // variables used to look up and range check thing_t sprites patches
 
-spritedef_t *sprites = NULL;
+spritedef_t *sprites;
 int numsprites;
 
 #define MAX_SPRITE_FRAMES 29          /* Macroized -- killough 1/25/98 */
@@ -122,22 +122,6 @@ static void R_InstallSpriteLump(int lump, unsigned frame,
     }
 }
 
-void R_FreeSpriteDefs(void)
-{
-  int i;
-
-  if (!sprites)
-    return;
-  for (i=0 ; i<numsprites ; i++)
-    if (sprites[i].spriteframes)
-    {
-      Z_Free(sprites[i].spriteframes);
-      sprites[i].spriteframes = NULL;
-    }
-  Z_Free(sprites);
-  sprites = NULL;
-}
-
 //
 // R_InitSpriteDefs
 // Pass a null terminated list of sprite names
@@ -164,7 +148,7 @@ void R_FreeSpriteDefs(void)
 
 #define R_SpriteNameHash(s) ((unsigned)((s)[0]-((s)[1]*3-(s)[3]*2-(s)[2])*2))
 
-void R_InitSpriteDefs(const char * const * namelist)
+static void R_InitSpriteDefs(const char * const * namelist)
 {
   size_t numentries = lastspritelump-firstspritelump+1;
   struct { int index, next; } *hash;
@@ -173,8 +157,6 @@ void R_InitSpriteDefs(const char * const * namelist)
   if (!numentries || !*namelist)
     return;
 
-  R_FreeSpriteDefs();
-
   // count the number of sprite names
   for (i=0; namelist[i]; i++)
     ;
@@ -182,7 +164,6 @@ void R_InitSpriteDefs(const char * const * namelist)
   numsprites = i;
 
   sprites = Z_Malloc(numsprites *sizeof(*sprites), PU_STATIC, NULL);
-  memset(sprites, 0, numsprites *sizeof(*sprites));
 
   // Create hash table based on just the first four letters of each sprite
   // killough 1/31/98
@@ -213,7 +194,7 @@ void R_InitSpriteDefs(const char * const * namelist)
           maxframe = -1;
           do
             {
-              register lumpinfo_t *lump = &lumpinfo[j + firstspritelump];
+              register lumpinfo_t *lump = lumpinfo + j + firstspritelump;
 
               // Fast portable comparison -- killough
               // (using int pointer cast is nonportable):
@@ -310,7 +291,7 @@ void R_ClearSprites (void)
 // R_NewVisSprite
 //
 
-vissprite_t *R_NewVisSprite(void)
+static vissprite_t *R_NewVisSprite(void)
 {
   if (num_vissprite >= num_vissprite_alloc)             // killough
     {
@@ -327,33 +308,28 @@ vissprite_t *R_NewVisSprite(void)
 //  in posts/runs of opaque pixels.
 //
 
-short   *mfloorclip;
-short   *mceilingclip;
+int   *mfloorclip;   // dropoff overflow
+int   *mceilingclip; // dropoff overflow
 fixed_t spryscale;
 fixed_t sprtopscreen;
 
-//---------------------------------------------------------------------------
 void R_DrawMaskedColumn(
-  const TPatch *patch,
-  const TPatchColumn *column, 
-  const TPatchColumn *prevColumn, 
-  const TPatchColumn *nextColumn
-) {
+  const rpatch_t *patch,
+  const rcolumn_t *column
+)
+{
+  int     i;
   int     topscreen;
   int     bottomscreen;
   fixed_t basetexturemid = dcvars.texturemid;
-  
-  int i;
-  
-  dcvars.texheight = patch->height;//0; // killough 11/98
+
+  dcvars.texheight = patch->height; // killough 11/98
   for (i=0; i<column->numPosts; i++) {
-    const TPatchPost *post = &column->posts[i]; 
-  
-  //while (column->topdelta != 0xff)
-  //  {
+      const rpost_t *post = &column->posts[i];
+
       // calculate unclipped screen coordinates for post
-      topscreen = sprtopscreen + spryscale*post->startY;
-      bottomscreen = sprtopscreen + spryscale*(post->startY+post->length-1);
+      topscreen = sprtopscreen + spryscale*post->topdelta;
+      bottomscreen = topscreen + spryscale*post->length;
 
       dcvars.yl = (topscreen+FRACUNIT-1)>>FRACBITS;
       dcvars.yh = (bottomscreen-1)>>FRACBITS;
@@ -364,25 +340,16 @@ void R_DrawMaskedColumn(
       if (dcvars.yl <= mceilingclip[dcvars.x])
         dcvars.yl = mceilingclip[dcvars.x]+1;
 
+      // killough 3/2/98, 3/27/98: Failsafe against overflow/crash:
       if (dcvars.yl <= dcvars.yh && dcvars.yh < viewheight)
         {
-          dcvars.source = column->pixels + post->startY;
-          
-          dcvars.nextsource = nextColumn->pixels + post->startY;
-          dcvars.prevsource = prevColumn->pixels + post->startY;
-          dcvars.texturemid = basetexturemid - (post->startY<<FRACBITS);
+          dcvars.source = column->pixels + post->topdelta;
+          dcvars.texturemid = basetexturemid - (post->topdelta<<FRACBITS);
 
-          if (rdrawvars.maskedColumnEdgeType == RDRAW_MASKEDCOLUMNEDGE_SLOPED) { // POPE
-            dcvars.edgeSlope = post->edgeSloping;
-          }
           // Drawn by either R_DrawColumn
           //  or (SHADOW) R_DrawFuzzColumn.
-          dcvars.drawingmasked = 1; // POPE
           colfunc ();
-          dcvars.drawingmasked = 0; // POPE
         }
-
-      //column = (const column_t *)(  (byte *)column + column->length + 4);
     }
   dcvars.texturemid = basetexturemid;
 }
@@ -392,35 +359,34 @@ void R_DrawMaskedColumn(
 //  mfloorclip and mceilingclip should also be set.
 //
 // CPhipps - new wad lump handling, *'s to const*'s
-void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
+static void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
 {
   int      texturecolumn;
   fixed_t  frac;
-  const TPatch *patch = R_GetPatch(vis->patch+firstspritelump);
-  
+  const rpatch_t *patch = R_CachePatchNum(vis->patch+firstspritelump);
+
   dcvars.colormap = vis->colormap;
-  dcvars.nextcolormap = dcvars.colormap; // POPE
 
   // killough 4/11/98: rearrange and handle translucent sprites
   // mixed with translucent/non-translucenct 2s normals
 
   if (!dcvars.colormap)   // NULL colormap = shadow draw
-    colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_FUZZ); // POPE
+    colfunc = R_DrawFuzzColumn;    // killough 3/14/98
   else
     if (vis->mobjflags & MF_TRANSLATION)
       {
-        colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_TRANSLATED); // POPE
+        colfunc = R_DrawTranslatedColumn;
         dcvars.translation = translationtables - 256 +
           ((vis->mobjflags & MF_TRANSLATION) >> (MF_TRANSSHIFT-8) );
       }
     else
       if (vis->mobjflags & MF_TRANSLUCENT && general_translucency) // phares
         {
-          colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_TRANSLUCENT); // POPE
+          colfunc = R_DrawTLColumn;
           tranmap = main_tranmap;       // killough 4/11/98
         }
       else
-        colfunc =R_GetDrawFunc(RDRAW_PIPELINE_COL_STANDARD); // POPE
+        colfunc = R_DrawColumn;         // killough 3/14/98, 4/11/98
 
 // proff 11/06/98: Changed for high-res
   dcvars.iscale = FixedDiv (FRACUNIT, vis->scale);
@@ -432,23 +398,19 @@ void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
   for (dcvars.x=vis->x1 ; dcvars.x<=vis->x2 ; dcvars.x++, frac += vis->xiscale)
     {
       texturecolumn = frac>>FRACBITS;
-      dcvars.texu = frac;
 
 #ifdef RANGECHECK
-      if (texturecolumn < 0 || texturecolumn >= SHORT(patch->width))
+      if (texturecolumn < 0 || texturecolumn >= patch->width)
         I_Error ("R_DrawSpriteRange: Bad texturecolumn");
 #endif
 
       R_DrawMaskedColumn(
-        patch, 
-        R_GetPatchColumnClamped(patch, texturecolumn),
-        R_GetPatchColumnClamped(patch, texturecolumn-1),
-        R_GetPatchColumnClamped(patch, texturecolumn+1)
+        patch,
+        R_GetPatchColumnClamped(patch, texturecolumn)
       );
     }
-  
-  colfunc = R_GetDrawFunc(RDRAW_PIPELINE_COL_STANDARD); // POPE
-  W_UnlockLumpNum(vis->patch+firstspritelump); // cph - release lump
+  colfunc = R_DrawColumn;         // killough 3/14/98
+  R_UnlockPatchNum(vis->patch+firstspritelump); // cph - release lump
 }
 
 //
@@ -456,7 +418,7 @@ void R_DrawVisSprite(vissprite_t *vis, int x1, int x2)
 // Generates a vissprite for a thing if it might be visible.
 //
 
-void R_ProjectSprite (mobj_t* thing, int lightlevel)
+static void R_ProjectSprite (mobj_t* thing, int lightlevel)
 {
   fixed_t   gzt;               // killough 3/27/98
   fixed_t   tx;
@@ -468,6 +430,7 @@ void R_ProjectSprite (mobj_t* thing, int lightlevel)
   int       lump;
   boolean   flip;
   vissprite_t *vis;
+  fixed_t   iscale;
   int heightsec;      // killough 3/27/98
 
   // transform the origin point
@@ -531,17 +494,24 @@ void R_ProjectSprite (mobj_t* thing, int lightlevel)
     }
 
   {
-    const TPatch *patch = R_GetPatch(lump+firstspritelump);
+    const rpatch_t* patch = R_CachePatchNum(lump+firstspritelump);
 
-    // calculate edges of the shape
-    tx -= SHORT(patch->leftOffset)<<FRACBITS;
-    x1 = (centerxfrac + FixedMul(tx,xscale)) >>FRACBITS;
+    /* calculate edges of the shape
+     * cph 2003/08/1 - fraggle points out that this offset must be flipped
+     * if the sprite is flipped; e.g. FreeDoom imp is messed up by this. */
+    if (flip) {
+      tx -= (patch->width - patch->leftoffset) << FRACBITS;
+    } else {
+      tx -= patch->leftoffset << FRACBITS;
+    }
+    x1 = (centerxfrac + FixedMul(tx,xscale)) >> FRACBITS;
 
-    tx += SHORT(patch->width)<<FRACBITS;
-    x2 = ((centerxfrac + FixedMul (tx,xscale) ) >>FRACBITS) - 1;
+    tx += patch->width<<FRACBITS;
+    x2 = ((centerxfrac + FixedMul (tx,xscale) ) >> FRACBITS) - 1;
 
-    gzt = thing->z + (SHORT(patch->topOffset)<<FRACBITS);
-    width = SHORT(patch->width);
+    gzt = thing->z + (patch->topoffset << FRACBITS);
+    width = patch->width;
+    R_UnlockPatchNum(lump+firstspritelump);
   }
 
   // off the side?
@@ -561,11 +531,12 @@ void R_ProjectSprite (mobj_t* thing, int lightlevel)
 
   if (heightsec != -1)   // only clip things which are in special sectors
     {
-      if (viewheightsec != -1 && viewz < sectors[viewheightsec].floorheight ?
+      int phs = viewplayer->mo->subsector->sector->heightsec;
+      if (phs != -1 && viewz < sectors[phs].floorheight ?
           thing->z >= sectors[heightsec].floorheight :
           gzt < sectors[heightsec].floorheight)
         return;
-      if (viewheightsec != -1 && viewz > sectors[viewheightsec].ceilingheight ?
+      if (phs != -1 && viewz > sectors[phs].ceilingheight ?
           gzt < sectors[heightsec].ceilingheight &&
           viewz >= sectors[heightsec].ceilingheight :
           thing->z >= sectors[heightsec].ceilingheight)
@@ -587,53 +558,48 @@ void R_ProjectSprite (mobj_t* thing, int lightlevel)
 
     return;
   }
-  else
 #endif
-  {
-    fixed_t   iscale;
+  // killough 3/27/98: save sector for special clipping later
+  vis->heightsec = heightsec;
 
-    // killough 3/27/98: save sector for special clipping later
-    vis->heightsec = heightsec;
+  vis->mobjflags = thing->flags;
+// proff 11/06/98: Changed for high-res
+  vis->scale = FixedDiv(projectiony, tz);
+  vis->gx = thing->x;
+  vis->gy = thing->y;
+  vis->gz = thing->z;
+  vis->gzt = gzt;                          // killough 3/27/98
+  vis->texturemid = vis->gzt - viewz;
+  vis->x1 = x1 < 0 ? 0 : x1;
+  vis->x2 = x2 >= viewwidth ? viewwidth-1 : x2;
+  iscale = FixedDiv (FRACUNIT, xscale);
 
-    vis->mobjflags = thing->flags;
-    // proff 11/06/98: Changed for high-res
-    vis->scale = FixedDiv(projectiony, tz);
-    vis->gx = thing->x;
-    vis->gy = thing->y;
-    vis->gz = thing->z;
-    vis->gzt = gzt;                          // killough 3/27/98
-    vis->texturemid = vis->gzt - viewz;
-    vis->x1 = x1 < 0 ? 0 : x1;
-    vis->x2 = x2 >= viewwidth ? viewwidth-1 : x2;
-    iscale = FixedDiv (FRACUNIT, xscale);
+  if (flip)
+    {
+      vis->startfrac = (width<<FRACBITS)-1;
+      vis->xiscale = -iscale;
+    }
+  else
+    {
+      vis->startfrac = 0;
+      vis->xiscale = iscale;
+    }
 
-    if (flip)
-      {
-        vis->startfrac = (width<<FRACBITS)-1;
-        vis->xiscale = -iscale;
-      }
-    else
-      {
-        vis->startfrac = 0;
-        vis->xiscale = iscale;
-      }
+  if (vis->x1 > x1)
+    vis->startfrac += vis->xiscale*(vis->x1-x1);
+  vis->patch = lump;
 
-    if (vis->x1 > x1)
-      vis->startfrac += vis->xiscale*(vis->x1-x1);
-    vis->patch = lump;
-
-    // get light level
-    if (thing->flags & MF_SHADOW)
-        vis->colormap = NULL;             // shadow draw
-    else if (fixedcolormap)
-      vis->colormap = fixedcolormap;      // fixed map
-    else if (thing->frame & FF_FULLBRIGHT)
-      vis->colormap = fullcolormap;     // full bright  // killough 3/20/98
-    else
-      {      // diminished light
-        vis->colormap = R_ColourMap(lightlevel,xscale);
-      }
-  }
+  // get light level
+  if (thing->flags & MF_SHADOW)
+      vis->colormap = NULL;             // shadow draw
+  else if (fixedcolormap)
+    vis->colormap = fixedcolormap;      // fixed map
+  else if (thing->frame & FF_FULLBRIGHT)
+    vis->colormap = fullcolormap;     // full bright  // killough 3/20/98
+  else
+    {      // diminished light
+      vis->colormap = R_ColourMap(lightlevel,xscale);
+    }
 }
 
 //
@@ -667,7 +633,7 @@ void R_AddSprites(subsector_t* subsec, int lightlevel)
 // R_DrawPSprite
 //
 
-void R_DrawPSprite (pspdef_t *psp, int lightlevel)
+static void R_DrawPSprite (pspdef_t *psp, int lightlevel)
 {
   int           x1, x2;
   spritedef_t   *sprdef;
@@ -676,7 +642,7 @@ void R_DrawPSprite (pspdef_t *psp, int lightlevel)
   boolean       flip;
   vissprite_t   *vis;
   vissprite_t   avis;
-  int           spritewidth;
+  int           width;
   fixed_t       topoffset;
 
   // decide which patch to use
@@ -700,21 +666,20 @@ void R_DrawPSprite (pspdef_t *psp, int lightlevel)
   flip = (boolean) sprframe->flip[0];
 
   {
-    const TPatch *patch = R_GetPatch(lump+firstspritelump);
-    
+    const rpatch_t* patch = R_CachePatchNum(lump+firstspritelump);
     // calculate edges of the shape
     fixed_t       tx;
-    int           spriteoffset = SHORT(patch->leftOffset)<<FRACBITS;
     tx = psp->sx-160*FRACUNIT;
-    spritewidth = SHORT(patch->width)<<FRACBITS;
 
-    tx -= flip ? spritewidth - spriteoffset : spriteoffset;
+    tx -= patch->leftoffset<<FRACBITS;
     x1 = (centerxfrac + FixedMul (tx,pspritescale))>>FRACBITS;
 
-    tx += spritewidth;
+    tx += patch->width<<FRACBITS;
     x2 = ((centerxfrac + FixedMul (tx, pspritescale) ) >>FRACBITS) - 1;
 
-    topoffset = SHORT(patch->topOffset)<<FRACBITS;
+    width = patch->width;
+    topoffset = patch->topoffset<<FRACBITS;
+    R_UnlockPatchNum(lump+firstspritelump);
   }
 
   // off the side
@@ -735,7 +700,7 @@ void R_DrawPSprite (pspdef_t *psp, int lightlevel)
   if (flip)
     {
       vis->xiscale = -pspriteiscale;
-      vis->startfrac = spritewidth-1;
+      vis->startfrac = (width<<FRACBITS)-1;
     }
   else
     {
@@ -774,7 +739,7 @@ void R_DrawPSprite (pspdef_t *psp, int lightlevel)
       lightlevel=255;
     else
     {
-      //lightlevel = (viewplayer->mo->subsector->sector->lightlevel) + (extralight << LIGHTSEGSHIFT);
+//      lightlevel = (viewplayer->mo->subsector->sector->lightlevel) + (extralight << LIGHTSEGSHIFT);
       R_FakeFlat( viewplayer->mo->subsector->sector, &tmpsec,
                   &floorlightlevel, &ceilinglightlevel, false);
       lightlevel = ((floorlightlevel+ceilinglightlevel) >> 1) + (extralight << LIGHTSEGSHIFT);
@@ -786,7 +751,7 @@ void R_DrawPSprite (pspdef_t *psp, int lightlevel)
     }
     gld_DrawWeapon(lump,vis,lightlevel);
   }
-#endif	
+#endif
 }
 
 //
@@ -798,8 +763,6 @@ void R_DrawPlayerSprites(void)
   int i, lightlevel = viewplayer->mo->subsector->sector->lightlevel;
   pspdef_t *psp;
 
-  if (viewcamera) return;
-  
   // clip to screen bounds
   mfloorclip = screenheightarray;
   mceilingclip = negonearray;
@@ -898,11 +861,11 @@ void R_SortVisSprites (void)
 // R_DrawSprite
 //
 
-void R_DrawSprite (vissprite_t* spr)
+static void R_DrawSprite (vissprite_t* spr)
 {
   drawseg_t *ds;
-  short   clipbot[MAX_SCREENWIDTH];       // killough 2/8/98:
-  short   cliptop[MAX_SCREENWIDTH];       // change to MAX_*
+  int     clipbot[MAX_SCREENWIDTH]; // killough 2/8/98: // dropoff overflow
+  int     cliptop[MAX_SCREENWIDTH]; // change to MAX_*  // dropoff overflow
   int     x;
   int     r1;
   int     r2;
@@ -967,7 +930,7 @@ void R_DrawSprite (vissprite_t* spr)
   // Clip the sprite against deep water and/or fake ceilings.
   // killough 4/9/98: optimize by adding mh
   // killough 4/11/98: improve sprite clipping for underwater/fake ceilings
-  // killough 11/98: fix disappearing sprites 
+  // killough 11/98: fix disappearing sprites
 
   if (spr->heightsec != -1)  // only things in specially marked sectors
     {
@@ -983,10 +946,10 @@ void R_DrawSprite (vissprite_t* spr)
                 clipbot[x] = h;
           }
         else                        // clip top
-	  if (phs != -1 && viewz <= sectors[phs].floorheight) // killough 11/98
-	    for (x=spr->x1 ; x<=spr->x2 ; x++)
-	      if (cliptop[x] == -2 || h > cliptop[x])
-		cliptop[x] = h;
+    if (phs != -1 && viewz <= sectors[phs].floorheight) // killough 11/98
+      for (x=spr->x1 ; x<=spr->x2 ; x++)
+        if (cliptop[x] == -2 || h > cliptop[x])
+    cliptop[x] = h;
       }
 
       if ((mh = sectors[spr->heightsec].ceilingheight) < spr->gzt &&
@@ -1012,7 +975,7 @@ void R_DrawSprite (vissprite_t* spr)
   for (x = spr->x1 ; x<=spr->x2 ; x++) {
     if (clipbot[x] == -2)
       clipbot[x] = viewheight;
-    
+
     if (cliptop[x] == -2)
       cliptop[x] = -1;
   }
