@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -45,7 +47,6 @@
 #include "sounds.h"
 #include "dstrings.h"
 #include "r_draw.h"
-#include "c_runcmd.h"
 
 //
 // STATUS BAR DATA
@@ -296,7 +297,7 @@ static patchnum_t shortnum[10];
 
 // 3 key-cards, 3 skulls, 3 card/skull combos
 // jff 2/24/98 extend number of patches by three skull/card combos
-patchnum_t keys[NUMCARDS+3];
+static patchnum_t keys[NUMCARDS+3];
 
 // face status patches
 static patchnum_t faces[ST_NUMFACES];
@@ -314,14 +315,14 @@ static patchnum_t arms[6][2];
 static st_number_t w_ready;
 
 //jff 2/16/98 status color change levels
-int ammo_red=25;      // ammo percent less than which status is red
-int ammo_yellow=50;   // ammo percent less is yellow more green
-int health_red=25;    // health amount less than which status is red
-int health_yellow=50; // health amount less than which status is yellow
-int health_green=100; // health amount above is blue, below is green
-int armor_red=25;     // armor amount less than which status is red
-int armor_yellow=50;  // armor amount less than which status is yellow
-int armor_green=100;  // armor amount above is blue, below is green
+int ammo_red;      // ammo percent less than which status is red
+int ammo_yellow;   // ammo percent less is yellow more green
+int health_red;    // health amount less than which status is red
+int health_yellow; // health amount less than which status is yellow
+int health_green;  // health amount above is blue, below is green
+int armor_red;     // armor amount less than which status is red
+int armor_yellow;  // armor amount less than which status is yellow
+int armor_green;   // armor amount above is blue, below is green
 
  // in deathmatch only, summary of frags stats
 static st_number_t w_frags;
@@ -377,9 +378,9 @@ extern char     *mapnames[];
 // STATUS BAR CODE
 //
 
-void ST_Stop(void);
+static void ST_Stop(void);
 
-void ST_refreshBackground(void)
+static void ST_refreshBackground(void)
 {
   int y=0;
   int screen=BG;
@@ -395,8 +396,8 @@ void ST_refreshBackground(void)
       if (netgame)
       {
         V_DrawNumPatch(ST_FX, y, BG, faceback.lumpnum,
-		       displayplayer ? CR_LIMIT+displayplayer : CR_DEFAULT, 
-		       displayplayer ? (VPT_TRANS | VPT_STRETCH) : VPT_STRETCH);
+           displayplayer ? CR_LIMIT+displayplayer : CR_DEFAULT,
+           displayplayer ? (VPT_TRANS | VPT_STRETCH) : VPT_STRETCH);
       }
 
       V_CopyRect(ST_X, y, screen, ST_SCALED_WIDTH, ST_SCALED_HEIGHT, ST_X, ST_SCALED_Y, FG, VPT_NONE);
@@ -429,7 +430,7 @@ boolean ST_Responder(event_t *ev)
   return false;
 }
 
-int ST_calcPainOffset(void)
+static int ST_calcPainOffset(void)
 {
   static int lastcalc;
   static int oldhealth = -1;
@@ -450,7 +451,7 @@ int ST_calcPainOffset(void)
 //  dead > evil grin > turned head > straight ahead
 //
 
-void ST_updateFaceWidget(void)
+static void ST_updateFaceWidget(void)
 {
   int         i;
   angle_t     badguyangle;
@@ -620,9 +621,9 @@ void ST_updateFaceWidget(void)
 
 }
 
-int sts_traditional_keys = 0; // killough 2/28/98: traditional status bar keys
+int sts_traditional_keys; // killough 2/28/98: traditional status bar keys
 
-void ST_updateWidgets(void)
+static void ST_updateWidgets(void)
 {
   static int  largeammo = 1994; // means "n/a"
   int         i;
@@ -697,9 +698,9 @@ void ST_Ticker(void)
   st_oldhealth = plyr->health;
 }
 
-static int st_palette = 0;
+int st_palette = 0;
 
-void ST_doPaletteStuff(void)
+static void ST_doPaletteStuff(void)
 {
   int         palette;
   int cnt = plyr->damagecount;
@@ -717,6 +718,11 @@ void ST_doPaletteStuff(void)
       palette = (cnt+7)>>3;
       if (palette >= NUMREDPALS)
         palette = NUMREDPALS-1;
+
+      /* cph 2006/08/06 - if in the menu, reduce the red tint - navigating to
+       * load a game can be tricky if the screen is all red */
+      if (menuactive) palette >>=1;
+
       palette += STARTREDPALS;
     }
   else
@@ -733,16 +739,11 @@ void ST_doPaletteStuff(void)
       else
         palette = 0;
 
-  if (palette != st_palette) {
+  if (palette != st_palette)
     V_SetPalette(st_palette = palette); // CPhipps - use new palette function
-    
-    // have to redraw the entire status bar when the palette changes
-    // in truecolor modes - POPE
-    if (V_GetMode() == VID_MODE16 || V_GetMode() == VID_MODE32) st_firsttime = true;
-  }
 }
 
-void ST_drawWidgets(boolean refresh)
+static void ST_drawWidgets(boolean refresh)
 {
   int i;
 
@@ -802,7 +803,7 @@ void ST_drawWidgets(boolean refresh)
 
 }
 
-void ST_doRefresh(void)
+static void ST_doRefresh(void)
 {
 
   st_firsttime = false;
@@ -815,7 +816,7 @@ void ST_doRefresh(void)
 
 }
 
-void ST_diffDraw(void)
+static void ST_diffDraw(void)
 {
   // update all widgets
   ST_drawWidgets(false);
@@ -823,7 +824,7 @@ void ST_diffDraw(void)
 
 void ST_Drawer(boolean st_statusbaron, boolean refresh)
 {
-  /* cph - let status bar on be controlled 
+  /* cph - let status bar on be controlled
    * completely by the call from D_Display
    * proff - really do it
    */
@@ -851,7 +852,7 @@ static void ST_loadGraphics(boolean doload)
 {
   int  i, facenum;
   char namebuf[9];
-  // cph - macro that either acquires a pointer and lock for a lump, or 
+  // cph - macro that either acquires a pointer and lock for a lump, or
   // unlocks it. var is referenced exactly once in either case, so ++ in arg works
 
   // Load the numbers, tall and short
@@ -863,7 +864,7 @@ static void ST_loadGraphics(boolean doload)
       R_SetPatchNum(&shortnum[i],namebuf);
     }
 
-  // Load percent key. 
+  // Load percent key.
   R_SetPatchNum(&tallpercent,"STTPRCNT");
 
   // key cards
@@ -918,17 +919,17 @@ static void ST_loadGraphics(boolean doload)
   R_SetPatchNum(&faces[facenum++], "STFDEAD0");
 }
 
-void ST_loadData(void)
+static void ST_loadData(void)
 {
   ST_loadGraphics(true);
 }
 
-void ST_unloadData(void)
+static void ST_unloadData(void)
 {
   ST_loadGraphics(false);
 }
 
-void ST_initData(void)
+static void ST_initData(void)
 {
   int i;
 
@@ -957,7 +958,7 @@ void ST_initData(void)
   STlib_init();
 }
 
-void ST_createWidgets(void)
+static void ST_createWidgets(void)
 {
   int i;
 
@@ -1125,7 +1126,7 @@ void ST_Start(void)
   st_stopped = false;
 }
 
-void ST_Stop(void)
+static void ST_Stop(void)
 {
   if (st_stopped)
     return;
@@ -1138,34 +1139,3 @@ void ST_Init(void)
   veryfirsttime = 0;
   ST_loadData();
 }
-
-CONSOLE_INT(ammo_red, ammo_red, NULL, 0, 100, NULL, 0) { }
-CONSOLE_INT(ammo_yellow, ammo_yellow, NULL, 0, 100, NULL, 0) { }
-CONSOLE_INT(health_red, health_red, NULL, 0, 200, NULL, 0) { }
-CONSOLE_INT(health_yellow, health_yellow, NULL, 0, 200, NULL, 0) { }
-CONSOLE_INT(health_green, health_green, NULL, 0, 200, NULL, 0) { }
-CONSOLE_INT(armor_red, armor_red, NULL, 0, 200, NULL, 0) { }
-CONSOLE_INT(armor_yellow, armor_yellow, NULL, 0, 200, NULL, 0) { }
-CONSOLE_INT(armor_green, armor_green, NULL, 0, 200, NULL, 0) { }
-
-CONSOLE_BOOLEAN(st_graypct, sts_pct_always_gray, NULL, yesno, 0) { }
-CONSOLE_BOOLEAN(st_rednum, sts_always_red, NULL, yesno, 0) {}
-CONSOLE_BOOLEAN(st_singlekey, sts_traditional_keys, NULL, yesno, 0) { }
-
-void ST_AddCommands()
-{
-  C_AddCommand(ammo_red);
-  C_AddCommand(ammo_yellow);
-  
-  C_AddCommand(health_red);
-  C_AddCommand(health_yellow);
-  C_AddCommand(health_green);
-  
-  C_AddCommand(armor_red);
-  C_AddCommand(armor_yellow);
-  C_AddCommand(armor_green);
-  
-  C_AddCommand(st_graypct);
-  C_AddCommand(st_rednum);
-  C_AddCommand(st_singlekey);
-}
