@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -42,12 +44,8 @@
 #include "lprintf.h"  // jff 08/03/98 - declaration of lprintf
 #include "r_draw.h"
 
-#ifdef COMPILE_VIDD
-#include "vidd/vidd.h"
-#endif
-
 // Ty 03/17/98: flag that new par times have been loaded in d_deh
-extern boolean deh_pars;  
+extern boolean deh_pars;
 
 //
 // Data needed to add patches to full screen intermission pics.
@@ -67,11 +65,11 @@ extern boolean deh_pars;
 
 // Not used
 // in tics
-//U #define PAUSELEN    (TICRATE*2) 
+//U #define PAUSELEN    (TICRATE*2)
 //U #define SCORESTEP    100
 //U #define ANIMPERIOD    32
 // pixel distance from "(YOU)" to "PLAYER N"
-//U #define STARDIST  10 
+//U #define STARDIST  10
 //U #define WK 1
 
 
@@ -91,7 +89,7 @@ extern boolean deh_pars;
 
 // NET GAME STUFF
 #define NG_STATSY     50
-#define NG_STATSX     (32 + R_NamePatchWidth(star)/2 + 32*!dofrags)
+#define NG_STATSX     (32 + V_NamePatchWidth(star)/2 + 32*!dofrags)
 
 #define NG_SPACINGX   64
 
@@ -153,12 +151,12 @@ typedef struct
   // ALWAYS: n/a,
   // RANDOM: random base period,
   // LEVEL: n/a
-  int   data2; 
+  int   data2;
 
   /* actual graphics for frames of animations
    * cphipps - const
    */
-  patchnum_t p[3]; 
+  patchnum_t p[3];
 
   // following must be initialized to zero before use!
 
@@ -170,9 +168,9 @@ typedef struct
 
   // next frame number to animate
   int   ctr;
-  
+
   // used by RANDOM and LEVEL when animating
-  int   state;  
+  int   state;
 } anim_t;
 
 
@@ -190,7 +188,7 @@ static point_t lnodes[NUMEPISODES][NUMMAPS] =
     { 135, 29 },  // location of level 7 (CJ)
     { 71, 24 }    // location of level 8 (CJ)
   },
-  
+
   // Episode 1 World Map should go here
   {
     { 254, 25 },  // location of level 0 (CJ)
@@ -203,7 +201,7 @@ static point_t lnodes[NUMEPISODES][NUMMAPS] =
     { 148, 140 }, // location of level 7 (CJ)
     { 235, 158 }  // location of level 8 (CJ)
   },
-  
+
   // Episode 2 World Map should go here
   {
     { 156, 168 }, // location of level 0 (CJ)
@@ -290,8 +288,8 @@ static anim_t *anims[NUMEPISODES] =
 #define SP_KILLS    0
 #define SP_ITEMS    2
 #define SP_SECRET   4
-#define SP_FRAGS    6 
-#define SP_TIME     8 
+#define SP_FRAGS    6
+#define SP_TIME     8
 #define SP_PAR      ST_TIME
 
 #define SP_PAUSE    1
@@ -316,13 +314,13 @@ static wbstartstruct_t* wbs;
 static wbplayerstruct_t* plrs;  // wbs->plyr[]
 
 // used for general timing
-static int    cnt;  
+static int    cnt;
 
 // used for timing of background animation
 static int    bcnt;
 
 // signals to refresh everything for one frame
-static int    firstrefresh; 
+static int    firstrefresh;
 
 static int    cnt_time;
 static int    cnt_total_time;
@@ -335,7 +333,7 @@ static int    cnt_pause;
 
 // You Are Here graphic
 static const char* yah[2] = { "WIURH0", "WIURH1" };
-  
+
 // splat
 static const char* splat = "WISPLAT";
 
@@ -371,7 +369,7 @@ static const char sucks[] = {"WISUCKS"};
 
 // "killers", "victims"
 static const char killers[] = {"WIKILRS"};
-static const char victims[] = {"WIVCTMS"}; 
+static const char victims[] = {"WIVCTMS"};
 
 // "Total", your face, your dead face
 static const char total[] = {"WIMSTT"};
@@ -385,89 +383,13 @@ static const char facebackp[] = {"STPB0"};
 // CODE
 //
 
-/**********************
-
-  INTERMISSION CAMERA
-
-***********************/
-
-// is this just some boring picture, or a view of the level?
-static int realbackdrop=1;
-camera_t        intercam;
-
-mapthing_t *camerathing[MAXCAMERAS];
-mapthing_t *wi_camera;
-int numcameraviews=0;
-
-void WI_StopCamera()
-{
-  numcameraviews = 0;
-  realbackdrop = 0;
-}
-
-void WI_TickerCamera()
-{
-  if(!realbackdrop) return;
-}
-
-void WI_AddCamera(mapthing_t *mthing)
-{
-  camerathing[numcameraviews] = mthing;
-  numcameraviews++;
-}
-
-// set up the intermissions camera
-
-void WI_StartCamera()
-{
-  int i;
-  
-  if(numcameraviews)
-    {
-      realbackdrop = 1;
-      // pick a camera at random
-      wi_camera = camerathing[M_Random() % numcameraviews];
-      
-      // centre the view
-      // players[displayplayer].updownangle = 0;
-      
-      // remove the player mobjs (look silly in camera view)
-      for(i=0; i<MAXPLAYERS; i++)
-	{
-	  if(!playeringame[i]) continue;
-	  // this is strange. the monsters can still
-	  // see the player Mobj, (and even kill it!)
-	  // even tho it has been removed from the
-	  // level. I make it unshootable first so
-	  // they lose interest.
-	  players[i].mo->flags &= ~MF_SHOOTABLE;
-	  P_RemoveMobj(players[i].mo);
-	}
-      
-      
-      intercam.x = wi_camera->x*FRACUNIT;
-      intercam.y = wi_camera->y*FRACUNIT;
-      intercam.angle = R_WadToAngle(wi_camera->angle);
-      intercam.updownangle = 0;
-      intercam.z = R_PointInSubsector(intercam.x, intercam.y)
-	->sector->floorheight + 41*FRACUNIT;
-      R_SetViewSize (11);     // force fullscreen
-    }
-  else            // no camera, boring interpic
-    {
-      realbackdrop = 0;
-      wi_camera = NULL;
-    }
-}
-
 static void WI_endDeathmatchStats(void);
 static void WI_endNetgameStats(void);
-void WI_unloadData(void);
 #define WI_endStats WI_endNetgameStats
 
 /* ====================================================================
  * WI_levelNameLump
- * Purpore: Returns the name of the graphic lump containing the name of 
+ * Purpore: Returns the name of the graphic lump containing the name of
  *          the given level.
  * Args:    Episode and level, and buffer (must by 9 chars) to write to
  * Returns: void
@@ -475,7 +397,7 @@ void WI_unloadData(void);
 void WI_levelNameLump(int epis, int map, char* buf)
 {
   if (gamemode == commercial) {
-    sprintf(buf, "CWILV%2.2d", map);    
+    sprintf(buf, "CWILV%2.2d", map);
   } else {
     sprintf(buf, "WILV%d%d", epis, map);
   }
@@ -493,7 +415,7 @@ static void WI_slamBackground(void)
 
   if (gamemode == commercial || (gamemode == retail && wbs->epsd == 3))
     strcpy(name, "INTERPIC");
-  else 
+  else
     sprintf(name, "WIMAP%d", wbs->epsd);
 
   // background
@@ -526,19 +448,19 @@ void WI_drawLF(void)
   int y = WI_TITLEY;
   char lname[9];
 
-  // draw <LevelName> 
+  // draw <LevelName>
   /* cph - get the graphic lump name and use it */
   WI_levelNameLump(wbs->epsd, wbs->last, lname);
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - R_NamePatchWidth(lname))/2, y, 
-		 FB, lname, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - V_NamePatchWidth(lname))/2, y,
+     FB, lname, CR_DEFAULT, VPT_STRETCH);
 
   // draw "Finished!"
-  y += (5*R_NamePatchHeight(lname))/4;
-  
+  y += (5*V_NamePatchHeight(lname))/4;
+
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - R_NamePatchWidth(finished))/2, y, 
-		 FB, finished, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - V_NamePatchWidth(finished))/2, y,
+     FB, finished, CR_DEFAULT, VPT_STRETCH);
 }
 
 
@@ -558,15 +480,15 @@ void WI_drawEL(void)
 
   // draw "Entering"
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - R_NamePatchWidth(entering))/2, 
-		  y, FB, entering, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - V_NamePatchWidth(entering))/2,
+      y, FB, entering, CR_DEFAULT, VPT_STRETCH);
 
   // draw level
-  y += (5*R_NamePatchHeight(lname))/4;
+  y += (5*V_NamePatchHeight(lname))/4;
 
   // CPhipps - patch drawing updated
-  V_DrawNamePatch((320 - R_NamePatchWidth(lname))/2, y, FB, 
-		 lname, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch((320 - V_NamePatchWidth(lname))/2, y, FB,
+     lname, CR_DEFAULT, VPT_STRETCH);
 }
 
 
@@ -592,18 +514,18 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
     int            top;
     int            right;
     int            bottom;
-    int            lump = W_GetNumForName(c[i]);
-    const TPatch *patch = R_GetPatch(lump); // POPE
+    const rpatch_t* patch = R_CachePatchName(c[i]);
+
+    left = lnodes[wbs->epsd][n].x - patch->leftoffset;
+    top = lnodes[wbs->epsd][n].y - patch->topoffset;
+    right = left + patch->width;
+    bottom = top + patch->height;
+    R_UnlockPatchName(c[i]);
 
-    left = lnodes[wbs->epsd][n].x - SHORT(patch->leftOffset);
-    top = lnodes[wbs->epsd][n].y - SHORT(patch->topOffset);
-    right = left + SHORT(patch->width);
-    bottom = top + SHORT(patch->height);
-  
     if (left >= 0
-       && right < SCREENWIDTH
+       && right < 320
        && top >= 0
-       && bottom < SCREENHEIGHT)
+       && bottom < 200)
     {
       fits = true;
     }
@@ -617,7 +539,7 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
   {
     // CPhipps - patch drawing updated
     V_DrawNamePatch(lnodes[wbs->epsd][n].x, lnodes[wbs->epsd][n].y,
-		   FB, c[i], CR_DEFAULT, VPT_STRETCH);
+       FB, c[i], CR_DEFAULT, VPT_STRETCH);
   }
   else
   {
@@ -634,7 +556,7 @@ WI_drawOnLnode  // draw stuff at a location by episode/map#
 // Args:    none
 // Returns: void
 //
-static void WI_initAnimatedBack(void)
+void WI_initAnimatedBack(void)
 {
   int   i;
   anim_t* a;
@@ -651,14 +573,14 @@ static void WI_initAnimatedBack(void)
 
     // init variables
     a->ctr = -1;
-  
+
     // specify the next time to draw it
     if (a->type == ANIM_ALWAYS)
       a->nexttic = bcnt + 1 + (M_Random()%a->period);
-    else 
+    else
       if (a->type == ANIM_RANDOM)
         a->nexttic = bcnt + 1 + a->data2+(M_Random()%a->data1);
-      else 
+      else
         if (a->type == ANIM_LEVEL)
           a->nexttic = bcnt + 1;
   }
@@ -671,7 +593,7 @@ static void WI_initAnimatedBack(void)
 // Args:    none
 // Returns: void
 //
-static void WI_updateAnimatedBack(void)
+void WI_updateAnimatedBack(void)
 {
   int   i;
   anim_t* a;
@@ -702,10 +624,10 @@ static void WI_updateAnimatedBack(void)
                a->ctr = -1;
                a->nexttic = bcnt+a->data2+(M_Random()%a->data1);
              }
-             else 
+             else
                a->nexttic = bcnt + a->period;
              break;
-    
+
         case ANIM_LEVEL:
              // gawd-awful hack for level anims
              if (!(state == StatCount && i == 7)
@@ -728,7 +650,7 @@ static void WI_updateAnimatedBack(void)
 // Args:    none
 // Returns: void
 //
-static void WI_drawAnimatedBack(void)
+void WI_drawAnimatedBack(void)
 {
   int     i;
   anim_t*   a;
@@ -761,7 +683,7 @@ static void WI_drawAnimatedBack(void)
 // CPhipps - static
 static int WI_drawNum (int x, int y, int n, int digits)
 {
-  int   fontwidth = SHORT(num[0].width);
+  int   fontwidth = num[0].width;
   int   neg;
   int   temp;
 
@@ -799,7 +721,7 @@ static int WI_drawNum (int x, int y, int n, int digits)
   {
     x -= fontwidth;
     // CPhipps - patch drawing updated
-    V_DrawNumPatch(x, y, FB, num[n % 10].lumpnum, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNumPatch(x, y, FB, num[ n % 10 ].lumpnum, CR_DEFAULT, VPT_STRETCH);
     n /= 10;
   }
 
@@ -814,7 +736,7 @@ static int WI_drawNum (int x, int y, int n, int digits)
 
 // ====================================================================
 // WI_drawPercent
-// Purpose: Draws a percentage, really just a call to WI_drawNum 
+// Purpose: Draws a percentage, really just a call to WI_drawNum
 //          after putting a percent sign out there
 // Args:    x, y   -- location
 //          p      -- the percentage value to be drawn, no negatives
@@ -843,7 +765,7 @@ static void WI_drawPercent(int x, int y, int p)
 //         - largely rewritten to display hours and use slightly better algorithm
 
 static void WI_drawTime(int x, int y, int t)
-{ 
+{
   int   n;
 
   if (t<0)
@@ -853,17 +775,17 @@ static void WI_drawTime(int x, int y, int t)
     for(;;) {
       n = t % 60;
       t /= 60;
-      x = WI_drawNum(x, y, n, (t || n>9) ? 2 : 1) - R_NamePatchWidth(colon);
+      x = WI_drawNum(x, y, n, (t || n>9) ? 2 : 1) - V_NamePatchWidth(colon);
 
       // draw
       if (t)
-	// CPhipps - patch drawing updated
+  // CPhipps - patch drawing updated
         V_DrawNamePatch(x, y, FB, colon, CR_DEFAULT, VPT_STRETCH);
       else break;
-    } 
+    }
   else // "sucks" (maybe should be "addicted", even I've never had a 100 hour game ;)
-    V_DrawNamePatch(x - R_NamePatchWidth(sucks), 
-		    y, FB, sucks, CR_DEFAULT, VPT_STRETCH); 
+    V_DrawNamePatch(x - V_NamePatchWidth(sucks),
+        y, FB, sucks, CR_DEFAULT, VPT_STRETCH);
 }
 
 
@@ -875,8 +797,6 @@ static void WI_drawTime(int x, int y, int t)
 //
 void WI_End(void)
 {
-  WI_unloadData();
-
   if (deathmatch)
     WI_endDeathmatchStats();
   else if (netgame)
@@ -892,7 +812,7 @@ void WI_End(void)
 // Args:    none
 // Returns: void
 //
-static void WI_initNoState(void)
+void WI_initNoState(void)
 {
   state = NoState;
   acceleratestage = 0;
@@ -916,12 +836,12 @@ static void WI_drawTimeStats(int cnt_time, int cnt_total_time, int cnt_par)
   V_DrawNamePatch(SP_TIMEX, (SP_TIMEY+200)/2, FB, total, CR_DEFAULT, VPT_STRETCH);
   WI_drawTime(320/2 - SP_TIMEX, (SP_TIMEY+200)/2, cnt_total_time);
 
-  // Ty 04/11/98: redid logic: should skip only if with pwad but 
+  // Ty 04/11/98: redid logic: should skip only if with pwad but
   // without deh patch
   // killough 2/22/98: skip drawing par times on pwads
   // Ty 03/17/98: unless pars changed with deh patch
 
-  if (!(modifiedgame && !deh_pars))   
+  if (!(modifiedgame && !deh_pars))
   {
     if (wbs->epsd < 3)
     {
@@ -937,7 +857,7 @@ static void WI_drawTimeStats(int cnt_time, int cnt_total_time, int cnt_par)
 // Args:    none
 // Returns: void
 //
-void WI_updateNoState(void) 
+void WI_updateNoState(void)
 {
 
   WI_updateAnimatedBack();
@@ -951,11 +871,11 @@ static boolean    snl_pointeron = false;
 
 // ====================================================================
 // WI_initShowNextLoc
-// Purpose: Prepare to show the next level's location 
+// Purpose: Prepare to show the next level's location
 // Args:    none
 // Returns: void
 //
-static void WI_initShowNextLoc(void)
+void WI_initShowNextLoc(void)
 {
   if ((gamemode != commercial) && (gamemap == 8)) {
     G_WorldDone();
@@ -976,7 +896,7 @@ static void WI_initShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-static void WI_updateShowNextLoc(void)
+void WI_updateShowNextLoc(void)
 {
   WI_updateAnimatedBack();
 
@@ -993,7 +913,7 @@ static void WI_updateShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-static void WI_drawShowNextLoc(void)
+void WI_drawShowNextLoc(void)
 {
   int   i;
   int   last;
@@ -1001,7 +921,7 @@ static void WI_drawShowNextLoc(void)
   WI_slamBackground();
 
   // draw animated background
-  WI_drawAnimatedBack(); 
+  WI_drawAnimatedBack();
 
   if ( gamemode != commercial)
   {
@@ -1010,7 +930,7 @@ static void WI_drawShowNextLoc(void)
       WI_drawEL();  // "Entering..." if not E1 or E2
       return;
     }
-  
+
     last = (wbs->last == 8) ? wbs->next - 1 : wbs->last;
 
     // draw a splat on taken cities.
@@ -1023,13 +943,13 @@ static void WI_drawShowNextLoc(void)
 
     // draw flashing ptr
     if (snl_pointeron)
-      WI_drawOnLnode(wbs->next, yah); 
+      WI_drawOnLnode(wbs->next, yah);
   }
 
   // draws which level you are entering..
   if ( (gamemode != commercial)
      || wbs->next != 30)  // check for MAP30 end game
-  WI_drawEL();  
+  WI_drawEL();
 }
 
 // ====================================================================
@@ -1038,7 +958,7 @@ static void WI_drawShowNextLoc(void)
 // Args:    none
 // Returns: void
 //
-static void WI_drawNoState(void)
+void WI_drawNoState(void)
 {
   snl_pointeron = true;
   WI_drawShowNextLoc();
@@ -1051,21 +971,21 @@ static void WI_drawNoState(void)
 // Args:    playernum -- the player to be calculated
 // Returns: the total frags for this player
 //
-static int WI_fragSum(int playernum)
+int WI_fragSum(int playernum)
 {
   int   i;
   int   frags = 0;
-    
+
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
     if (playeringame[i]  // is this player playing?
        && i!=playernum) // and it's not the player we're calculating
     {
-      frags += plrs[playernum].frags[i];  
+      frags += plrs[playernum].frags[i];
     }
   }
 
-  
+
   // JDC hack - negative frags.
   frags -= plrs[playernum].frags[playernum];
 
@@ -1079,12 +999,12 @@ static short int   *dm_totals;  // totals by player
 
 // ====================================================================
 // WI_initDeathmatchStats
-// Purpose: Set up to display DM stats at end of level.  Calculate 
+// Purpose: Set up to display DM stats at end of level.  Calculate
 //          frags for all players.
 // Args:    none
 // Returns: void
 //
-static void WI_initDeathmatchStats(void)
+void WI_initDeathmatchStats(void)
 {
   int   i; // looping variables
 
@@ -1101,7 +1021,7 @@ static void WI_initDeathmatchStats(void)
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
     if (playeringame[i])
-    { 
+    {
       // CPhipps - allocate frags line
       dm_frags[i] = calloc(MAXPLAYERS, sizeof(**dm_frags)); // set all counts to zero
 
@@ -1129,21 +1049,21 @@ void WI_endDeathmatchStats(void)
 
 // ====================================================================
 // WI_updateDeathmatchStats
-// Purpose: Advance Deathmatch stats screen animation.  Calculate 
+// Purpose: Advance Deathmatch stats screen animation.  Calculate
 //          frags for all players.  Lots of noise and drama around
 //          the presentation.
 // Args:    none
 // Returns: void
 //
-static void WI_updateDeathmatchStats(void)
+void WI_updateDeathmatchStats(void)
 {
   int   i;
   int   j;
-    
+
   boolean stillticking;
-  
+
   WI_updateAnimatedBack();
-  
+
   if (acceleratestage && dm_state != 4)  // still ticking
   {
     acceleratestage = 0;
@@ -1159,18 +1079,18 @@ static void WI_updateDeathmatchStats(void)
         dm_totals[i] = WI_fragSum(i);
       }
     }
-  
+
 
     S_StartSound(0, sfx_barexp);  // bang
     dm_state = 4;  // we're done with all 4 (or all we have to do)
   }
 
-    
+
   if (dm_state == 2)
   {
     if (!(bcnt&3))
       S_StartSound(0, sfx_pistol);  // noise while counting
-  
+
     stillticking = false;
 
     for (i=0 ; i<MAXPLAYERS ; i++)
@@ -1192,7 +1112,7 @@ static void WI_updateDeathmatchStats(void)
 
             if (dm_frags[i][j] < -999)
               dm_frags[i][j] = -999;
-      
+
             stillticking = true;
           }
         }
@@ -1200,7 +1120,7 @@ static void WI_updateDeathmatchStats(void)
 
         if (dm_totals[i] > 999)
           dm_totals[i] = 999;
-    
+
         if (dm_totals[i] < -999)
           dm_totals[i] = -999;  // Ty 03/17/98 end 3-digit frag count
       }
@@ -1215,7 +1135,7 @@ static void WI_updateDeathmatchStats(void)
   else if (dm_state == 4)
   {
     if (acceleratestage)
-    {   
+    {
       S_StartSound(0, sfx_slop);
 
       if ( gamemode == commercial)
@@ -1250,22 +1170,22 @@ void WI_drawDeathmatchStats(void)
   int   x;
   int   y;
   int   w;
-  
+
   int   lh; // line height
-  int   halfface = R_NamePatchWidth(facebackp)/2;
+  int   halfface = V_NamePatchWidth(facebackp)/2;
 
   lh = WI_SPACINGY;
 
   WI_slamBackground();
-  
+
   // draw animated background
-  WI_drawAnimatedBack(); 
+  WI_drawAnimatedBack();
   WI_drawLF();
 
   // draw stat titles (top line)
-  V_DrawNamePatch(DM_TOTALSX-R_NamePatchWidth(total)/2,
-		 DM_MATRIXY-WI_SPACINGY+10, FB, total, CR_DEFAULT, VPT_STRETCH);
-  
+  V_DrawNamePatch(DM_TOTALSX-V_NamePatchWidth(total)/2,
+     DM_MATRIXY-WI_SPACINGY+10, FB, total, CR_DEFAULT, VPT_STRETCH);
+
   V_DrawNamePatch(DM_KILLERSX, DM_KILLERSY, FB, killers, CR_DEFAULT, VPT_STRETCH);
   V_DrawNamePatch(DM_VICTIMSX, DM_VICTIMSY, FB, victims, CR_DEFAULT, VPT_STRETCH);
 
@@ -1277,19 +1197,19 @@ void WI_drawDeathmatchStats(void)
   {
     if (playeringame[i]) {
       //int trans = playernumtotrans[i];
-      V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY, 
-		     FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
-		     VPT_STRETCH | (i ? VPT_TRANS : 0));
-      V_DrawNamePatch(DM_MATRIXX-halfface, y, 
-		     FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
-		     VPT_STRETCH | (i ? VPT_TRANS : 0));
+      V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY,
+         FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
+         VPT_STRETCH | (i ? VPT_TRANS : 0));
+      V_DrawNamePatch(DM_MATRIXX-halfface, y,
+         FB, facebackp, i ? CR_LIMIT+i : CR_DEFAULT,
+         VPT_STRETCH | (i ? VPT_TRANS : 0));
 
       if (i == me)
       {
-        V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY, 
-		       FB, bstar, CR_DEFAULT, VPT_STRETCH);
-        V_DrawNamePatch(DM_MATRIXX-halfface, y, 
-		       FB, star, CR_DEFAULT, VPT_STRETCH);
+        V_DrawNamePatch(x-halfface, DM_MATRIXY - WI_SPACINGY,
+           FB, bstar, CR_DEFAULT, VPT_STRETCH);
+        V_DrawNamePatch(DM_MATRIXX-halfface, y,
+           FB, star, CR_DEFAULT, VPT_STRETCH);
       }
     }
     x += DM_SPACINGX;
@@ -1350,7 +1270,7 @@ static void WI_endNetgameStats(void)
 // Args:    none
 // Returns: void
 //
-static void WI_initNetgameStats(void)
+void WI_initNetgameStats(void)
 {
   int i;
 
@@ -1383,11 +1303,11 @@ static void WI_initNetgameStats(void)
 // Returns: void
 // Comment: This stuff sure is complicated for what it does
 //
-static void WI_updateNetgameStats(void)
+void WI_updateNetgameStats(void)
 {
   int   i;
   int   fsum;
-    
+
   boolean stillticking;
 
   WI_updateAnimatedBack();
@@ -1405,7 +1325,7 @@ static void WI_updateNetgameStats(void)
       cnt_items[i] = (plrs[i].sitems * 100) / wbs->maxitems;
 
       // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-      cnt_secret[i] = wbs->maxsecret ? 
+      cnt_secret[i] = wbs->maxsecret ?
                       (plrs[i].ssecret * 100) / wbs->maxsecret : 100;
       if (dofrags)
         cnt_frags[i] = WI_fragSum(i);  // we had frags
@@ -1433,10 +1353,10 @@ static void WI_updateNetgameStats(void)
       else
         stillticking = true; // still got stuff to tally
     }
-  
+
     if (!stillticking)
     {
-      S_StartSound(0, sfx_barexp); 
+      S_StartSound(0, sfx_barexp);
       ng_state++;
     }
   }
@@ -1444,21 +1364,21 @@ static void WI_updateNetgameStats(void)
   {
     if (!(bcnt&3))
       S_StartSound(0, sfx_pistol);
-  
+
     stillticking = false;
-  
+
     for (i=0 ; i<MAXPLAYERS ; i++)
     {
       if (!playeringame[i])
         continue;
-  
+
       cnt_items[i] += 2;
       if (cnt_items[i] >= (plrs[i].sitems * 100) / wbs->maxitems)
         cnt_items[i] = (plrs[i].sitems * 100) / wbs->maxitems;
       else
         stillticking = true;
     }
-  
+
     if (!stillticking)
     {
       S_StartSound(0, sfx_barexp);
@@ -1481,12 +1401,12 @@ static void WI_updateNetgameStats(void)
 
       // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
 
-      if (cnt_secret[i] >= (wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : 100))
+      if (cnt_secret[i] >= (wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : compatibility_level < lxdoom_1_compatibility ? 0 : 100))
         cnt_secret[i] = wbs->maxsecret ? (plrs[i].ssecret * 100) / wbs->maxsecret : 100;
       else
         stillticking = true;
     }
-  
+
     if (!stillticking)
     {
       S_StartSound(0, sfx_barexp);
@@ -1512,7 +1432,7 @@ static void WI_updateNetgameStats(void)
       else
         stillticking = true;
     }
-      
+
     if (!stillticking)
     {
       S_StartSound(0, sfx_pldeth);
@@ -1554,32 +1474,32 @@ void WI_drawNetgameStats(void)
   int   i;
   int   x;
   int   y;
-  int   pwidth = R_NamePatchWidth(percent);
-  int   fwidth = R_NamePatchWidth(facebackp);
+  int   pwidth = V_NamePatchWidth(percent);
+  int   fwidth = V_NamePatchWidth(facebackp);
 
   WI_slamBackground();
-  
+
   // draw animated background
-  WI_drawAnimatedBack(); 
+  WI_drawAnimatedBack();
 
   WI_drawLF();
 
   // draw stat titles (top line)
-  V_DrawNamePatch(NG_STATSX+NG_SPACINGX-R_NamePatchWidth(kills),
-		 NG_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch(NG_STATSX+NG_SPACINGX-V_NamePatchWidth(kills),
+     NG_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
 
-  V_DrawNamePatch(NG_STATSX+2*NG_SPACINGX-R_NamePatchWidth(items),
-		 NG_STATSY, FB, items, CR_DEFAULT, VPT_STRETCH);
+  V_DrawNamePatch(NG_STATSX+2*NG_SPACINGX-V_NamePatchWidth(items),
+     NG_STATSY, FB, items, CR_DEFAULT, VPT_STRETCH);
+
+  V_DrawNamePatch(NG_STATSX+3*NG_SPACINGX-V_NamePatchWidth(secret),
+     NG_STATSY, FB, secret, CR_DEFAULT, VPT_STRETCH);
 
-  V_DrawNamePatch(NG_STATSX+3*NG_SPACINGX-R_NamePatchWidth(secret),
-		 NG_STATSY, FB, secret, CR_DEFAULT, VPT_STRETCH);
-  
   if (dofrags)
-    V_DrawNamePatch(NG_STATSX+4*NG_SPACINGX-R_NamePatchWidth(frags),
-		   NG_STATSY, FB, frags, CR_DEFAULT, VPT_STRETCH);
+    V_DrawNamePatch(NG_STATSX+4*NG_SPACINGX-V_NamePatchWidth(frags),
+       NG_STATSY, FB, frags, CR_DEFAULT, VPT_STRETCH);
 
   // draw stats
-  y = NG_STATSY + R_NamePatchHeight(kills);
+  y = NG_STATSY + V_NamePatchHeight(kills);
 
   for (i=0 ; i<MAXPLAYERS ; i++)
   {
@@ -1588,9 +1508,9 @@ void WI_drawNetgameStats(void)
       continue;
 
     x = NG_STATSX;
-    V_DrawNamePatch(x-fwidth, y, FB, facebackp, 
-		   i ? CR_LIMIT+i : CR_DEFAULT,
-		   VPT_STRETCH | (i ? VPT_TRANS : 0));
+    V_DrawNamePatch(x-fwidth, y, FB, facebackp,
+       i ? CR_LIMIT+i : CR_DEFAULT,
+       VPT_STRETCH | (i ? VPT_TRANS : 0));
 
     if (i == me)
       V_DrawNamePatch(x-fwidth, y, FB, star, CR_DEFAULT, VPT_STRETCH);
@@ -1621,14 +1541,14 @@ static int  sp_state;
 // Comment: Seems like we could do all these stats in a more generic
 //          set of routines that weren't duplicated for dm, coop, sp
 //
-static void WI_initStats(void)
+void WI_initStats(void)
 {
   state = StatCount;
   acceleratestage = 0;
   sp_state = 1;
 
   // CPhipps - allocate (awful code, I know, but saves changing it all) and initialise
-  *(cnt_kills = malloc(sizeof(*cnt_kills))) = 
+  *(cnt_kills = malloc(sizeof(*cnt_kills))) =
   *(cnt_items = malloc(sizeof(*cnt_items))) =
   *(cnt_secret= malloc(sizeof(*cnt_secret))) = -1;
   cnt_time = cnt_par = cnt_total_time = -1;
@@ -1643,7 +1563,7 @@ static void WI_initStats(void)
 // Args:    none
 // Returns: void
 //
-static void WI_updateStats(void)
+void WI_updateStats(void)
 {
   WI_updateAnimatedBack();
 
@@ -1654,7 +1574,7 @@ static void WI_updateStats(void)
     cnt_items[0] = (plrs[me].sitems * 100) / wbs->maxitems;
 
     // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-    cnt_secret[0] = (wbs->maxsecret ? 
+    cnt_secret[0] = (wbs->maxsecret ?
       (plrs[me].ssecret * 100) / wbs->maxsecret : 100);
 
     cnt_total_time = wbs->totaltimes / TICRATE;
@@ -1700,10 +1620,11 @@ static void WI_updateStats(void)
       S_StartSound(0, sfx_pistol);
 
     // killough 2/22/98: Make secrets = 100% if maxsecret = 0:
-    if (cnt_secret[0] >= (wbs->maxsecret ? 
+    if ((!wbs->maxsecret && compatibility_level < lxdoom_1_compatibility) ||
+	cnt_secret[0] >= (wbs->maxsecret ?
       (plrs[me].ssecret * 100) / wbs->maxsecret : 100))
     {
-      cnt_secret[0] = (wbs->maxsecret ? 
+      cnt_secret[0] = (wbs->maxsecret ?
         (plrs[me].ssecret * 100) / wbs->maxsecret : 100);
       S_StartSound(0, sfx_barexp);
       sp_state++;
@@ -1770,7 +1691,7 @@ static void WI_updateStats(void)
 void WI_drawStats(void)
 {
   // line height
-  int lh; 
+  int lh;
 
   lh = (3*num[0].height)/2;
 
@@ -1778,7 +1699,7 @@ void WI_drawStats(void)
 
   // draw animated background
   WI_drawAnimatedBack();
-  
+
   WI_drawLF();
 
   V_DrawNamePatch(SP_STATSX, SP_STATSY, FB, kills, CR_DEFAULT, VPT_STRETCH);
@@ -1806,13 +1727,6 @@ void WI_checkForAccelerate(void)
   int   i;
   player_t  *player;
 
-#ifdef COMPILE_VIDD
-  if (sp_state != 10 && VIDD_PLAY_inProgress() || VIDD_MENU_getAdvanceIntermission()) { // POPE
-    acceleratestage = 1;
-    return;
-  }
-#endif
-
   // check for button presses to skip delays
   for (i=0, player = players ; i<MAXPLAYERS ; i++, player++)
   {
@@ -1849,15 +1763,15 @@ void WI_checkForAccelerate(void)
 void WI_Ticker(void)
 {
   // counter for general background animation
-  bcnt++;  
+  bcnt++;
 
   if (bcnt == 1)
   {
     // intermission music
     if ( gamemode == commercial )
-      S_ChangeMusicNum(mus_dm2int, true);
+      S_ChangeMusic(mus_dm2int, true);
     else
-      S_ChangeMusicNum(mus_inter, true); 
+      S_ChangeMusic(mus_inter, true);
   }
 
   WI_checkForAccelerate();
@@ -1869,11 +1783,11 @@ void WI_Ticker(void)
          else if (netgame) WI_updateNetgameStats();
          else WI_updateStats();
          break;
-  
+
     case ShowNextLoc:
          WI_updateShowNextLoc();
          break;
-  
+
     case NoState:
          WI_updateNoState();
          break;
@@ -1887,8 +1801,8 @@ void WI_Ticker(void)
  * Args:    none
  * Returns: void
  *
- * CPhipps - modified for new wad lump handling. 
- *         - no longer preload most graphics, other funcs can use 
+ * CPhipps - modified for new wad lump handling.
+ *         - no longer preload most graphics, other funcs can use
  *           them by name
  */
 
@@ -1909,7 +1823,7 @@ void WI_loadData(void)
         for (i=0;i<a->nanims;i++)
         {
           // MONDO HACK!
-          if (wbs->epsd != 1 || j != 8) 
+          if (wbs->epsd != 1 || j != 8)
           {
             // animations
             sprintf(name, "WIA%d%.2d%.2d", wbs->epsd, j, i);
@@ -1918,7 +1832,7 @@ void WI_loadData(void)
           else
           {
             // HACK ALERT!
-            a->p[i] = anims[1][4].p[i]; 
+            a->p[i] = anims[1][4].p[i];
           }
         }
       }
@@ -1933,42 +1847,6 @@ void WI_loadData(void)
   }
 }
 
-// ====================================================================
-// WI_unloadData
-// Purpose: Free up the space allocated during WI_loadData
-// Args:    none
-// Returns: void
-//
-// CPhipps - reverse of WI_loadData, goes through the same lumps, but unlocking
-void WI_unloadData(void)
-{
-  int   i,j;
-  char  name[9];  // limited to 8 characters
-
-  // cph - unlock gamemode dependent stuff here
-  if (gamemode != commercial) {
-    if (wbs->epsd < 3) {
-      for (j=0;j<NUMANIMS[wbs->epsd];j++) {
-	anim_t* a = &anims[wbs->epsd][j];
-	for (i=0; i<a->nanims; i++) {
-	  // MONDO HACK!
-	  if (wbs->epsd != 1 || j != 8) {
-	    // animations
-	    sprintf(name, "WIA%d%.2d%.2d", wbs->epsd, j, i);  
-	    W_UnlockLumpName(name);
-	  }
-	}
-      }
-    }
-  }
-
-  for (i=0;i<10;i++) {
-    // numbers 0-9
-    sprintf(name, "WINUM%d", i);     
-    W_UnlockLumpName(name);
-  }
-}
-
 
 // ====================================================================
 // WI_Drawer
@@ -1989,11 +1867,11 @@ void WI_Drawer (void)
          else
            WI_drawStats();
          break;
-  
+
     case ShowNextLoc:
          WI_drawShowNextLoc();
          break;
-  
+
     case NoState:
          WI_drawNoState();
          break;
@@ -2008,7 +1886,7 @@ void WI_Drawer (void)
 // Args:    wbstartstruct -- pointer to the structure with the data
 // Returns: void
 //
-static void WI_initVariables(wbstartstruct_t* wbstartstruct)
+void WI_initVariables(wbstartstruct_t* wbstartstruct)
 {
 
   wbs = wbstartstruct;
@@ -2042,12 +1920,6 @@ static void WI_initVariables(wbstartstruct_t* wbstartstruct)
   if (!wbs->maxitems)
     wbs->maxitems = 1;
 
-  // killough 2/22/98: Keep maxsecret=0 if it's zero, so
-  // we can detect 0/0 as as a special case and print 100%.
-  //
-  //    if (!wbs->maxsecret)
-  //  wbs->maxsecret = 1;
-
   if ( gamemode != retail )
     if (wbs->epsd > 2)
       wbs->epsd -= 3;
@@ -2057,14 +1929,12 @@ static void WI_initVariables(wbstartstruct_t* wbstartstruct)
 // WI_Start
 // Purpose: Call the various init routines
 //          Note: wbstartstruct_t is defined in d_player.h
-// Args:    wbstartstruct -- pointer to the structure with the 
+// Args:    wbstartstruct -- pointer to the structure with the
 //          intermission data
 // Returns: void
 //
 void WI_Start(wbstartstruct_t* wbstartstruct)
 {
-  WI_StartCamera();  //set up camera
-
   WI_initVariables(wbstartstruct);
   WI_loadData();
 
