@@ -1,14 +1,16 @@
-/* Emacs style mode select   -*- C++ -*- 
+/* Emacs style mode select   -*- C++ -*-
  *-----------------------------------------------------------------------------
  *
  *
- *  PrBoom a Doom port merged with LxDoom and LSDLDoom
+ *  PrBoom: a Doom port merged with LxDoom and LSDLDoom
  *  based on BOOM, a modified and improved DOOM engine
  *  Copyright (C) 1999 by
  *  id Software, Chi Hoang, Lee Killough, Jim Flynn, Rand Phares, Ty Halderman
  *  Copyright (C) 1999-2000 by
  *  Jess Haas, Nicolas Kalkhof, Colin Phipps, Florian Schulze
- *  
+ *  Copyright 2005, 2006 by
+ *  Florian Schulze, Colin Phipps, Neil Stevens, Andrey Budko
+ *
  *  This program is free software; you can redistribute it and/or
  *  modify it under the terms of the GNU General Public License
  *  as published by the Free Software Foundation; either version 2
@@ -21,7 +23,7 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 
+ *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
  *  02111-1307, USA.
  *
  * DESCRIPTION:
@@ -117,7 +119,7 @@ static int free_memory = 0;
 static int active_memory = 0;
 static int purgable_memory = 0;
 
-void Z_DrawStats(void)            // Print allocation statistics
+static void Z_DrawStats(void)            // Print allocation statistics
 {
   unsigned long total_memory = free_memory + memory_size + active_memory + purgable_memory;
   double s = 100.0 / total_memory;
@@ -650,6 +652,8 @@ char *(Z_Strdup)(const char *s, int tag, void **user
 void (Z_CheckHeap)(
 #ifdef INSTRUMENTED
        const char *file, int line
+#else
+       void
 #endif
        )
 {
